Package: incidence
Type: Package
Title: Compute, Handle, Plot and Model Incidence of Dated Events
Version: 1.7.6
Authors@R: c(
  person("Thibaut", "Jombart", role = c("aut"), email = "thibautjombart@gmail.com"), 
  person("Zhian N.", "Kamvar", role = "aut", email = "zkamvar@gmail.com",
         comment = c(ORCID = "0000-0003-1458-7108")),
  person("Rich", "FitzJohn", role = "aut", email = "rich.fitzjohn@gmail.com"),
  person("Tim", "Taylor", role = "cre", email = "tim.taylor@hiddenelephants.co.uk",
         comment = c(ORCID = "0000-0002-8587-7113")),
  person("Jun", "Cai", role = "ctb", email = "cai-j12@mails.tsinghua.edu.cn", 
         comment = c(ORCID = "0000-0001-9495-1226")), 
  person("Sangeeta", "Bhatia", role = "ctb", email = "sangeetabhatia03@gmail.com"),
  person("Jakob", "Schumacher", role = "ctb"),
  person("Juliet R.C.", "Pulliam", role = "ctb", email = "pulliam@sun.ac.za",
         comment = c(ORCID = "0000-0003-3314-8223"))
  )
Description: Provides functions and classes to compute, handle and visualise
  incidence from dated events for a defined time interval. Dates can be provided
  in various standard formats. The class 'incidence' is used to store computed
  incidence and can be easily manipulated, subsetted, and plotted. In addition,
  log-linear models can be fitted to 'incidence' objects using 'fit'. This
  package is part of the RECON (<https://www.repidemicsconsortium.org/>) toolkit
  for outbreak analysis.
Encoding: UTF-8
License: MIT + file LICENSE
URL: https://www.repidemicsconsortium.org/incidence/
BugReports: https://github.com/reconhub/incidence/issues
RoxygenNote: 7.3.2
Imports: ggplot2 (>= 3.3.2), aweek (>= 0.2.0)
Suggests: magrittr, outbreaks, testthat, vdiffr, knitr, rmarkdown,
        scales, cowplot
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2025-06-19 21:23:53 UTC; tim
Author: Thibaut Jombart [aut],
  Zhian N. Kamvar [aut] (ORCID: <https://orcid.org/0000-0003-1458-7108>),
  Rich FitzJohn [aut],
  Tim Taylor [cre] (ORCID: <https://orcid.org/0000-0002-8587-7113>),
  Jun Cai [ctb] (ORCID: <https://orcid.org/0000-0001-9495-1226>),
  Sangeeta Bhatia [ctb],
  Jakob Schumacher [ctb],
  Juliet R.C. Pulliam [ctb] (ORCID:
    <https://orcid.org/0000-0003-3314-8223>)
Maintainer: Tim Taylor <tim.taylor@hiddenelephants.co.uk>
Repository: CRAN
Date/Publication: 2025-06-19 21:50:02 UTC
Built: R 4.6.0; ; 2026-04-26 06:43:53 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method("[",incidence)
S3method("group_names<-",default)
S3method("group_names<-",incidence)
S3method(as.data.frame,incidence)
S3method(as.incidence,data.frame)
S3method(as.incidence,matrix)
S3method(as.incidence,numeric)
S3method(cumulate,default)
S3method(cumulate,incidence)
S3method(dim,incidence)
S3method(get_counts,incidence)
S3method(get_dates,default)
S3method(get_dates,incidence)
S3method(get_fit,incidence_fit)
S3method(get_fit,incidence_fit_list)
S3method(get_info,incidence_fit)
S3method(get_info,incidence_fit_list)
S3method(get_interval,default)
S3method(get_interval,incidence)
S3method(get_n,default)
S3method(get_n,incidence)
S3method(get_timespan,default)
S3method(get_timespan,incidence)
S3method(group_names,default)
S3method(group_names,incidence)
S3method(incidence,Date)
S3method(incidence,POSIXt)
S3method(incidence,character)
S3method(incidence,default)
S3method(incidence,integer)
S3method(incidence,numeric)
S3method(plot,incidence)
S3method(plot,incidence_fit)
S3method(plot,incidence_fit_list)
S3method(print,incidence)
S3method(print,incidence_fit)
S3method(print,incidence_fit_list)
S3method(subset,incidence)
export("group_names<-")
export(add_incidence_fit)
export(as.incidence)
export(bootstrap)
export(cumulate)
export(estimate_peak)
export(find_peak)
export(fit)
export(fit_optim_split)
export(get_counts)
export(get_dates)
export(get_fit)
export(get_info)
export(get_interval)
export(get_n)
export(get_timespan)
export(group_names)
export(incidence)
export(incidence_pal1)
export(incidence_pal1_dark)
export(incidence_pal1_light)
export(make_breaks)
export(pool)
export(scale_x_incidence)
importFrom(grDevices,colorRampPalette)
importFrom(graphics,plot)
importFrom(stats,as.ts)
importFrom(utils,head)
importFrom(utils,tail)

#' Example 1 ------------------------------------------------------------------
#'
#' **computing and manipulating stratified weekly incidence**
#'
#' 1) import data
#'
library('outbreaks')

dat1 <- ebola_sim_clean$linelist
str(dat1, strict.width = "cut", width = 76)

#' 2) build an incidence object
#'
#+ incidence-curve, fig.width=9, fig.height=5
library('incidence')
library('ggplot2')

# compute weekly stratified incidence
i.7.group <- incidence(dat1$date_of_onset, interval = 7, groups = dat1$hospital)
# print incidence object
i.7.group

# plot incidence object
my_theme <- theme_bw(base_size = 12) +
  theme(panel.grid.minor = element_blank()) +
  theme(axis.text.x = element_text(angle = 90, hjust = 1, vjust = 0.5, color = "black"))

plot(i.7.group, border = "white") + 
  my_theme + 
  theme(legend.position = c(0.8, 0.75))

#' 3) Manipulate incidence object
#'
#+ incidence-early-curve, fig.width=6, fig.height=6
# plot the first 18 weeks, defined hospitals, and use different colors
i.7.sub <- i.7.group[1:18, c(1:2, 4:5)]
hosp_colors <- c("#899DA4", "#C93312", "#FAEFD1", "#DC863B")
plot(i.7.sub, show_cases = TRUE, border = "black", color = hosp_colors) + 
  my_theme +
  theme(legend.position = c(0.35, 0.8)) 
# exclude NA group by disabling treating NA as a separate group
i.7.group0 <- incidence(dat1$date_of_onset,
                        interval    = 7,
                        groups      = dat1$hospital,
                        na_as_group = FALSE)

# exclude NA using [ operator
i.7.group1 <- subset(i.7.group, groups = -ncol(i.7.group))

# exclude NA group using [ operator
i.7.group2 <- i.7.group[, -ncol(i.7.group)]

# check the resulting incidence objects are identical
identical(i.7.group0$counts, i.7.group1$counts)
identical(i.7.group1, i.7.group2)

# check groups
colnames(i.7.group1$counts)

#' Example 2 ------------------------------------------------------------------
#'
#' **importing pre-computed daily incidence and fitting log-linear model**
#'
#' 1) Import pre-computed daily incidence
#'
#+ incidence-curve2, fig.width=9, fig.height=6
# preview datasets
head(zika_girardot_2015, 3)
head(zika_sanandres_2015, 3)

# combine two datasets into one
dat2 <- merge(zika_girardot_2015, zika_sanandres_2015, by = "date", all = TRUE)

# rename variables
names(dat2)[2:3] <- c("Girardot", "San Andres")

# replace NA with 0
dat2[is.na(dat2)] <- 0

# convert pre-computed incidence in data.frame into incidence object
# grouped by locations
i.group <- as.incidence(x = dat2[, 2:3], dates = dat2$date)

# pool incidence across two locations
i.pooled <- pool(i.group)
cowplot::plot_grid(
	plot(i.group, border = "white")  + my_theme + theme(legend.position = c(0.9, 0.7)),
	plot(i.pooled, border = "white") + my_theme,
	ncol       = 1,
	labels     = c("(A)", "(B)"),
	label_size = 16,
	label_x    = 0.06,
	label_y    = 0.94
)

#' 2) Fit log-linear regression model
#'
#+ incidence-fit, fig.width=9, fig.height=4
library('magrittr')

fos <- fit_optim_split(i.pooled)
fos$split
fos$fit
plot(i.pooled, border = "white") %>%
  add_incidence_fit(fos$fit) +
  my_theme

#' Enforce cromulence of groups
#'
#' This enforces that groups is either:
#'
#'   - NULL
#'   - a factor and the same length as dates
#' It also treats missing groups (NA) as a separate group is needed.
#' 
#' @param x a vector denoting groups
#' @param dates a vector representing dates
#' @param na_as_group a logical indicating whether or not NA should be
#'   considered a separate group.
#' @noRd
check_groups <- function(x, dates, na_as_group){
  if (is.null(x)) {
    return(NULL)
  }
  x   <- factor(x)
  lev <- levels(x)
  if (na_as_group && any(is.na(x))) {
    x <- as.character(x)
    x[is.na(x)] <- "NA"
    lev <- c(lev, "NA")
  }
  if (length(x) != length(dates)) {
    stop(sprintf(
                 "'x' does not have the same length as dates (%d vs %d)",
                 length(x),
                 length(dates)
                 )
        )
  }
  factor(x, levels = lev)
}

context("Non-exported functions")

test_that("check_dates works", {
  msg <- "NA detected in the dates"
  expect_error(check_dates(c(1,2,NA), TRUE), msg)

  msg <- paste0("Flooring from non-integer date caused approximations:\n",
                "Mean relative difference: 0.1")
  expect_warning(check_dates(1.1), msg)

  msg <- paste0("Input could not be converted to date. Accepted formats are:\n",
                "Date, POSIXct, integer, numeric")
  expect_error(check_dates(factor("2001-01-01")), msg)

  x <- list(1L,
            as.POSIXct("2001-01-01"),
            as.Date("2001-01-01") + 1:10,
            1.0,
            100:1)
  for (e in x) {
    expect_equal(e, check_dates(e))
  }
})

test_that("check_interval", {
  skip_on_cran()

  expect_error(check_interval(),
               "Interval is missing or NULL")
  expect_error(check_interval(NULL),
               "Interval is missing or NULL")
  expect_error(check_interval(1:2),
               "Exactly one value should be provided as interval \\(2 provided\\)")
  expect_error(check_interval(integer(0)),
               "Exactly one value should be provided as interval \\(0 provided\\)")
  expect_error(check_interval(NA),
               "Interval is not finite")
  expect_error(check_interval(-Inf),
               "Interval is not finite")
  expect_error(check_interval(.1),
               "Interval must be at least 1 \\(input: 0.100; after rounding: 0\\)")
  expect_equal(check_interval(1), 1)
  expect_equal(check_interval(2.4), 2)
  expect_equal(check_interval(2.7), 3)
})

test_that("check_groups", {
  skip_on_cran()

  expect_is(check_groups(1, 1, FALSE), "factor")
  expect_error(check_groups(1, 1:2, FALSE),
               "'x' does not have the same length as dates \\(1 vs 2\\)")
  expect_equal(check_groups(NULL, NULL, FALSE), NULL)
  expect_equal(check_groups(c(1, 1, 2, NA, 2), 1:5, na_as_group = FALSE),
               factor(c(1, 1, 2, NA, 2)))
  expect_equal(check_groups(c(1,1,2,NA,2), 1:5, na_as_group = TRUE),
               factor(c(1, 1, 2, "NA", 2)))
})

#' Retrieve dates from an incidence object
#'
#' @param x an [incidence] object
#' @param ... Unused
#'
#' @return a vector of dates or numerics
#' @export
#'
#' @examples
#'
#' set.seed(999)
#' dat <- as.Date(Sys.Date()) + sample(-3:50, 100, replace = TRUE)
#' x <- incidence(dat, interval = "month")
#' get_dates(x)
#' get_dates(x, position = "middle")
get_dates <- function(x, ...) {
  UseMethod("get_dates")
}

#' @rdname get_dates
#' @export
#' @aliases get_dates.default
get_dates.default <- function(x, ...) {
    stop(sprintf("Not implemented for class %s",
                 paste(class(x), collapse = ", ")))
}

#' @param position One of "left", "center", "middle", or "right" specifying what
#'   side of the bin the date should be drawn from.
#' @param count_days If `TRUE`, the result will be represented as the number of
#'   days from the first date.
#'
#' @rdname get_dates
#' @keywords accessors
#' @export
#'
#' @examples
#' set.seed(999)
#' dat <- as.Date(Sys.Date()) + sample(-3:50, 100, replace = TRUE)
#' x <- incidence(dat, interval = "month")
#' get_dates(x)
#' get_dates(x, "center")
#' get_dates(x, "right")
#'
#' # Return dates by number of days from the first date
#' get_dates(x, count_days = TRUE)
#' get_dates(incidence(-5:5), count_days = TRUE)
get_dates.incidence <- function(x, position = "left", count_days = FALSE, ...) {
  POSITION <- match.arg(position, c("left", "center", "middle", "right"))
  if (!count_days && POSITION == "left") return(x$dates)
  # Default: left side of bins
  first_date <- min(x$dates)
  res        <- x$dates - first_date
  if (POSITION %in% c("center", "middle", "right")) {
    divisor <- if (POSITION == "right") 1L else 2L
    res     <- res + get_interval(x, integer = TRUE)/divisor
  }
  # This part is necessary to avoid the Date class rounding the result -_-
  res <- as.numeric(res)
  if (count_days) {
    return(res)
  } else {
    return(first_date + res)
  }
}

context("incidence object accessor tests")

set.seed(999)
int   <- sample(-3:50, 100, replace = TRUE)
dat   <- as.Date("2018-01-31") + int
grp   <- sample(letters[1:3], length(dat), replace = TRUE)
xg    <- incidence(dat, groups = grp)
x.1   <- incidence(int)
x.7   <- incidence(int, 7L)
x.day <- incidence(dat, "day")
x.wee <- incidence(dat, "week")
x.mon <- incidence(dat, "month")
x.mon2yr <- incidence(c(as.Date("2017-12-15"), dat), "month")
x.yer <- incidence(c(dat, dat - 365, dat - 365 * 2), "year")

test_that("get_interval works for integers", {
  expect_equal(get_interval(x.1), 1L)
  expect_equal(get_interval(x.1, integer = FALSE), 1L)
})

test_that("get_timespan works", {
  expect_equal(get_timespan(x.1), x.1$timespan)
  expect_equal(get_timespan(x.mon), x.mon$timespan)
  expect_error(get_timespan("coffee"), "Not implemented for class character")
})

test_that("get_n works", {
  expect_equal(get_n(x.1), x.1$n)
  expect_equal(get_n(x.7), x.7$n)
  expect_equal(get_n(x.mon), x.mon$n)
  expect_error(get_n("coffee"), "Not implemented for class character")
})

test_that("group_names works", {
  expect_identical(group_names(xg), letters[1:3])
  expect_null(group_names(x.1))
  group_names(xg) <- c("foo", "bar", "baz")
  expect_identical(group_names(xg), c("foo", "bar", "baz"))
  expect_error(group_names(letters), "Not implemented for class character")
  expect_error(group_names(letters) <- 1:10)
})

test_that("group_names can collapse groups", {
  xg2 <- group_names(xg, rep("a", 3))
  xg3 <- group_names(xg, c("a", "b", "b"))
  expect_error(group_names(xg, letters[1:4]), "value must be the same length as the number of groups")
  expect_error(group_names(xg, c(letters[1:2], NA)), "value must be able to be coerced to a character vector")
  expect_equal(ncol(xg3), 2L)
  expect_equal(sum(get_counts(xg3)), sum(get_counts(xg)))
  expect_equal(colSums(get_counts(xg3))[["b"]], sum(colSums(get_counts(xg))[c("b", "c")]))
  expect_equivalent(xg2, pool(xg))
})

test_that("ncol works", {
  expect_equal(ncol(xg), 3L)
  expect_equal(ncol(x.1), 1L)
  expect_equal(ncol(subset(xg, groups = 1:2)), 2L)
})

test_that("get_dates works for integers", {
  expect_equal(get_dates(x.1), x.1$dates)
  expect_equal(get_dates(x.1, count_days = TRUE), seq_along(x.1$dates) - 1.0)
  expect_equal(get_dates(x.1, "center"), x.1$dates + 0.5)
  expect_equal(get_dates(x.1, "right"),  x.1$dates + 1)
})

test_that("get_dates borks correctly", {
  expect_error(get_dates("grind", "Not implemented for class character"))
})

test_that("get_interval works for integer weeks", {
  expect_equal(get_interval(x.7), 7L)
  expect_equal(get_interval(x.7, integer = FALSE), 7L)
  expect_error(get_interval("pizza"), "Not implemented for class character")
})

test_that("get_dates works for integer weeks", {
  expect_equal(get_dates(x.7), x.7$dates)
  expect_equal(get_dates(x.7, count_days = TRUE), 7 * (seq_along(x.7$dates) - 1.0))
  expect_equal(get_dates(x.7, "center"), x.7$dates + 3.5)
  expect_equal(get_dates(x.7, "right"),  x.7$dates + 7)
})


test_that("get_interval works for character days", {
  expect_equal(get_interval(x.day), 1L)
  expect_equal(get_interval(x.day, integer = FALSE), "day")
})

test_that("get_dates works for character days", {
  expect_equal(get_dates(x.day), x.day$dates)
  expect_equal(get_dates(x.day, count_days = TRUE), (seq_along(x.day$dates) - 1.0))
  expect_equal(get_dates(x.day, "center"), x.day$dates + 0.5)
  expect_equal(get_dates(x.day, "right"),  x.day$dates + 1.0)
})

test_that("get_interval works for character weeks", {
  expect_equal(get_interval(x.wee), 7L)
  expect_equal(get_interval(x.wee, integer = FALSE), "week")
})

test_that("get_dates works for character weeks", {
  expect_equal(get_dates(x.wee), x.wee$dates)
  expect_equal(get_dates(x.wee, count_days = TRUE), 7 * (seq_along(x.wee$dates) - 1.0))
  expect_equal(get_dates(x.wee, "center"), x.wee$dates + 3.5)
  expect_equal(get_dates(x.wee, "right"),  x.wee$dates + 7.0)
})


test_that("get_interval works for character months", {
  expect_equal(get_interval(x.mon), c(31, 28, 31))
  expect_equal(get_interval(x.mon, integer = FALSE), "month")
  expect_equal(get_interval(x.mon2yr), c(31, 31, 28, 31))
})

test_that("get_interval works for character years", {
  expect_equal(get_interval(x.yer), c(366, 365, 365))
  expect_equal(get_interval(x.yer, integer = FALSE), "year")
})

test_that("get_dates works for character months", {
  expect_equal(get_dates(x.mon), x.mon$dates)
  expect_equal(get_dates(x.mon, count_days = TRUE), c(0, 31, 59))
  expect_equal(get_dates(x.mon, "center"), x.mon$dates + c(31, 28, 31) / 2)
  expect_equal(get_dates(x.mon, "right"),  x.mon$dates + c(31, 28, 31))
})

test_that("errors happen", {
  xx          <- x.1
  xx$interval <- factor("what")
  expect_error(get_interval(xx), "factor")
})




# Data for the get_info and get_fit accessors
set.seed(1)
dat2 <- sample(1:50, 200, replace = TRUE, prob = 1 + exp(1:50 * 0.1))
sex <- sample(c("female", "male"), 200, replace = TRUE)
i.sex.o <- incidence(c(dat2, abs(dat2 - 45) + 45), 7L, groups = c(sex, rev(sex)))

test_that("get_counts works with and without groups", {
  expect_is(get_counts(i.sex.o), "matrix")
  expect_identical(get_counts(i.sex.o), i.sex.o$counts)
  expect_identical(get_counts(i.sex.o, "female"), get_counts(i.sex.o, 1))
  expect_message(get_counts(i.sex.o, c("female", "nb")),
		 "The following groups were not recognised: nb")
  expect_error(suppressMessages(get_counts(i.sex.o, "what")),
	       "No groups matched those present in the data: female, male")
})

context("incidence_fit* object accessor tests")


i.fitlist <- fit_optim_split(i.sex.o)
fits <- get_fit(i.fitlist$fit)

# Creating an `incidence_fit_list` object with no groups column
fits_list <- fits
for (i in names(fits)) {
  fits_list[[i]]$info$pred$groups <- NULL  
}
class(fits_list) <- "incidence_fit_list"
attr(fits_list, "locations") <- as.list(names(fits))

test_that("fit_optim_split() returns an incidence_fit_list", {
  expect_is(i.fitlist$fit, "incidence_fit_list")
})

test_that("get_fit() returns a list of incidence fit objects", {
  for (i in names(fits)) {
    expect_is(fits[[i]], "incidence_fit", label = i)
  }
  expect_identical(fits[[1]], get_fit(fits[[1]]))
})

test_that("get_info() will return a vector for r", {
  rvec <- get_info(i.fitlist$fit, "r")
  expect_length(rvec, 4) 
}) 

test_that("get_info() will return a vector for doubling/halving", {
  dvec    <- get_info(i.fitlist$fit, "doubling")
  dvec.na <- get_info(i.fitlist$fit, "doubling", na.rm = FALSE)
  expect_length(dvec, 2)
  expect_length(dvec.na, 4)
  expect_identical(dvec, dvec.na[1:2])

  hvec    <- get_info(i.fitlist$fit, "halving")
  hvec.na <- get_info(i.fitlist$fit, "halving", na.rm = FALSE)
  expect_length(hvec, 2)
  expect_length(hvec.na, 4)
  expect_identical(hvec, hvec.na[3:4])
})

test_that("get_info() will return a data frame for pred", {
  # Should have groups be female and male
  pred.g  <- get_info(i.fitlist$fit, "pred")
  # Should have no groups
  pred.ng <- get_info(fits_list, "pred", groups = NULL)
  # Should have groups be female and male
  pred.g1 <- get_info(fits_list, "pred", groups = 1)
  # Should have groups be before and after
  pred.g2 <- get_info(i.fitlist$fit, "pred", groups = 2)
  
  expect_null(pred.ng$groups)
  expect_identical(pred.g$groups, pred.g1$groups)
  expect_identical(levels(pred.g2$groups), c("before", "after"))
})

test_that("get_info() will return matrices for *.conf", {
  hconf    <- get_info(i.fitlist$fit, "halving.conf")
  hconf.na <- get_info(i.fitlist$fit, "halving.conf", na.rm = FALSE)
  expect_is(hconf, "matrix")
  expect_is(hconf.na, "matrix")
  expect_length(hconf, 4)
  expect_identical(hconf, hconf.na[-(1:2), ])
})

context("Incidence main function")

# setting up the data --------------------------------------------------
the_seed <- eval(parse(text = as.character(Sys.Date())))

# Date incidence      --------------------------------------------------
# note: the choice of dates here makes sure first date is 28 Dec 2015, which
# starts an iso week, so that counts will be comparable with/without iso.
# This also ensures that the last date is 2016-04-04 so that there are 15 weeks
# represented here. 
set.seed(the_seed)
dat       <- as.integer(c(-3, sample(-3:100, 49, replace = TRUE), 100))
dat_dates <- as.Date("2015-12-31") + dat

test_that("construction - default, integer input", {


  ## USING DAILY INCIDENCE
  x <- incidence(dat)

  ## classes
  expect_is(x, "incidence")
  expect_is(x$dates, class(dat))
  expect_is(x$counts, "matrix")

  ## dimensions
  expect_equal(nrow(x$counts), length(x$dates))

  ## results
  expect_false(any(is.na(x$counts)))
  expect_equal(length(x$dates), diff(range(dat)) + 1)
  expect_equal(sum(x$counts), length(dat))
  expect_equal(sum(x$counts), x$n)
  expect_true(all(diff(x$dates) == x$interval))

  ## USING INCIDENCE PER 3 DAYS
  x <- incidence(dat, 3)

  ## String numbers can be interpreted as intervals
  expect_identical(x, incidence(dat, "3"))

  ## classes
  expect_is(x, "incidence")
  expect_is(x$dates, class(dat))
  expect_is(x$counts, "matrix")

  ## dimensions
  expect_equal(nrow(x$counts), length(x$dates))

  ## results
  expect_false(any(is.na(x$counts)))
  expect_equal(sum(x$counts), length(dat))
  expect_equal(sum(x$counts), x$n)
  expect_true(all(diff(x$dates) == x$interval))
})

test_that("construction - ISO week", {


  ## USING WEEKLY INCIDENCE
  inc.week    <- incidence(dat_dates, interval = 7, standard = FALSE)
  inc.isoweek <- incidence(dat_dates, interval = 7)

  ## classes
  expect_is(inc.week, "incidence")
  expect_is(inc.isoweek, "incidence")

  ## dimensions
  expect_equal(setdiff(names(inc.isoweek), names(inc.week)), c("weeks", "isoweeks"))
  expect_equal(length(inc.isoweek$isoweeks), length(inc.isoweek$dates))
  expect_equal(nrow(inc.isoweek$counts), length(inc.isoweek$dates))

  ## results
  expect_false(any(is.na(inc.isoweek$counts)))
  expect_equal(sum(inc.isoweek$counts), length(dat))
  expect_equal(sum(inc.isoweek$counts), inc.isoweek$n)
  expect_true(all(diff(inc.isoweek$dates) == inc.isoweek$interval))
})

test_that("construction - numeric input", {

  
  ## USING DAILY INCIDENCE
  dat_int <- c(0L, 2L, 5L, 9L, -1L, 9L, 10L, 6L, 5L, -3L, -1L, -1L, 6L, 2L, 7L,
               3L, 7L, 10L, 2L, 7L, 10L, -1L, 6L, -2L, 0L, 2L, -3L, 2L, 9L, 1L,
               3L, 5L, 3L, -1L, 8L, 6L, 8L, -2L, 7L, 2L, 8L, 6L, 7L, 4L, 4L,
               8L, -3L, 3L, 7L, 6L, 3L, 9L, 3L, 0L, -3L, -2L, 1L, 4L, 6L, 2L,
               9L, 1L, 3L, 1L, 6L, 0L, 3L, 7L, -2L, 9L, 1L, 8L, 1L, 1L, 3L, 9L,
               9L, 2L, 7L, 10L, 3L, 6L, 2L, 1L, 7L, -1L, 6L, -2L, 0L, -1L, 0L,
               -3L, 5L, 9L, 7L, 8L, 3L, 2L, 8L, 5L)

  dat_num <- dat_int + 0.1

  msg <- paste0("Flooring from non-integer date caused approximations:\n",
                "Mean relative difference: 0.0228833")
  expect_warning(incidence(dat_num),
                 msg)

  x_num <- suppressWarnings(incidence(dat_num))
  x_int <- incidence(dat_int)

  ## compare outputs
  expect_equal(x_num, x_int)
  expect_is(x_num$dates, "numeric")
  expect_is(x_int$dates, "integer")
})

test_that("construction - Date input", {


  x         <- incidence(dat)
  x.dates   <- incidence(dat_dates)
  expect_message(x.i.trim  <- incidence(dat, first_date = 0),
                 "[0-9]+ observations outside of \\[0, [0-9]+\\] were removed."
  )
  expect_warning({
  expect_message({
    x.d.trim  <- incidence(dat_dates, first_date = "2016-01-01")
  }, "[0-9]+ observations outside of \\[2016-01-01, [-0-9]{10}\\] were removed.")
  }, "options\\(incidence.warn.first_date = FALSE\\)")
  expect_message({
  expect_failure(expect_warning({
    x.d.trim  <- incidence(dat_dates, first_date = "2016-01-01")
  }, "options\\(incidence.warn.first_date = FALSE\\)"))
  }, "[0-9]+ observations outside of \\[2016-01-01, [-0-9]{10}\\] were removed.")
  x.7       <- incidence(dat_dates, 7L, standard = FALSE)
  x.7.iso   <- incidence(dat_dates, "week")
  x.7.week  <- incidence(dat_dates, "week", standard = FALSE)
  expect_warning(x.7.week2  <- incidence(dat_dates, "week", iso_week = FALSE),
                 "`iso_week` has been deprecated")
  # iso_week can reset standard, but is given a warning
  expect_identical(x.7.week2, x.7.week)

  ## Here, we can test if starting on a different day gives us expected results
  x.ds       <- incidence(dat_dates + 1L)
  x.7.ds     <- incidence(dat_dates + 1L, 7L, standard = FALSE)
  x.w.ds     <- incidence(dat_dates + 1L, "week", standard = FALSE)
  x.7.ds.iso <- incidence(dat_dates + 1L, 7L)
  x.w.ds.iso <- incidence(dat_dates + 1L, "week")

  ## Testing monthly input
  w <- "The first_date \\(2015-11-30\\) represents a day that does not occur in all months."
  w <- gsub(" ", "\\\\s", w)
  expect_warning(x.mo.no <- incidence(dat_dates - 28, "month", standard = FALSE), w)

  x.mo.iso <- incidence(dat_dates, "month")
  expect_equal(format(x.mo.iso$dates, "%m"), unique(format(sort(dat_dates), "%m")))
  expect_equal(format(x.mo.iso$dates, "%d"), rep("01", 5)) # all starts on first
  expect_equal(x.mo.iso$dates[[1]], as.Date("2015-12-01"))
  expect_equal(sum(x.mo.iso$counts), 51L)

  x.mo <- incidence(dat_dates, "month", standard = FALSE)
  expect_equal(format(x.mo$dates, "%m"), unique(format(sort(dat_dates), "%m"))[-5])
  expect_equal(format(x.mo$dates, "%d"), rep("28", 4)) # all starts on the 28th
  expect_equal(x.mo$dates[[1]], as.Date("2015-12-28"))
  expect_equal(sum(x.mo$counts), 51L)

  ## Testing quarterly input
  w <- "The first_date \\(2015-11-30\\) represents a day that does not occur in all months."
  w <- gsub(" ", "\\\\s", w)
  expect_warning(x.qu.no <- incidence(dat_dates - 28, "quarter", standard = FALSE), w)

  x.qu.iso <- incidence(dat_dates, "quarter")
  expect_equal(x.qu.iso$dates, as.Date(c("2015-10-01", "2016-01-01", "2016-04-01")))
  expect_equal(sum(x.qu.iso$counts), 51L)

  x.qu     <- incidence(dat_dates, "quarter", standard = FALSE)
  expect_equal(x.qu$dates, as.Date(c("2015-12-28", "2016-03-28")))
  expect_equal(sum(x.qu$counts), 51L)

  ## Testing yearly input
  dat.yr <- c(dat_dates,
              sample(dat_dates + 366, replace = TRUE),
              sample(dat_dates + 366 + 365, replace = TRUE)
  )
  x.yr.iso <- incidence(dat.yr, "year")
  x.yr     <- incidence(dat.yr, "year", standard = FALSE)
  w <- "The first_date \\(2016-02-29\\) represents a day that does not occur in all years."
  w <- gsub(" ", "\\\\s", w)
  expect_warning(x.yr.no  <- incidence(dat.yr, "year", first_date = as.Date("2016-02-29"), standard = FALSE), w 
  )
  expect_equal(get_dates(x.yr.iso), as.Date(c("2015-01-01", "2016-01-01", "2017-01-01", "2018-01-01")))
  expect_equal(get_dates(x.yr), as.Date(c("2015-12-28", "2016-12-28", "2017-12-28")))
  expect_equal(sum(x.yr$counts), sum(x.yr.iso$counts))

  ## compare outputs
  expect_equal(x$counts, x.dates$counts)
  expect_is(x$dates, "integer")
  expect_is(x.dates$dates, "Date")
  expect_equal(x.7$counts, x.7.iso$counts)
  expect_equal(x.7.iso$dates, x.7.week$dates)

  # shifting days gives the desired effect
  expect_equal(x.ds$dates[[1]], x.7.ds$dates[[1]])
  expect_equal(x.ds$dates[[1]] - 1L, x.7.ds.iso$dates[[1]])
  expect_identical(x.7.ds.iso$dates, x.w.ds.iso$dates)
  expect_failure({
    expect_identical(x.w.ds$dates, x.w.ds.iso$dates)
  })

  ## Printing will be different with text-based interval
  expect_output(print(x.7), "\\$interval: 7 days")
  expect_output(print(x.7.iso), "\\$interval: 1 week")
})

test_that("construction - POSIXct input", {


  ## USING DAILY INCIDENCE
  dat.pos <- as.POSIXct(dat_dates)
  x.dates <- incidence(dat_dates)
  x.pos <- incidence(dat.pos)

  ## compare outputs
  expect_equal(x.dates$counts, x.pos$counts)
  expect_is(x.dates$dates, "Date")
  expect_is(x.pos$dates, "POSIXct")
})

test_that("construction - character input", {
  dats <- Sys.Date() + sample(-100:100, 5)
  datc <- as.character(dats)

  i.date <- incidence(dats)
  i.char <- incidence(datc)
  i.chaw <- incidence(paste(datc, "   "))
  expect_message(i.cham <- incidence(c(datc, NA, NA)), "2 missing observations were removed.")
  expect_is(i.date, "incidence")
  expect_identical(i.date, i.char)  
  expect_identical(i.date, i.chaw)  
  expect_identical(i.date, i.cham)  
})


test_that("corner cases", {


  expect_error(incidence(integer(0)),
               "At least one \\(non-NA\\) date must be provided")

  expect_error(incidence(numeric(0)),
               "At least one \\(non-NA\\) date must be provided")

  expect_error(incidence(NA),
               "At least one \\(non-NA\\) date must be provided")

  expect_error(incidence(NULL),
               "dates is NULL")

  expect_error(incidence(Inf),
               "At least one \\(non-NA\\) date must be provided")

  expect_error(incidence(1, "grind"),
               "The interval 'grind' is not valid. Please supply an integer.")

  expect_error(incidence(as.Date(Sys.Date()), last_date = "core"),
               "last_date \\(core\\) could not be converted to Date. Dates must be in ISO 8601 standard format \\(yyyy-mm-dd\\)")

  expect_error(incidence(1, "week"),
               "The interval 'week' can only be used for Dates")

  expect_error(incidence(as.Date(Sys.Date()), standard = "TRUE"),
               "The argument `standard` must be either `TRUE` or `FALSE`")

  expect_error(incidence(sample(10), intrval = 2),
               "intrval : interval")

  expect_error(incidence(1, were = "wolf"), "were")


  expect_warning(incidence(c(dat_dates, as.Date("1900-01-01"))),
                 "greater than 18262 days \\[1900-01-01 to"
  )
  
  msg <- 'Not all dates are in ISO 8601 standard format \\(yyyy-mm-dd\\). The first incorrect date is'
  expect_error(incidence('daldkadl'), paste(msg, "daldkadl"))
  
  dats <- as.character(Sys.Date() + sample(-10:10, 5))
  dats[3] <- "1Q84-04-15" 
  expect_error(incidence(dats), paste(msg, "1Q84-04-15"))

  dats[3] <- "2018-69-11"
  expect_error(incidence(dats), paste(msg, "2018-69-11"))

  dats[3] <- "01-01-11"
  expect_error(incidence(dats), paste(msg, "01-01-11"))

  dats[3] <- "01-Apr-11"
  expect_error(incidence(dats), paste(msg, "01-Apr-11"))

  msg <- paste0("Input could not be converted to date. Accepted formats are:\n",
                "Date, POSIXct, integer, numeric, character")
  expect_error(incidence(factor("2001-01-01")), msg)
})

test_that("incidence constructor can handle missing data", {
  miss_dat <- dat
  miss_dat[5] <- NA
  expect_message(incidence(miss_dat), "1 missing observations were removed.")
})

test_that("incidence constructor can handle data out of range with groups", {
  set.seed(the_seed)
  g <- sample(letters[1:2], length(dat), replace = TRUE)
  expect_message(incidence(dat, first_date = 0, groups = g),
                 "[0-9]+ observations outside of \\[0, [0-9]+\\] were removed."
  )
})

test_that("Expected values, no group", {


  expect_true(all(incidence(1:10)$counts == 1L))
  expect_true(all(incidence(sample(1:10))$counts == 1L))

  # set.seed(1)
  res1 <- incidence(c(3,2,-1,1,1))
  res2 <- incidence(c(0,0,0))
  # res3 <- incidence(sample(1:80, 1000, replace = TRUE))
  # res4 <- incidence(as.Date("1984-01-01") + sample(1:100, 200, replace = TRUE))
  res5 <- incidence(c(3,2,-1,1,1), 2L)
  res6 <- incidence(c(0,0,0), 3L)
  # res7 <- incidence(sample(1:80, 1000, replace = TRUE), 4L)
  # res8 <- incidence(as.Date("1984-01-01") + sample(1:100, 200, replace = TRUE), 12L)

  expect_equal_to_reference(res1, file = "rds/incidence.res1.rds")
  expect_equal_to_reference(res2, file = "rds/incidence.res2.rds")
  # expect_equal_to_reference(res3, file = "rds/incidence.res3.rds")
  # expect_equal_to_reference(res4, file = "rds/incidence.res4.rds")
  expect_equal_to_reference(res5, file = "rds/incidence.res5.rds")
  expect_equal_to_reference(res6, file = "rds/incidence.res6.rds")
  # expect_equal_to_reference(res7, file = "rds/incidence.res7.rds")
  # expect_equal_to_reference(res8, file = "rds/incidence.res8.rds")
})

test_that("Expected values, with groups", {


  dat <- list(
    as.integer(c(3,2,-1,1,1)),
    as.integer(c(0,0,0)),
    as.integer(c(0,1,2,2,3,5,7))
  )

  fac <- list(
    factor(c(1,1,2,2,2)),
    factor(c('a','b','a')),
    factor(c(1, 2, 3, 3, 3, 3, 1))
  )

  res.g.1 <- incidence(dat[[1]], groups = fac[[1]])
  res.g.2 <- incidence(dat[[2]], groups = fac[[2]])
  res.g.3 <- incidence(dat[[3]], groups = fac[[3]])

  expect_equal_to_reference(res.g.1, file = "rds/res.g.1.rds")
  expect_equal_to_reference(res.g.2, file = "rds/res.g.2.rds")
  expect_equal_to_reference(res.g.3, file = "rds/res.g.3.rds")
})

test_that("user-defined group levels are preserved", {
  g <- sample(LETTERS[1:5], 100, replace = TRUE)
  g <- factor(g, levels = LETTERS[5:1])
  i <- incidence(rpois(100, 10), groups = g)
  expect_identical(group_names(i), levels(g))
  i.df <- as.data.frame(i, long = TRUE)
  expect_identical(levels(i.df$groups), levels(g))
})

test_that("Printing returns the object", {


  x <- incidence("2001-01-01")
  y <- incidence(1:2, groups = factor(1:2))
  z <- incidence(dat_dates, interval = 7)
  expect_equal_to_reference(capture.output(print(x)),
                            file = "rds/print1.rds")
  expect_equal_to_reference(capture.output(print(y)),
                            file = "rds/print2.rds")
  expect_equal_to_reference(capture.output(print(z)),
                            file = "rds/print3.rds")
})

#' Compute cumulative 'incidence'
#'
#' `cumulate` is an S3 generic to compute cumulative numbers, with methods
#' for different types of objects:
#'
#' \itemize{
#'
#' \item default method is a wrapper for `cumsum`
#'
#' \item `incidence` objects: computes cumulative incidence over time
#'
#' \item `projections` objects: same, for `projections` objects,
#' implemented in the similarly named package; see `?cumulate.projections`
#' for more information, after loading the package
#'
#' }
#'
#'
#' @author Thibaut Jombart \email{thibautjombart@@gmail.com}
#'
#' @seealso The [incidence()] function to generate the 'incidence'
#' objects.
#'
#' @param x An incidence object.
#'
#' @export
#'
#' @examples
#' dat <- as.integer(c(0,1,2,2,3,5,7))
#' group <- factor(c(1, 2, 3, 3, 3, 3, 1))
#' i <- incidence(dat, groups = group)
#' i
#' plot(i)
#'
#' i_cum <- cumulate(i)
#' i_cum
#' plot(i_cum)
#'

#' @rdname cumulate
cumulate <- function(x) {
  UseMethod("cumulate", x)
}




#' @rdname cumulate
#' @export
cumulate.default <- function(x) {
  cumsum(x)
}




#' @rdname cumulate
#' @export
cumulate.incidence <- function(x) {
  if (isTRUE(x$cumulative)) {
    stop("x is already a cumulative incidence")
  }
  out <- x
  out$counts <- apply(x$counts, 2, cumsum)
  out$cumulative <- TRUE
  out
}

context("Test cumulate")

test_that("Results as expected", {
  skip_on_cran()

  dat <- as.Date("2016-01-02") + as.integer(c(7,7,7,0,1,2,2,2,3,3,5,7))
  i <- incidence(dat)

  out <- cumulate(i)
  expect_identical(cumsum(i$counts), as.vector(out$counts))
  expect_true(out$cumulative)
  expect_identical(cumulate(-3:10), cumsum(-3:10))
  expect_error(cumulate(out), "x is already a cumulative incidence")

})

context("Test plotting")

test_that("plot for incidence object", {
  skip_on_cran()

  set.seed(1)
  # dat <- sample(1:50, 200, replace = TRUE, prob = 1 + exp(1:50 * 0.1))
  dat <- readRDS("data_cache/mfdat.rds")[1:200]
  dat2 <- as.Date("2016-01-02") + dat
  dat3 <- as.POSIXct(dat2)
  sex <- c(1, 1, 2, 1, 1, 2, 2, 1, 1, 2, 2, 1, 2, 2, 2, 1, 2, 2, 2, 1, 1, 1, 1,
           2, 2, 2, 1, 1, 1, 2, 1, 1, 1, 2, 1, 2, 2, 2, 1, 1, 1, 2, 2, 1, 1, 1,
           1, 1, 1, 2, 2, 2, 1, 2, 1, 1, 1, 1, 1, 2, 2, 1, 1, 2, 2, 1, 1, 2, 2,
           1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1,
           2, 1, 2, 2, 1, 1, 1, 2, 2, 1, 1, 1, 1, 2, 1, 2, 1, 1, 1, 1, 2, 2, 1,
           1, 1, 1, 2, 2, 2, 1, 1, 2, 2, 2, 2, 2, 1, 2, 2, 1, 1, 2, 1, 1, 1, 1,
           2, 2, 2, 1, 2, 1, 2, 1, 1, 1, 1, 2, 1, 1, 2, 1, 2, 1, 2, 1, 2, 1, 1,
           1, 2, 1, 2, 2, 1, 2, 2, 1, 1, 2, 2, 1, 1, 1, 1, 2, 1, 1, 1, 2, 2, 2,
           1, 1, 2, 2, 1, 2, 1, 1, 2, 2, 1, 1, 2, 1, 1, 1)
  sex <- ifelse(sex == 1, "female", "male")
  dat4 <- c(dat2,
            sample(dat2, replace = TRUE) + 50,
            sample(dat2, replace = TRUE) + 100
           )

  # constructing data
  i <- incidence(dat)
  iog <- incidence(dat, groups = rep("this group", 200))
  i.3 <- incidence(dat, 3L)
  i.14 <- incidence(dat, 14L)
  i.sex <- incidence(dat, 7L, groups = sex)

  # Dates ---------------------------------------------------------------------
  i.POSIX <- incidence(as.POSIXct(dat2, tz = "GMT"))
  i.isoweek <- incidence(dat2, 7L, standard = TRUE)

  # character intervals -------------------------------------------------------
  # Weekly intervals ------------------------------------------
  i.epiweek <- incidence(dat2, "1 epiweek", standard = TRUE)
  i.twoepiweek <- incidence(dat2, "2 epiweeks", standard = TRUE)
  i.sunweek <- incidence(dat2, "1 sunday week")
  i.monweek <- incidence(dat2, "1 monday week")
  i.tueweek <- incidence(dat2, "1 tuesday week")
  i.wedweek <- incidence(dat2, "1 wednesday week")
  i.thuweek <- incidence(dat2, "1 thursday week")
  i.friweek <- incidence(dat2, "1 friday week")
  i.satweek <- incidence(dat2, "1 saturday week")

  expect_identical(i.epiweek, i.sunweek)
  expect_identical(i.isoweek$weeks, i.monweek$weeks)
  expect_identical(get_counts(i.isoweek), get_counts(i.monweek))

  # months and quarters ---------------------------------------
  i.sexmonth <- incidence(dat4, "1 month", groups = rep(sex, 3))
  i.sexquarter <- incidence(dat4, "1 quarter", groups = rep(sex, 3))
  # special case for fit_optim_split:
  i.sex.o <- incidence(c(dat, abs(dat - 45) + 45), 7L, groups = c(sex, rev(sex)))

  fit.i <- suppressWarnings(fit(i))
  fit.i.2 <- suppressWarnings(fit(i, split = 30))
  fit.i.3 <- suppressWarnings(fit(i.3[5:13]))
  fit.POSIX <- suppressWarnings(fit(i.POSIX))
  fit.sex <- suppressWarnings(fit(i.sex))
  fit.sex.o <- suppressWarnings(fit_optim_split(i.sex.o))
  fit.o <- suppressWarnings(fit_optim_split(pool(i.sex.o)))
  p.fit.i <- plot(fit.i)
  p.fit.i.2 <- plot(i, fit = fit.i.2, color = "lightblue")
  p.fit.sex <- plot(fit.sex)
  p.optim.sex   <- fit.sex.o$plot
  p.optim.sex.fit <- plot(fit.sex.o$fit)
  p.optim   <- fit.o$plot
  p.i <- plot(i)
  p.i.cum <- plot(cumulate(i))
  p.i.square <- plot(i, show_cases = TRUE)
  

  p.i.14 <- plot(i.14)
  p.i.2 <- plot(i, color = "blue", alpha = .2)
  p.i.3 <- plot(i.3, fit = fit.i.3, color = "red")
  p.sex <- plot(i.sex)
  p.sex.cum <- plot(cumulate(i.sex))
  p.sex.2 <- plot(i.sex, fit = fit.sex)
  suppressMessages(p.sex.o <- plot(i.sex, fit = fit.sex.o$fit))
  p.sex.3 <- plot(i.sex, fit = fit.sex, col_pal = rainbow)
  p.sex.4 <- plot(i.sex, fit = fit.sex,
                  color = c(male = "salmon3", female = "gold2"))
  p.isoweek   <- plot(i.isoweek)
  p.isoweek.2 <- plot(i.isoweek, labels_week = FALSE)
  p.epiweek   <- plot(i.epiweek)
  p.epiweek.2 <- plot(i.epiweek, labels_week = FALSE)
  p.epiweek.b <- plot(i.epiweek, labels_week = FALSE, n_breaks = nrow(i.epiweek))
  p.twoepiweek <- plot(i.twoepiweek, n_breaks = nrow(i.twoepiweek))
  
  p.sunweek   <- plot(i.sunweek)
  expect_warning({
    p.sunweek.2 <- plot(i.sunweek, labels_iso = FALSE)
  }, "labels_iso is deprecated. Use `labels_week` instead")
  p.monweek   <- plot(i.monweek)
  expect_warning({
    p.monweek.2 <- plot(i.monweek, labels_week = FALSE, labels_iso = TRUE)
  }, "labels_iso is deprecated. The value of `labels_week` will be used")
  p.tueweek   <- plot(i.tueweek)
  p.tueweek.2 <- plot(i.tueweek, labels_week = FALSE)
  p.wedweek   <- plot(i.wedweek)
  p.wedweek.2 <- plot(i.wedweek, labels_week = FALSE)
  p.thuweek   <- plot(i.thuweek)
  p.thuweek.2 <- plot(i.thuweek, labels_week = FALSE)
  p.friweek   <- plot(i.friweek)
  p.friweek.2 <- plot(i.friweek, labels_week = FALSE)
  p.satweek   <- plot(i.satweek)
  p.satweek.2 <- plot(i.satweek, labels_week = FALSE)
  
  p.POSIX     <- plot(i.POSIX)
  p.POSIX.f   <- plot(i.POSIX, fit = fit.POSIX)
  p.month     <- plot(i.sexmonth)
  p.quarter   <- plot(i.sexquarter)

  ## messages
  expect_message(plot(i.sex, show_cases = TRUE, stack = FALSE),
                 "`show_cases` requires the argument `stack = TRUE`")
  expect_message(p.iog <- plot(iog, color = c("this group" = "blue", "that group" = "red")), 
                 "1 colors were not used: \"that group\" = \"red\"")

  ## errors
  expect_error(plot(i, fit = "tamere"),
               "Fit must be a 'incidence_fit' object, or a list of these")
  expect_error(plot(i, fit = list(fit.i, "tamere")),
               "The 2-th item in 'fit' is not an 'incidence_fit' object, but a character")

  ## Normal plots
  vdiffr::expect_doppelganger("incidence fit", p.fit.i)
  vdiffr::expect_doppelganger("incidence plot with two fitting models", p.fit.i.2)
  vdiffr::expect_doppelganger("grouped incidence fit", p.fit.sex)
  vdiffr::expect_doppelganger("incidence plot with default interval", p.i)
  vdiffr::expect_doppelganger("incidence plot with default interval, cumulative", p.i.cum)
  vdiffr::expect_doppelganger("incidence plot with interval of 14 days", p.i.14)
  vdiffr::expect_doppelganger("incidence plot with specified color and alpha", p.i.2)
  vdiffr::expect_doppelganger("incidence plot with interval of 3 days, fit and specified color", p.i.3)
  vdiffr::expect_doppelganger("grouped incidence plot", p.sex)
  vdiffr::expect_doppelganger("grouped incidence plot with one group", p.iog)
  vdiffr::expect_doppelganger("grouped incidence plot, cumulative", p.sex.cum)
  vdiffr::expect_doppelganger("grouped incidence plot with fit", p.sex.2)
  vdiffr::expect_doppelganger("grouped incidence plot with color palette", p.sex.3)
  vdiffr::expect_doppelganger("grouped incidence plot with specified color", p.sex.4)
  vdiffr::expect_doppelganger("incidence plot from POSIXct data", p.POSIX)
  vdiffr::expect_doppelganger("incidence plot from POSIXct data with fit", p.POSIX.f)
  vdiffr::expect_doppelganger("incidence plot with isoweek labels", p.isoweek)
  vdiffr::expect_doppelganger("incidence plot without isoweek labels", p.isoweek.2)
  vdiffr::expect_doppelganger("incidence plot by month", p.month)
  vdiffr::expect_doppelganger("incidence plot by quarter", p.quarter)
  vdiffr::expect_doppelganger("incidence fit plot with split", p.sex.o)
  vdiffr::expect_doppelganger("incidence fit list plot with split", p.optim.sex.fit)
  vdiffr::expect_doppelganger("split optimum plot", p.optim.sex)
  vdiffr::expect_doppelganger("split optimum plot pooled", p.optim)
  vdiffr::expect_doppelganger("epiquares single plot", p.i.square)

  ## Weekly plots
  vdiffr::expect_doppelganger("sun weekly incidence with labels", p.sunweek)
  vdiffr::expect_doppelganger("sun weekly incidence with labels", p.epiweek)
  vdiffr::expect_doppelganger("sun weekly incidence with dates", p.sunweek.2)
  vdiffr::expect_doppelganger("sun weekly incidence with dates", p.epiweek.2)
  vdiffr::expect_doppelganger("sun weekly incidence with dates and full breaks", p.epiweek.b)
  vdiffr::expect_doppelganger("sun semi-weekly incidence with dates and full breaks", p.twoepiweek)


  vdiffr::expect_doppelganger("mon weekly incidence with labels", p.monweek)
  vdiffr::expect_doppelganger("mon weekly incidence with dates", p.monweek.2)

  vdiffr::expect_doppelganger("tue weekly incidence with labels", p.tueweek)
  vdiffr::expect_doppelganger("tue weekly incidence with dates", p.tueweek.2)
  
  vdiffr::expect_doppelganger("wed weekly incidence with labels", p.wedweek)
  vdiffr::expect_doppelganger("wed weekly incidence with dates", p.wedweek.2)
  
  vdiffr::expect_doppelganger("thu weekly incidence with labels", p.thuweek)
  vdiffr::expect_doppelganger("thu weekly incidence with dates", p.thuweek.2)
  
  vdiffr::expect_doppelganger("fri weekly incidence with labels", p.friweek)
  vdiffr::expect_doppelganger("fri weekly incidence with dates", p.friweek.2)
  
  vdiffr::expect_doppelganger("sat weekly incidence with labels", p.satweek)
  vdiffr::expect_doppelganger("sat weekly incidence with dates", p.satweek.2)
})

## Non-exported function extracting info and predictions from a lm object
## - reg is a lm object
## - origin is the first date of the incidence
## - level is a confidence level, defaulting to .95

extract_info <- function(reg, origin, level){
  if (is.null(reg)) {
    return(NULL)
  }

  ## extract growth rates (r)
  ## here we need to keep all coefficients when there are interactions
  to.keep <- grep("^dates.x.*$", names(stats::coef(reg)), value = TRUE)
  r <- stats::coef(reg)[to.keep]
  use.groups <- length(r) > 1
  if (use.groups) {
    names(r) <- reg$xlevels[[1]] # names = levels if groups
  } else {
    names(r) <- NULL # no names otherwise
  }
  r.conf <- stats::confint(reg, to.keep, level)
  rownames(r.conf) <- names(r)
  if (use.groups) {
    r[-1] <- r[-1] + r[1] # add coefs to intercept
    r.conf[-1,] <- r.conf[-1,] + r.conf[1,] # add coefs to intercept
  }


  ## need to pass new data spanning all dates and groups here
  if (use.groups) {
    new.data <- expand.grid(sort(unique(reg$model$dates.x)),
                            levels(reg$model$groups))
    names(new.data) <- c("dates.x", "groups")
  } else {
    new.data <- data.frame(dates.x = sort(unique(reg$model$dates.x)))
  }
  pred <- exp(stats::predict(reg, newdata = new.data, interval = "confidence",
                             level = level))
  ## keep track of dates and groups for plotting
  pred <- cbind.data.frame(new.data, pred)
  info <- list(r = r, r.conf = r.conf,
               pred = pred)

  if (r[1] > 0 ) { # note: choice of doubling vs halving only based on 1st group
    info$doubling <- log(2) / r
    info$doubling.conf <- log(2) / r.conf
    o.names <- colnames(info$doubling.conf)
    info$doubling.conf <- info$doubling.conf[, rev(seq_along(o.names)),
                                             drop = FALSE]
    colnames(info$doubling.conf) <- o.names
  } else {
    info$halving <- log(0.5) / r
    info$halving.conf <- log(0.5) / r.conf
  }

  ## We need to store the date corresponding to 'day 0', as this will be used
  ## to create actual dates afterwards (as opposed to mere numbers of days).
  ## origin <- min(x$dates)

  ## Dates are reconstructed from info$pred$dates.x and origin).  Note that
  ## this is approximate, as dates are forced to be integers. A better option
  ## would be to convert the dates to numbers, but ggplot2 is no longer
  ## consistent when mixing up Date and decimal numbers (it works only in some
  ## cases / geom).
  dates <- origin + pred$dates.x
  info$pred <- cbind.data.frame(dates, info$pred)
  out <- list(model = reg, info = info, origin = origin)
  class(out) <- "incidence_fit"
  out
}

context("Test fit functions")

test_that("fit", {
  skip_on_cran()

  dat <- readRDS("data_cache/mfdat.rds")
  sex <- readRDS("data_cache/mfcat.rds")

  i <- incidence(dat, 5L)
  i.sex <- incidence(dat, 5L, groups = sex)
  fit.i <- fit(i)
  expect_warning(fit.i.sex <- fit(i.sex),
                 "3 dates with incidence of 0 ignored for fitting")

  expect_equal_to_reference(fit.i, file = "rds/fit.i.rds")
  expect_equal_to_reference(fit.i.sex, file = "rds/fit.i.sex.rds")
  expect_equal_to_reference(capture.output(fit.i), file = "rds/print.fit.i.rds")
  expect_equal_to_reference(capture.output(fit.i.sex), file = "rds/print.fit.sex.rds")

  ## errors
  x <- incidence(c(1, 0, 0, 0), interval = 7)
  expect_error(fit(x), "Only 1 date with non-zero incidence. Cannot fit model to 1 data point.")
})

test_that("fit_optim_split", {
  skip_on_cran()

  dat <- readRDS("data_cache/mfdat.rds")
  sex <- readRDS("data_cache/mfcat.rds")

  i     <- incidence(dat, 5L)
  i.sex <- incidence(dat, 5L, groups = sex)
  i.fit     <- fit_optim_split(i, plot = FALSE)
  i.fit.sex <- fit_optim_split(i.sex, plot = FALSE)
  expect_equal_to_reference(i.fit,
                            file = "rds/o.fit.i.rds")
  expect_equal_to_reference(i.fit.sex,
                            file = "rds/o.fit.i.sex.rds")

  expect_is(i.fit$df, "data.frame")
  expect_is(i.fit$fit, "incidence_fit_list")

  expect_is(i.fit.sex$df, "data.frame")
  expect_is(i.fit.sex$fit, "incidence_fit_list")

  expect_output(print(i.fit.sex$fit), "<list of incidence_fit objects>")
  expect_output(print(i.fit.sex$fit), "[^N][^A]")
  expect_true(any(is.na(get_info(i.fit.sex$fit, "halving", na.rm = FALSE))))

  ## errors
  expect_error(fit_optim_split(i, window = -1),
               "No date left to try after defining splits to try.")
})

test_that("internals for fitting", {
  skip_on_cran()

  expect_null(extract_info(NULL))
})

test_that("fitting results are the same for incidence fits on Dates and POSIXct", {
  days <- 1:14
  dat_cases <- round(exp(.2*(days)))
  dat_dates_Date <- rep(as.Date(Sys.Date() + days), dat_cases)
  dat_dates_POSIXct <- as.POSIXct(dat_dates_Date)

  iD <- incidence(dat_dates_Date)
  iP <- incidence(dat_dates_POSIXct)

  expect_equal(fit(iP), fit(iD))
})

##' Pool 'incidence' across groups
##'
##' This function pools incidence across all groups of an `incidence`
##' object. The resulting [incidence()] object will contains counts
##' summed over all groups present in the input.
##'
##' @author Thibaut Jombart \email{thibautjombart@@gmail.com}
##'
##' @seealso The [incidence()] function to generate the 'incidence'
##' objects.
##'
##' @inheritParams incidence
##'
##' @export
##'
##' @examples
##' dat <- as.integer(c(0,1,2,2,3,5,7))
##' group <- factor(c(1, 2, 3, 3, 3, 3, 1))
##' i <- incidence(dat, groups = group)
##' i
##' i$counts
##'
##' ## pool all groups
##' pool(i)
##' pool(i)$counts
##'
##' ## pool only groups 1 and 3
##' pool(i[,c(1,3)])
##' pool(i[,c(1,3)])$counts
##'

pool <- function(x){
    if (!inherits(x, "incidence")) {
        stop(sprintf(
            "x should be an 'incidence' object (its class is: %s)",
            class(x)))
    }
    if (ncol(x$counts) == 1) return(x)
    x$counts <- matrix(apply(x$counts, 1 , sum), ncol = 1)
    x
}

context("Pool")

test_that("Pool works", {
  skip_on_cran()

  expect_error(pool(1),
               "x should be an 'incidence' object \\(its class is: numeric\\)")
  expect_identical(incidence(1L), pool(incidence(1L)))

  dat <- as.integer(c(0, 1, 2, 2, 3, 5, 7))
  fac <- factor(c(1, 2, 3, 3, 3, 3, 1))
  expect_identical(pool(incidence(dat, groups = fac)), incidence(dat))
})

context("Bootstrapping incidence")

set.seed(1)
dates <- as.integer(sample(-3:100, 50, replace = TRUE))
DATES <- as.Date("2016-09-20") + dates
groups <- sample(c("toto", "tata"), 50, replace = TRUE)


test_that("Bootstrap needs an incidence object", {
  expect_error(bootstrap(DATES), "x is not an incidence object")
})

test_that("estimate_peak needs an incidence object", {
  expect_error(estimate_peak(DATES), "x is not an incidence object")
})

test_that("Bootstrap incidence with groups", {
  skip_on_cran()

  x <- incidence(DATES, 3, groups = groups)
  y <- bootstrap(x)
  z <- bootstrap(x, TRUE)
  expect_identical(colSums(x$counts), colSums(y$counts))
  expect_identical(colSums(x$counts), colSums(z$counts))
  expect_identical(colnames(x$counts), colnames(y$counts))
  expect_identical(colnames(x$counts), colnames(z$counts))
  expect_identical(x$interval, y$interval)
  expect_identical(x$interval, z$interval)

})


context("Mountain Climbing")

test_that("find_peak can find the peak", {
  skip_on_cran()


  x <- incidence(DATES, 3, groups = groups)
 
  expect_error(find_peak(1:10), "x is not an incidence object")
  expect_message(p1 <- find_peak(x), "'x' is stratified by groups\npooling groups before finding peaks")
  expect_length(p1, 1L)
  expect_named(find_peak(x, pool = FALSE), c("tata", "toto"))
})


test_that("estimate_peak can roughly estimate it", {
  x <- incidence(DATES, 3, groups = groups)
  y <- incidence(dates, 3)
  expect_message(e1 <- estimate_peak(x), "'x' is stratified by groups\npooling groups before finding peaks")
  e2 <- estimate_peak(y)
  expect_named(e1, c("observed", "estimated", "ci", "peaks"))
  expect_named(e2, c("observed", "estimated", "ci", "peaks"))
  ## The observed is identical to find_peak
  expect_identical(e1$observed, find_peak(pool(x)))
  expect_identical(e2$observed, find_peak(pool(y)))
  ## The number of peaks defaults to 100
  expect_length(e1$peaks, 100)
  expect_length(e2$peaks, 100)
  ## The observed falls within the confidence interval
  expect_gt(as.integer(e1$observed), as.integer(e1$ci[1]))
  expect_lt(as.integer(e1$observed), as.integer(e1$ci[2]))
  expect_gt(as.integer(e2$observed), as.integer(e2$ci[1]))
  expect_lt(as.integer(e2$observed), as.integer(e2$ci[2]))
})

.onLoad <- function(...) {
  op <- options()
  op.incidence <- list(incidence.max.days = 18262,
                      incidence.warn.first_date = TRUE)
  toset <- !names(op.incidence) %in% op
  if (any(toset)) options(op.incidence[toset])
}

#' Check the user-fed arguments and give warnings if they are wrong.
#'
#' @param dots a list of user-supplied arguments
#' @param args names of arguments appropriate for the function.
#'
#' @return dots, modified if necessary
#' @noRd
#' @keywords internal
#'
#' @examples
#' dots <- c(hAy = 1, "lsdflk" = 1, "stuperman" = TRUE, iso_week = TRUE)
#' args <- c("superman", "hey", "ho", "lets", "go", "standard")
#' check_dots(dots, args)
check_dots <- function(dots, args) {
  if (length(dots) == 0) {
    return(dots)
  }
  dnames     <- names(dots)
  scores     <- utils::adist(paste0("^", dnames), args, fixed = FALSE) < 2
  recognized <- rowSums(scores) > 0
  msg <- ""
  if (sum(scores) > 0) {
    words <- apply(scores[recognized, , drop = FALSE], 1,
                   function(i) paste(args[i], collapse = ", "))
    errs  <- paste(format(dnames[recognized]), format(words), sep = " : ")
    errs  <- paste(errs, collapse = "\n\t")
    msg   <- sprintf("\n\nPotentially misspelled options:\n\t%s", errs)
  }
  if (sum(!recognized) > 0) {
    dre <- paste(dnames[!recognized & dnames != "iso_week"], collapse = ", ")
    msg <- if (dre != "") paste0(msg, "\n\nUnrecognised options:\n\t", dre) else msg
  }
  if ("iso_week" %in% dnames) {
    warning(paste("The parameter `iso_week` has been deprecated as of incidence",
                  "version 1.3. Please use `standard` instead."),
            call. = FALSE
    )
    names(dots)[dnames == "iso_week"] <- "standard"
  }
  MSG <- "Misspelled or unrecognized options were found."
  if (msg != "") {
    stop(paste(MSG, msg), call. = FALSE)
  }
  dots
}

#' Compute incidence of events from a vector of dates.
#'
#' This function computes incidence based on dates of events provided in
#' various formats. A fixed interval, provided as numbers of days, is used to
#' define time intervals. Counts within an interval always include the first
#' date, after which they are labeled, and exclude the second. For instance,
#' intervals labeled as 0, 3, 6, ... mean that the first bin includes days 0, 1
#' and 2, the second interval includes 3, 4 and 5 etc.
#'
#' @param dates A vector of dates, which can be provided as objects of the
#' class: integer, numeric, Date, POSIXct, POSIXlt, and character. (See Note 
#' about `numeric` and `character` formats) 
#'
#' @param interval An integer or character indicating the (fixed) size of the
#' time interval used for computing the incidence; defaults to 1 day. This can
#' also be a text string that corresponds to a valid date interval: day, week,
#' month, quarter, or year. (See Note).
#'
#' @param groups An optional factor defining groups of observations for which
#' incidence should be computed separately.
#'
#' @param na_as_group A logical value indicating if missing group (NA) should be
#' treated as a separate group.
#'
#' @param first_date,last_date optional first/last dates to be used in the
#'   epicurve. When these are `NULL` (default), the dates from the first/last
#'   dates are taken from the observations. If these dates are provided, the
#'   observations will be trimmed to the range of \[first_date, last_date\].
#'
#' @param ... Additional arguments passed to other methods (none are used).
#'
#' @return An list with the class `incidence`, which contains the
#' following items:
#'
#'
#' - **dates**: The dates marking the left side of the bins used for counting
#' events. When `standard = TRUE` and the interval represents weeks, months,
#' quarters, or years, the first date will represent the first standard date
#' (See Interval specification, below).
#'    
#' - **counts**: A matrix of incidence counts, which one column per group (and
#' a single column if no groups were used).
#'
#' - **timespan**: The length of the period for which incidence is computed, in
#' days.
#'
#' - **interval**: The bin size. If it's an integer, it represents the number
#'   of days between each bin. It can also be a character, e.g. "2 weeks" or 
#'   "6 months".
#'
#' - **n**: The total number of cases.
#'
#' - **weeks**: Dates in week format (YYYY-Www), where YYYY corresponds to the
#'   year of the given week and ww represents the numeric week of the year. 
#'   This will be a produced from the function [aweek::date2week()]. Note that
#'   these will have a special `"week_start"` attribute indicating which day of
#'   the ISO week the week starts on (see Weeks, below).
#'   
#' - **isoweeks**: ISO 8601 week format YYYY-Www, which is returned only when
#' ISO week-based weekly incidence is computed.
#'
#'
#' @details For details about the `incidence class`, see the dedicated
#' vignette:\cr `vignette("incidence_class", package = "incidence")`
#'
#' @note \subsection{Input data (`dates`)}{
#'  - **Decimal (numeric) dates**: will be truncated with a warning
#'  - **Character dates** should be in the unambiguous `yyyy-mm-dd` (ISO 8601)
#'   format. Any other format will trigger an error.
#' }
#' 
#' \subsection{Interval specification (`interval`)}{
#' If `interval` is a valid character (e.g. "week" or "1 month"), then
#' the bin will start at the beginning of the interval just before the first
#' observation by default. For example, if the first case was recorded on
#' Wednesday, 2018-05-09:
#'
#'  - "week"    : first day of the week (i.e. Monday, 2018-05-07) (defaults to ISO weeks, see "Week intervals", below)
#'  - "month"   : first day of the month (i.e. 2018-05-01)
#'  - "quarter" : first day of the quarter (i.e. 2018-04-01)
#'  - "year"    : first day of the calendar year (i.e. 2018-01-01)
#'
#' These default intervals can be overridden with `standard = FALSE`, which
#' sets the interval to begin at the first observed case.
#' }
#' 
#' \subsection{Week intervals}{
#'
#' As of _incidence_ version 1.7.0, it is possible to construct standardized
#' incidence objects standardized to any day of the week thanks to the
#' [aweek::date2week()] function from the \pkg{aweek} package. The default
#' state is to use ISO 8601 definition of weeks, which start on Monday. You can
#' specify the day of the week an incidence object should be standardised to by
#' using the pattern "{n} {W} weeks" where "{W}" represents the weekday in an
#' English or current locale and "{n}" represents the duration, but this can be
#' ommitted.  Below are examples of specifying weeks starting on different days
#' assuming we had data that started on 2016-09-05, which is ISO week 36 of
#' 2016:
#'
#'  - interval = "2 monday weeks" (Monday 2016-09-05)
#'  - interval = "1 tue week" (Tuesday 2016-08-30)
#'  - interval = "1 Wed week" (Wednesday 2016-08-31)
#'  - interval = "1 Thursday week" (Thursday 2016-09-01) 
#'  - interval = "1 F week" (Friday 2016-09-02)
#'  - interval = "1 Saturday week" (Saturday 2016-09-03)
#'  - interval = "Sunday week" (Sunday 2016-09-04)
#'
#' It's also possible to use something like "3 weeks: Saturday"; In addition,
#' there are keywords reserved for specific days of the week:
#'   
#'   - interval = "week", standard = TRUE (Default, Monday)
#'   - interval = "ISOweek"  (Monday)
#'   - interval = "EPIweek"  (Sunday)
#'   - interval = "MMWRweek" (Sunday)
#'
#' The "EPIweek" specification is not strictly reserved for CDC epiweeks, but
#' can be prefixed (or posfixed) by a day of the week: "1 epiweek: Saturday".
#'
#' }
#'
#' \subsection{The `first_date` argument}{
#' Previous versions of _incidence_ had the `first_date` argument override
#' `standard = TRUE`. It has been changed as of _incidence_ version 1.6.0 to
#' be more consistent with the behavior when `first_date = NULL`. This, however
#' may be a change in behaviour, so a warning is now issued once and only once
#' if `first_date` is specified, but `standard` is not. To never see this
#' warning, use `options(incidence.warn.first_date = FALSE)`. 
#' }
#'
#' The intervals for "month", "quarter", and "year" will necessarily vary in the
#' number of days they encompass and warnings will be generated when the first
#' date falls outside of a calendar date that is easily represented across the
#' interval.
#' 
#'
#' @seealso
#' The main other functions of the package include:
#'
#'  - [incidence::plot.incidence()]: Plot epicurves from an incidence object.
#'
#'  - [incidence::fit()]: Fit log-linear model to computed incidence.
#'
#'  - [incidence::fit_optim_split()]: Find the optimal peak of the epidemic
#' and fits log-linear models on either side of the peak.
#'
#'  - [incidence::subset()]: Handling of `incidence`
#' objects.
#'
#'  - [incidence::pool()]: Sum incidence over groups.
#'
#'  - [incidence::as.data.frame.incidence()]: Convert an `incidence` object to a
#' `data.frame`.
#'
#' The following vignettes are also available:
#'
#' - `overview`: Provides an overview of the package's features.
#'
#' - `customize_plot`: Provides some tips on finer plot customization.
#'
#' - `incidence_class`: Details the content of the `incidence`
#' class.
#'
#'
#'
#'
#' @author Thibaut Jombart, Rich Fitzjohn, Zhian Kamvar
#'
#' @rdname incidence
#'
#' @importFrom utils head tail
#'
#' @export
#'
#' @examples
#' ## toy example
#' incidence(c(1, 5, 8, 3, 7, 2, 4, 6, 9, 2))
#' incidence(c(1, 5, 8, 3, 7, 2, 4, 6, 9, 2), 2)
#'
#' ## example using simulated dataset
#' if(require(outbreaks)) { withAutoprint({
#'   onset <- outbreaks::ebola_sim$linelist$date_of_onset
#'
#'   ## daily incidence
#'   inc <- incidence(onset)
#'   inc
#'   plot(inc)
#'
#'   ## weekly incidence
#'   inc.week <- incidence(onset, interval = 7, standard = FALSE)
#'   inc.week
#'   plot(inc.week)
#'   plot(inc.week, border = "white") # with visible border
#'   
#'   # Starting on Monday
#'   inc.isoweek <- incidence(onset, interval = "isoweek")
#'   inc.isoweek
#'   
#'   # Starting on Sunday
#'   inc.epiweek <- incidence(onset, interval = "epiweek")
#'   inc.epiweek
#' 
#'   # Starting on Saturday
#'   inc.epiweek <- incidence(onset, interval = "saturday epiweek")
#'   inc.epiweek
#'
#'   ## use group information
#'   sex <- outbreaks::ebola_sim$linelist$gender
#'   inc.week.gender <- incidence(onset, interval = 7,
#'                                groups = sex, standard = FALSE)
#'   inc.week.gender
#'   head(inc.week.gender$counts)
#'   plot(inc.week.gender, border = "grey90")
#'   inc.satweek.gender <- incidence(onset, interval = "2 epiweeks: saturday",
#'                                   groups = sex)
#'   inc.satweek.gender
#'   plot(inc.satweek.gender, border = "grey90")
#'
#' })}
#' 
#' # Use of first_date
#' d <- Sys.Date() + sample(-3:10, 10, replace = TRUE)
#' 
#' # `standard` specified, no warning
#' di <- incidence(d, interval = "week", first_date = Sys.Date() - 10, standard = TRUE)
#'
#' # warning issued if `standard` not specified
#' di <- incidence(d, interval = "week", first_date = Sys.Date() - 10)
#'
#' # second instance: no warning issued
#' di <- incidence(d, interval = "week", first_date = Sys.Date() - 10)
#'
#'
incidence <- function(dates, interval = 1L, ...) {
  UseMethod("incidence")
}


#' @export
#' @rdname incidence
incidence.default <- function(dates, interval = 1L, ...) {
  incidence(check_dates(dates), interval = interval, ...)
}

#' @export
#' @rdname incidence
#'
#' @param standard (Only applicable to Date objects) When `TRUE` (default) and the
#'   `interval` one of "week", "month", "quarter", or "year", then this will
#'   cause the bins for the counts to start at the beginning of the interval
#'   (See Note). 

incidence.Date <- function(dates, interval = 1L, standard = TRUE, groups = NULL,
                           na_as_group = TRUE, first_date = NULL,
                           last_date = NULL, ...) {
  the_call <- match.call()
  warnme <- getOption('incidence.warn.first_date', FALSE) 
  if (warnme && !is.null(the_call[["first_date"]]) && is.null(the_call[["standard"]])) {
    fd  <- as.character(deparse(the_call[["first_date"]]))
    msg <- "\n\nAs of incidence version 1.6.0, the default behavior has been"
    msg <- paste(msg, "modified so that `first_date` no longer overrides")
    msg <- paste(msg, "`standard`. This means that the first date will be")
    msg <- paste(msg, "either on or before %s.\nIf you want to use %s as the precise")
    msg <- paste(msg, "`first_date`, set `standard = FALSE`.")
    msg <- paste(msg, "To remove this warning in the future,  explicitly set the `standard` argument OR use `options(incidence.warn.first_date = FALSE)`\n", sep = "\n\n")
    warning(sprintf(msg, first_date, fd))
    # turn the warning off so that it's not so noisy
    options(incidence.warn.first_date = FALSE)
  }
  dots <- check_dots(list(...), names(formals(incidence.Date)))
  ## make sure input can be used
  if (!is.logical(standard)) {
    stop("The argument `standard` must be either `TRUE` or `FALSE`.")
  }
  if ("standard" %in% names(dots)) {
    # the user specified iso_week and was given a warning.
    standard <- dots$standard
  }
  out <- make_incidence(dates = dates,
                        interval = interval,
                        groups = groups,
                        na_as_group = na_as_group,
                        first_date = first_date,
                        last_date = last_date,
                        standard = standard,
                        ...)
  if (check_week(interval) && standard) {
    # dates are the first days of corresponding ISOweeks.
    week_start   <- get_week_start(interval)
    out$weeks <- aweek::date2week(out$dates, week_start, floor_day = TRUE)
    if (attr(out$weeks, "week_start") == 1) {
      out$isoweeks <- as.character(out$weeks)
    }
  }

  out
}


#' @export
#' @rdname incidence
incidence.character <- function(dates, interval = 1L, standard = TRUE, groups = NULL,
                           na_as_group = TRUE, first_date = NULL,
                           last_date = NULL, ...) {
  iso_std <- grepl("^[0-9]{4}-[01][0-9]-[0-3][0-9]$", trimws(dates))
  iso_std[is.na(dates)] <- TRUE # prevent false alarms
  if (!all(iso_std)) {
    msg <- paste("Not all dates are in ISO 8601 standard format (yyyy-mm-dd).",
                 "The first incorrect date is %s"
    )
    stop(sprintf(msg, dates[!iso_std][1]))
  }
  dots  <- check_dots(list(...), names(formals(incidence.Date)))
  dates <- check_dates(dates)

  ret <- incidence(as.Date(trimws(dates)),
                   interval = interval,
                   standard = standard,
                   groups = groups,
                   na_as_group = na_as_group,
                   first_date = first_date,
                   last_date = last_date,
                   ...)
  ret
}
## The default incidence is designed for dates provided as integers, and a fixed
## time interval defaulting to 1. 'bins' are time intervals, identified by the
## left date, left-inclusive and right-exclusive, i.e. the time interval defined
## by d1 and d2 is [d1, d2[.

#' @export
#' @rdname incidence

incidence.integer <- function(dates, interval = 1L, groups = NULL,
                              na_as_group = TRUE, first_date = NULL,
                              last_date = NULL, ...) {
  dots <- check_dots(list(...), names(formals(incidence.integer)))
  interval <- valid_interval_integer(interval)
  out <- make_incidence(dates = dates,
                        interval = interval,
                        groups = groups,
                        na_as_group = na_as_group,
                        first_date = first_date,
                        last_date = last_date,
                        ...)
  out$dates    <- as.integer(out$dates)
  out$timespan <- as.integer(out$timespan)
  out$interval <- as.integer(out$interval)
  out
}

#' @export
#' @rdname incidence

incidence.numeric <- function(dates, interval = 1L, groups = NULL,
                              na_as_group = TRUE, first_date = NULL,
                              last_date = NULL, ...) {
  dots <- check_dots(list(...), names(formals(incidence.numeric)))
  interval  <- valid_interval_integer(interval)
  ## make sure input can be used
  out <- make_incidence(dates = dates,
                        interval = interval,
                        groups = groups,
                        na_as_group = na_as_group,
                        first_date = first_date,
                        last_date = last_date,
                        ...)
  out$dates <- as.numeric(out$dates)
  out
}

#' @export
#' @rdname incidence

incidence.POSIXt <- function(dates, interval = 1L, standard = TRUE, groups = NULL,
                             na_as_group = TRUE, first_date = NULL,
                             last_date = NULL, ...) {
  ## make sure input can be used

  dots  <- check_dots(list(...), names(formals(incidence.Date)))
  dates <- check_dates(as.POSIXct(dates))

  ret <- incidence(as.Date(dates),
                   interval = interval,
                   standard = standard,
                   groups = groups,
                   na_as_group = na_as_group,
                   first_date = first_date,
                   last_date = last_date,
                   ...)

  ret$dates <- as.POSIXlt(ret$dates)
  if (inherits(dates, "POSIXct")) {
    ret$dates <- as.POSIXct(ret$dates)
  }
  ret
}

context("Conversions of incidence objects")

test_that("as.data.frame works", {
  skip_on_cran()
  # skip("re-write with aweek in place")

  dat <- as.integer(c(0,1,2,2,3,5,7))
  dat2 <- as.Date("2016-01-02") + dat
  fac <- factor(c(1, 2, 3, 3, 3, 3, 1))
  one_group <- rep("a", 7)


  i_group_df <- data.frame(
         dates = c(0L, 1L, 2L, 3L, 4L, 5L, 6L, 7L),
            "1" = c(1L, 0L, 0L, 0L, 0L, 0L, 0L, 1L),
            "2" = c(0L, 1L, 0L, 0L, 0L, 0L, 0L, 0L),
            "3" = c(0L, 0L, 2L, 1L, 0L, 1L, 0L, 0L),
            check.names = FALSE, stringsAsFactors = TRUE
  )
  # one group
  i_og_df <- i_group_df[, 1, drop = FALSE]
  i_og_df$a <- rowSums(i_group_df[-1])

  i_group_df_long <- data.frame(
         dates = c(0L, 1L, 2L, 3L, 4L, 5L, 6L, 7L, 0L, 1L, 2L, 3L, 4L, 5L, 6L,
                   7L, 0L, 1L, 2L, 3L, 4L, 5L, 6L, 7L),
        counts = c(1L, 0L, 0L, 0L, 0L, 0L, 0L, 1L, 0L, 1L, 0L, 0L, 0L, 0L, 0L,
                   0L, 0L, 0L, 2L, 1L, 0L, 1L, 0L, 0L),
        groups = as.factor(c("1", "1", "1", "1", "1", "1", "1", "1", "2", "2",
                             "2", "2", "2", "2", "2", "2", "3", "3", "3", "3",
                             "3", "3", "3", "3")), stringsAsFactors = TRUE
  )

  i <- incidence(dat, groups = fac)
  iog <- incidence(dat, groups = one_group)
  i.7 <- incidence(dat2, 7L, standard = TRUE)
  i.7.group <- incidence(dat2, 7L, standard = TRUE, groups = fac)
  df  <- as.data.frame(i)
  df1 <- as.data.frame(i, long = TRUE)
  df2 <- as.data.frame(incidence(1:2))
  df2res <- data.frame(
                       dates = c(1L, 2L),
                       counts = c(1L, 1L)
                       )
  df3 <- as.data.frame(i.7)
  df3res <- data.frame(
                       dates = as.Date(c("2015-12-28", "2016-01-04")),
                       weeks = c("2015-W53", "2016-W01"),
                       isoweeks = as.factor(c("2015-W53", "2016-W01")),
                       counts = c(2L, 5L)
                       )
  df3res$weeks <- aweek::date2week(df3res$dates, 1L, floor_day = TRUE, factor = TRUE)

  df4 <- as.data.frame(i.7, long = TRUE)
  df5 <- as.data.frame(i.7.group)
  df5res <- data.frame(
                       dates = as.Date(c("2015-12-28", "2016-01-04")),
                       weeks = c("2015-W53", "2016-W01"),
                       isoweeks = as.factor(c("2015-W53", "2016-W01")),
                       "1" = c(1L, 1L),
                       "2" = c(1L, 0L),
                       "3" = c(0L, 4L),
                       check.names = FALSE
                       )
  df5res$weeks <- aweek::date2week(df5res$dates, 1L, floor_day = TRUE, factor = TRUE)

  df6 <- as.data.frame(i.7.group, long = TRUE)
  df6res <- data.frame(
                       dates = as.Date(c("2015-12-28", "2016-01-04", "2015-12-28", "2016-01-04",
                                 "2015-12-28", "2016-01-04")),
                       weeks = c("2015-W53", "2016-W01", "2015-W53", "2016-W01", "2015-W53",
                                 "2016-W01"),
                       isoweeks = as.factor(c("2015-W53", "2016-W01", "2015-W53", "2016-W01",
                                              "2015-W53", "2016-W01")),
                       counts = c(1L, 1L, 1L, 0L, 0L, 4L),
                       groups = as.factor(c("1", "1", "2", "2", "3", "3"))
                       )
  df6res$weeks <- aweek::date2week(df6res$dates, 1L, floor_day = TRUE, factor = TRUE)
  
  df7 <- as.data.frame(iog)
  df7res <- data.frame(
                       dates = c(0L, 1L, 2L, 3L, 4L, 5L, 6L, 7L),
                       a = c(1L, 1L, 2L, 1L, 0L, 1L, 0L, 1L)
                       )

  df8 <- as.data.frame(iog, long = TRUE)
  df8res <- data.frame(
                       dates = c(0L, 1L, 2L, 3L, 4L, 5L, 6L, 7L),
                       counts = c(1L, 1L, 2L, 1L, 0L, 1L, 0L, 1L),
                       groups = as.factor(c("a", "a", "a", "a", "a", "a", "a", "a"))
                       )

  # expect_equal_to_reference(df, file = "rds/df.rds")
  expect_identical(df, i_group_df)
  # expect_equal_to_reference(dfl, file = "rds/dfl.rds")
  expect_identical(df1, i_group_df_long)
  # expect_equal_to_reference(df2, file = "rds/df2.rds")
  expect_identical(df2, df2res)
  # expect_equal_to_reference(df3, file = "rds/df3.rds")
  expect_identical(df3, df3res)
  expect_equal(df3, df4)
  # expect_equal_to_reference(df5, file = "rds/df5.rds")
  expect_identical(df5, df5res)
  # expect_equal_to_reference(df6, file = "rds/df6.rds")
  expect_identical(df6, df6res)
  expect_named(df7, c("dates", "a"))
  expect_named(df8, c("dates", "counts", "groups"))
})

test_that("as.incidence works", {
  skip_on_cran()

  dates_int <- sample(1:15, 100, replace = TRUE)
  dates <- as.Date("2017-04-01") + dates_int
  groups <- sample(letters[1:3], 100, replace = TRUE)

  i1 <- incidence(dates, interval = 2)
  i2 <- incidence(dates_int)
  i3 <- incidence(dates, interval = 7, groups = groups)
  i4 <- incidence(dates_int, interval = 7, groups = groups)

  expect_equal(as.incidence(i1$counts, i1$dates), i1)
  expect_equal(as.incidence(as.vector(i1$counts), i1$dates), i1)
  expect_equal(as.incidence(i2$counts, i2$dates), i2)
  expect_equal(as.incidence(i3$counts, i3$dates), i3)
  expect_equal(as.incidence(rep(1,10)), incidence(1:10))
  expect_equal(as.incidence(get_counts(i4), interval = 7L), i4)
  expect_equal(as.incidence(as.data.frame(get_counts(i4)), interval = 7L), i4)


  msg <- "Interval needs to be specified if there is only one date."
  expect_error(as.incidence(i3$counts[1,,drop = FALSE], i3$dates[1]),
               msg)

  msg <- "Columns should be named to label groups."
  expect_error(as.incidence(unname(i3$counts), i3$dates),
               msg)
})

context("standardisation tests") 


d <- c('2019-04-18', '2019-04-14', '2019-03-31', '2019-04-03', '2019-03-30',
       '2019-04-05', '2019-03-12', '2019-04-07', '2019-04-02', '2019-03-09',
       '2019-04-20', '2019-04-23', '2019-03-07', '2019-03-25', '2019-03-27',
       '2019-04-13', '2019-04-15', '2019-04-04', '2019-03-30', '2019-03-19')

test_that("standard will override first_date", {
       
  expect_output(print(incidence(d, interval = "week", standard = TRUE)), "2019-03-04")       
  expect_output(print(incidence(d, interval = "week", standard = TRUE)), "2019-W10")       
  expect_output(print(incidence(d, interval = "isoweek", standard = TRUE)), "2019-W10")       
  expect_output(print(incidence(d, interval = "1 isoweek", standard = TRUE)), "2019-W10")       
  expect_output(print(incidence(d, interval = "monday week", standard = TRUE)), "2019-W10")       
  expect_output(print(incidence(d, interval = "week", standard = FALSE)), "2019-03-07")       
  expect_error(incidence(d, interval = "isoweek", standard = FALSE), 
               "The interval 'isoweek' implies a standard and cannot be used with `standard = FALSE`")       

  expect_error(incidence(d, interval = "monday week", standard = FALSE), 
               "The interval 'monday week' implies a standard and cannot be used with `standard = FALSE`")       
  expect_output(print(incidence(d, interval = "month", standard = TRUE)), "2019-03-01")       
  expect_output(print(incidence(d, interval = "month", standard = FALSE)), "2019-03-07")       
       
  expect_output(print(incidence(d, interval = "year", standard = TRUE)), "2019-01-01")       
  expect_output(print(incidence(d, interval = "year", standard = FALSE)), "2019-03-07")       
       
})

context("Subset of incidence objects")

test_that("[ operator for incidence objects", {
  skip_on_cran()

  dat <- c(1L, 8L, 2L, 9L, 10L, -3L, 4L, 9L, 4L, 3L, 10L, 3L, 6L, 5L, -2L, 9L,
           0L, -3L, 1L, 10L, 9L, 6L, 5L, 10L, 6L, 6L, 4L, 5L, 1L, -1L, 10L, 9L,
           6L, 8L, -3L, 3L, 7L, 0L, 1L, 0L, -2L, 2L, 2L, 2L, -1L, -2L, 0L, 3L,
           0L, 9L)
  # set.seed(123)
  # dat <- as.integer(sample(-3:10, 50, replace = TRUE))
  x <- incidence(dat)
  y <- incidence(dat + as.Date("2016-01-12"), 7L)
  
  z <- incidence(dat + as.Date("2016-01-12"), "epiweek")
  s <- incidence(dat + as.Date("2016-01-12"), "sunday week")
  m <- incidence(dat + as.Date("2016-01-12"), "MMWR week")

  # epiweeks and MMWR weeks start on Sunday
  expect_identical(z[1:2], s[1:2])
  expect_identical(z[], m[])

  x.sub1 <- x[c(3,5,7,8)]
  expect_equal_to_reference(x.sub1, file = "rds/x.sub1.rds")

  x.sub2 <- x[-c(5,1,2)]
  expect_equal_to_reference(x.sub2, file = "rds/x.sub2.rds")

  expect_equal(y[1:2]$weeks, y$weeks[1:2])
  expect_equal(z[1:2]$weeks, z$weeks[1:2])
  
  y.sub1 <- y[1:2]
  expect_equal_to_reference(y.sub1, file = "rds/y.sub1.rds")
})




test_that("subset for incidence objects", {
  skip_on_cran()

  dat <- c(1L, 8L, 2L, 9L, 10L, -3L, 4L, 9L, 4L, 3L, 10L, 3L, 6L, 5L, -2L, 9L,
           0L, -3L, 1L, 10L, 9L, 6L, 5L, 10L, 6L, 6L, 4L, 5L, 1L, -1L, 10L, 9L,
           6L, 8L, -3L, 3L, 7L, 0L, 1L, 0L, -2L, 2L, 2L, 2L, -1L, -2L, 0L, 3L,
           0L, 9L)
  # set.seed(123)
  # dat <- as.integer(sample(-3:10, 50, replace = TRUE))
  x <- incidence(dat)

  x.sub3 <- subset(x, from = 0)
  expect_equal_to_reference(x.sub3, file = "rds/x.sub3.rds")

  x.sub4 <- subset(x, to = 5)
  expect_equal_to_reference(x.sub4, file = "rds/x.sub4.rds")

  x.sub5 <- subset(x, from = 1, to = 4)
  expect_equal_to_reference(x.sub5, file = "rds/x.sub5.rds")

  ## round trip
  expect_identical(x, x[])

  ## corner cases
  expect_error(subset(x, from = 19), "No data retained.")
  expect_error(subset(x, to = -2319), "No data retained.")
  expect_error(subset(x, from = Inf, to = -2319), "No data retained.")
})

test_that("numeric subset works with dates ", {
  skip_on_cran()

  x <-  incidence(as.Date("2001-01-01") + 1:10, 2L)

  expect_equal(subset(x, from = 1, to = 2)$dates,
               x$dates[1:2])

  expect_equal(subset(x, from = -10, to = 2)$dates,
               x$dates[1:2])

  expect_equal(as.data.frame(subset(x, from = 0, to = 1e3)),
               as.data.frame(x))
  expect_identical(x, x[])
})


test_that("numeric subset works with dates and character strings", {
  x <-  incidence(as.Date("2001-01-01") + 1:100, "month")

  expect_equal(subset(x, from = 1, to = -1)$dates,
	       x$dates[1])

  expect_equal(subset(x, from = 1, to = 2)$dates,
               x$dates[1:2])

  expect_equal(subset(x, from = -10, to = 2)$dates,
               x$dates[1:2])


  expect_equal(subset(x, from = 5, to = 5)$dates,
	       x$dates[4])

  expect_equal(as.data.frame(subset(x, from = 0, to = 1e3)),
               as.data.frame(x))
  expect_identical(x, x[])
})

test_that("an erroneous group will give a sensible error", {
  i <- incidence(sample(1:10, 100, replace = TRUE), groups = rep(c("a", "b"), length.out = 100))
  expect_error(i[, "c"], "The following group does not exist: 'c'")
  expect_error(i[, c("grind", "core")], "The following groups do not exist: 'grind', 'core'")
})

#' A cromulence check for first_date and _last date
#'
#' This will create a boundary from the data if there is none provided.
#'
#' @param dates a vector of dates, integers, or numerics
#' @param boundary a date, integer, numeric, or character string that can resolve to a date
#' @param what "first" or "last" for the first_date and last_date arguments
#'
#' @return a date or integer
#' @noRd
#' @keywords internal
check_boundaries <- function(dates, boundary = NULL, what = "first") {
  if (is.null(boundary)) {
    MINMAX <- if (what == "first") min else max
    boundary <- MINMAX(dates, na.rm = TRUE)
  }
  msg <- "%s_date (%s) could not be converted to Date."
  if (is.character(boundary) && !grepl("^[0-9]{4}-[01][0-9]-[0-3][0-9]$", boundary)) {
    msg <- paste(msg, 'Dates must be in ISO 8601 standard format (yyyy-mm-dd).')
    stop(sprintf(msg, what, boundary), call. = FALSE)
  }
  res <- try(check_dates(boundary), silent = TRUE)
  if (inherits(res, "try-error")) {
    msg <- paste(msg, "Accepted formats are:",
                 "\n  Date, POSIXct, integer, numeric, character.")
    stop(sprintf(msg, what, deparse(substitute(boundary))), call. = FALSE)
  }
  res
}

#' @return 
#'   - `get_timespan()`: an `integer` denoting the timespan represented by the 
#'   incidence object.
#' @export
#' @rdname accessors
#' @aliases get_timespan
get_timespan <- function(x) {
  UseMethod("get_timespan")
}

#' @export
#' @rdname accessors
#' @aliases get_timespan.default
get_timespan.default <- function(x) {
    stop(sprintf("Not implemented for class %s",
                 paste(class(x), collapse = ", ")))
}

#' @export
#' @rdname accessors
#' @aliases get_timespan.incidence
get_timespan.incidence <- function(x) {
  x$timespan
}

#' Return TRUE if the interval is a valid date character
#'
#' @param the_interval an interval string
#'
#' @return a logical value
#' @noRd
#' @keywords internal
is_date_interval <- function(the_interval) {
  valid_intervals <- "day|week|month|quarter|year"
  grepl(valid_intervals, the_interval, ignore.case = TRUE)
}

#' Validate potential character values for interval
#'
#' Characters are valid for intervals if they are of the
#' form "day", "week", "month", etc. They can ALSO be
#' valid if they are characters that convert to numbers.
#'
#' @param the_interval a character string of length one
#'
#' @author Zhian Kamvar
#' @return the character string OR a numeric value.
#' @noRd
valid_interval_character <- function(the_interval, standard = TRUE) {
  if (is.character(the_interval)) {
    if (!is_date_interval(the_interval)) {
      suppressWarnings({
        the_interval <- as.numeric(the_interval)
      })
      if (is.na(the_interval)) {
        stop('The interval must be a number or one of the following: "day", "week", "month", "quarter" or "year"', 
             call. = FALSE)
      }
    } else {
      valid_intervals <- "^\\d?\\s?(day|week|month|quarter|year)s?$"
      must_be_standard <- !grepl(valid_intervals, the_interval, ignore.case = TRUE)
      if (!standard && must_be_standard) {
        stop(sprintf("The interval '%s' implies a standard and cannot be used with `standard = FALSE`", the_interval))
      }
    }
  }
  the_interval
}

#' Check to make sure an interval is valid for integer dates
#'
#' This will try to convert the interval if its a character, but complain if
#' it doesn't pass check.
#'
#' @param interval either an integer or character
#'
#' @return interval or it will stop
#' @noRd
#' @keywords internal
valid_interval_integer <- function(interval) {
  if (is.character(interval)) {
    res <- try(valid_interval_character(interval), silent = TRUE)
    if (inherits(res, "try-error")) {
      msg <- sprintf("The interval '%s' is not valid. Please supply an integer.", interval)
      stop(msg, call. = FALSE)
    } else if (is.character(res)) {
      msg <- sprintf("The interval '%s' can only be used for Dates, not integers or numerics.",
                     interval)
      stop(msg, call. = FALSE)
    }
  }
  interval
}

#' Check for a valid week interval
#'
#' @param the_interval character, integer, or numeric
#'
#' @return a logical value indicating if any of the tests pass
#' @noRd
#' @keywords internal
check_week <- function(the_interval) {
  num_week  <- is.numeric(the_interval) && the_interval == 7
  int_week  <- is.integer(the_interval) && the_interval == 7L
  char_week <- is.character(the_interval) && grepl("week", the_interval, ignore.case = TRUE)
  num_week || int_week || char_week
}

#' @export
#' @rdname incidence
#' @param x An 'incidence' object.
print.incidence <- function(x, ...) {
  cat("<incidence object>\n")
  cat(sprintf("[%d cases from days %s to %s]\n",
              sum(x$n), min(x$dates), max(x$dates)))
  if ("weeks" %in% names(x)) {
    type_of_week <- get_type_of_week(x)
    cat(sprintf("[%d cases from %s weeks %s to %s]\n",
                sum(x$n), type_of_week, head(x$weeks, 1), tail(x$weeks, 1)))
  }
  if (!is.null(group_names(x))) {
    groups.txt <- paste(group_names(x), collapse = ", ")
    cat(sprintf("[%d groups: %s]\n", ncol(x), groups.txt))
  }
  cat(sprintf("\n$counts: matrix with %d rows and %d columns\n",
              nrow(x$counts), ncol(x$counts)))
  cat(sprintf("$n: %d cases in total\n", x$n))
  cat(sprintf("$dates: %d dates marking the left-side of bins\n",
              length(x$dates)))
  if (is.integer(x$interval)) {
    cat(sprintf("$interval: %d %s\n",
                x$interval, ifelse(x$interval < 2, "day", "days")))
  } else if (grepl("\\d",  x$interval)) {
    cat(sprintf("$interval: %s\n", x$interval))
  } else {
    cat(sprintf("$interval: 1 %s\n", x$interval))
  }
  cat(sprintf("$timespan: %d days\n", x$timespan))
  if (!is.null(x$cumulative)) {
    cat(sprintf("$cumulative: %s\n", x$cumulative))
  }
  cat("\n")
  invisible(x)
}



#' @export
#' @rdname fit
#' @param ... currently unused.
print.incidence_fit <- function(x, ...) {

  cat("<incidence_fit object>\n\n")
  cat("$model: regression of log-incidence over time\n\n")

  cat("$info: list containing the following items:\n")
  cat("  $r (daily growth rate):\n")
  print(x$info$r)
  cat("\n  $r.conf (confidence interval):\n")
  print(x$info$r.conf)
  if (x$info$r[1] > 0) {
    cat("\n  $doubling (doubling time in days):\n")
    print(x$info$doubling)
    cat("\n  $doubling.conf (confidence interval):\n")
    print(x$info$doubling.conf)
  } else {
    cat("\n  $halving (halving time in days):\n")
    print(x$info$halving)
    cat("\n  $halving.conf (confidence interval):\n")
    print(x$info$halving.conf)
  }

  cat(sprintf(
    "\n  $pred: data.frame of incidence predictions (%d rows, %d columns)\n",
    nrow(x$info$pred), ncol(x$info$pred)))
  invisible(x)
}

#' @export
#' @rdname fit
print.incidence_fit_list <- function(x, ...) {
  cat("<list of incidence_fit objects>\n\n")
  cat("attr(x, 'locations'): list of vectors with the locations of each incidence_fit object\n\n")
  locations <- attr(x, "locations")
  cat(sprintf("'%s'", vapply(locations, paste, character(1), collapse = "', '")), sep = "\n")
  cat("\n")
  cat("$model: regression of log-incidence over time\n\n")

  cat("$info: list containing the following items:\n")
  cat("  $r (daily growth rate):\n")
  print(get_info(x, "r"))
  cat("\n  $r.conf (confidence interval):\n")
  print(get_info(x, "r.conf"))
  if (any(get_info(x, "r") > 0)) {
    cat("\n  $doubling (doubling time in days):\n")
    print(get_info(x, "doubling", na.rm = TRUE))
    cat("\n  $doubling.conf (confidence interval):\n")
    print(get_info(x, "doubling.conf", na.rm = TRUE))
  }
  if (any(get_info(x, "r") < 0)) {
    cat("\n  $halving (halving time in days):\n")
    print(get_info(x, "halving", na.rm = TRUE))
    cat("\n  $halving.conf (confidence interval):\n")
    print(get_info(x, "halving.conf", na.rm = TRUE))
  }
  preds <- get_info(x, "pred")
  cat(sprintf(
    "\n  $pred: data.frame of incidence predictions (%d rows, %d columns)\n",
    nrow(preds), ncol(preds)))
  invisible(x)
}

#' @param ... arguments passed to [ggplot2::scale_x_date()],
#' [ggplot2::scale_x_datetime()], or [ggplot2::scale_x_continuous()], depending
#' on how the `$date` element is stored in the incidence object.
#' @export
#' @rdname plot.incidence
scale_x_incidence <- function(x, n_breaks = 6, labels_week = TRUE, ...) {

  breaks <- make_breaks(x, n_breaks, labels_week)


  if (inherits(x$dates, "Date")) {
    
    out <- ggplot2::scale_x_date(breaks = breaks$breaks, 
                                 labels = breaks$labels,
                                 ...)
  } else if (inherits(x$dates, "POSIXt")) {
    breaks$breaks <- as.POSIXct(as.POSIXlt(breaks$breaks))
    out <- ggplot2::scale_x_datetime(breaks   = breaks$breaks,
                                     labels   = breaks$labels,
                                     timezone = "UTC",
                                     ...
                                     )
  } else {
    out <- ggplot2::scale_x_continuous(breaks = breaks$breaks, ...)
  }
  out
}

#' @export
#' @rdname plot.incidence
make_breaks <- function(x, n_breaks = 6L, labels_week = TRUE) {
  stopifnot(inherits(x, "incidence"), is.logical(labels_week), is.numeric(n_breaks))
  ## Defining breaks for the x axis --------------------------------------------
  ##
  ## The x axis can either be integers, Dates, or POSIXt scales. Moreover,
  ## we need to make sure that the breaks align with the left-hand side of the
  ## bins (for now). This section first defines what the breaks should be 
  ## and then treats them according to whether or not the interval was specified
  ## as a character. 
  if (n_breaks == nrow(x)) {
    # The number of breaks are equal to the number of dates... don't worry about
    # adjusting
    breaks <- x$dates  
  } else {
    # adjust breaks to force first date to beginning.
    breaks <- pretty(x$dates, n_breaks)
    breaks <- breaks + (x$dates[1] - breaks[1])
  }
  ## Defining the x axis scale -------------------------------------------------
  ## 
  ## Choosing between scale_x_date, scale_x_datetime, and scale_x_continuous

  # labels should be dates or numbers
  if (is.character(x$interval)) {
    # The interval is a character like "2 weeks" and we have to figure out how
    # to split these manually
    has_number <- grepl("\\d", x$interval)
    tims       <- ceiling(x$timespan/(n_breaks*mean(get_interval(x, integer = TRUE))))
    if (has_number) {
      ni <- as.integer(strsplit(x$interval, " ", fixed = TRUE)[[1L]][1L])
      # the replacement should be a multiple of the number
      replacement <- if (tims <= ni) ni else ceiling(tims/ni)*ni 
      db <- gsub("\\d+", replacement, x$interval)
    } else if (x$interval == "quarter") { 
      db <- paste(tims * 3, "months")
    } else {
      db <- sprintf("%d %s", tims, x$interval)
    }
    breaks <- seq(x$dates[1], x$dates[nrow(x)], by = db)
  }

  if (!is.null(x$weeks)) {
    # If the data are in weeks, we should make sure that the line up correctly
    w <- aweek::date2week(breaks, 
                          week_start = attr(x$weeks, "week_start"), 
                          floor_day = TRUE)
    breaks <- aweek::week2date(w)
    labels <- if (labels_week) w else ggplot2::waiver() 
  } else {
    labels <- ggplot2::waiver()
  } 
  list(breaks = breaks, labels = labels)
}

#' Get counts from an incidence object
#' 
#' @param x an `incidence` object.
#' @param groups if there are groups, use this to specify a group or groups to 
#'   subset. Defaults to `NULL` indicating that all groups are returned.
#' 
#' @return a matrix of counts where each row represents a date bin
#' @export
#' @examples
#' if (require(outbreaks)) { withAutoprint({
#'   dat  <- ebola_sim$linelist$date_of_onset
#'   gend <- ebola_sim$linelist$gender
#'   i    <- incidence(dat, interval = "week", groups = gend)
#'   
#'   ## Use with an object and no arguments gives the counts matrix
#'   head(get_counts(i))
#'
#'   ## Specifying a position or group name will return a matrix subset to that
#'   ## group
#'   head(get_counts(i, 1L))
#'   head(get_counts(i, "f"))
#'
#'   ## Specifying multiple groups allows you to rearrange columns
#'   head(get_counts(i, c("m", "f")))
#'
#'   ## If you want a vector, you can use drop
#'   drop(get_counts(i, "f"))
#' })}
get_counts <- function(x, groups = NULL) {
  UseMethod("get_counts")
}

#' @rdname get_counts
#' @export
get_counts.incidence <- function(x, groups = NULL){
  if (is.null(groups) || ncol(x$counts) == 1) {
    return(x$counts)
  }
  if (is.character(groups)) {
    correct_groups <- groups[groups %in% colnames(x$counts)]
  }
  if (is.numeric(groups)) {
    correct_groups <- groups[groups %in% seq(ncol(x$counts))]
  }
  if (!identical(correct_groups, groups)) {
    grps <- paste(setdiff(groups, correct_groups), collapse = ", ")
    msg  <- sprintf("The following groups were not recognised: %s", grps)    
    message(msg)
  } 
  if (length(correct_groups) == 0) {
    grps <- paste(colnames(x$counts), collapse = ", ")
    stop(sprintf("No groups matched those present in the data: %s", grps))
  }
  return(x$counts[, correct_groups, drop = FALSE])
}

#' extract and set group names
#' @param x an [incidence()] object.
#' @param value character vector used to rename groups
#' @return an integer indicating the number of groups present in the incidence
#'   object.
#' @details This accessor will return a  
#' @export
#' @examples
#' i <- incidence(dates = sample(10, 100, replace = TRUE), 
#'                interval = 1L,
#'                groups = sample(letters[1:3], 100, replace = TRUE))
#' i
#' group_names(i)
#'
#' # change the names of the groups
#' group_names(i) <- c("Group 1", "Group 2", "Group 3")
#' i
#'
#' # example if there are mistakes in the original data, e.g. 
#' # something is misspelled
#' set.seed(50)
#' grps <- sample(c("child", "adult", "adlut"), 100, replace = TRUE, prob = c(0.45, 0.45, 0.05))
#' i <- incidence(dates = sample(10, 100, replace = TRUE), 
#'                interval = 1L,
#'                groups = grps)
#' colSums(get_counts(i))
#' 
#' # If you change the name of the mis-spelled group, it will be merged with the
#' # correctly-spelled group
#' gname <- group_names(i)
#' gname[gname == "adlut"] <- "adult"
#' # without side-effects
#' print(ii <- group_names(i, gname))
#' colSums(get_counts(i))  # original still has three groups
#' colSums(get_counts(ii))
#' # with side-effects
#' group_names(i) <- gname
#' colSums(get_counts(i))
group_names <- function(x, value) {
	UseMethod("group_names", x)
}

#' @export
#' @rdname group_names
"group_names<-" <- function(x, value) {
	UseMethod("group_names<-", x)
}

#' @rdname group_names
#' @export
#' @aliases group_names.default
group_names.default <- function(x, value) {
    stop(sprintf("Not implemented for class %s",
                 paste(class(x), collapse = ", ")))
}
#' @rdname group_names
#' @export
#' @aliases `group_names<-`.default
"group_names<-.default" <- function(x, value) {
    stop(sprintf("Not implemented for class %s",
                 paste(class(x), collapse = ", ")))
}

#' @rdname group_names
#' @export
#' @keywords accessors
group_names.incidence <- function(x, value = NULL){
	if (is.null(value)) {
		colnames(x$counts)
	} else {
		`group_names<-`(x, value)
	}
}

#' @rdname group_names
#' @export
"group_names<-.incidence" <- function(x, value) {
	if (length(value) != ncol(x)) {
		stop("value must be the same length as the number of groups.")
	}
	if (anyNA(value <- as.character(value))) {
		stop("value must be able to be coerced to a character vector")
	}
	uval <- unique(value)
	if (identical(uval, value)) {
		colnames(x$counts) <- value
	} else {
		the_counts <- x$counts
		out <- matrix(integer(nrow(the_counts)*length(uval)),
			      nrow = nrow(the_counts),
			      ncol = length(uval)
			     )
		colnames(out) <- uval
		for (i in uval) {
			out[, i] <- rowSums(the_counts[, value == i, drop = FALSE])
		}
		x$counts <- out
	}
	x
}

#' @return 
#'   - `get_n()` The total number of cases stored in the object
#' @export
#' @rdname accessors
#' @aliases get_n
get_n <- function(x) {
  UseMethod("get_n")
}

#' @export
#' @rdname accessors
#' @aliases get_n.default
get_n.default <- function(x) {
    stop(sprintf("Not implemented for class %s",
                 paste(class(x), collapse = ", ")))
}

#' @export
#' @rdname accessors
#' @aliases get_n.incidence
get_n.incidence <- function(x) {
  x$n
}

#' Accessors for `incidence_fit` objects
#' 
#' @param x an `incidence_fit` or `incidence_fit_list`
#'   object.
#' @return a list of `incidence_fit` objects. 
#' @export
#' @examples
#' 
#' if (require(outbreaks)) { withAutoprint({
#' 
#'  dat <- ebola_sim$linelist$date_of_onset
#'
#'  ## EXAMPLE WITH A SINGLE MODEL
#'
#'  ## compute weekly incidence
#'  sex <- ebola_sim$linelist$gender
#'  i.sex <- incidence(dat, interval = 7, group = sex)
#'  
#'  ## Compute the optimal split for each group separately
#'  fits  <- fit_optim_split(i.sex, separate_split = TRUE)
#'
#'  ## `fits` contains an `incidence_fit_list` object
#'  fits$fit
#'  
#'  ## Grab the list of `incidence_fit` objects
#'  get_fit(fits$fit)
#'  
#'  ## Get the predictions for all groups
#'  get_info(fits$fit, "pred", groups = 1)
#'  
#'  ## Get the predictions, but set `groups` to "before" and "after"
#'  get_info(fits$fit, "pred", groups = 2)
#'  
#'  ## Get the reproduction number
#'  get_info(fits$fit, "r")
#'
#'  ## Get the doubling confidence interval
#'  get_info(fits$fit, "doubling.conf")
#'
#'  ## Get the halving confidence interval
#'  get_info(fits$fit, "halving.conf")
#' })}
get_fit <- function(x) {
  UseMethod("get_fit")
}

#' @rdname get_fit
#' @export
get_fit.incidence_fit <- function(x) {
  x
}

#' @rdname get_fit
#' @export
get_fit.incidence_fit_list <- function(x) {
  locations <- attr(x, "locations")
  res <- lapply(locations, function(i) x[[i]])
  names(res) <- vapply(locations, paste, character(1), collapse = "_")
  res
}

##' Default internal constructor for incidence objects.
##'
##'
##'
##' @param dates A vector of dates, which can be provided as objects of the
##' class: integer, numeric, Date, POSIXct. Note that decimal numbers will be
##' floored with a warning.
##'
##' @param interval An integer indicating the (fixed) size of the time interval
##' used for computing the incidence; defaults to 1 day.
##'
##' @param groups An optional factor defining groups of observations for which
##' incidence should be computed separately.
##'
##' @param na_as_group A logical value indicating if missing group (NA) should be
##' treated as a separate group.
##'
##' @param last_date The last date to be included in the produced epicurve. If
##'   `NULL` (default), the last date will be the most recent provided in
##'   `dates`.
##'
##' @param ... Additional arguments passed to other methods (none are used).
##'
##' @author Zhian Kamvar
##' @return an incidence object
##' @noRd
make_incidence <- function(dates, interval = 1L, groups = NULL,
                           na_as_group = TRUE, first_date = NULL,
                           last_date = NULL, ...) {
  dots     <- list(...)

  ## make sure input can be used
  dates    <- check_dates(dates)
  interval <- check_interval(interval, if (is.null(dots$standard)) TRUE else dots$standard)
  groups   <- check_groups(groups, dates, na_as_group)

  ## Check the interval and arrange the breaks
  first_date <- check_boundaries(dates, first_date, "first")
  last_date  <- check_boundaries(dates, last_date, "last")
  breaks     <- make_breaks_easier(dates,
                                   the_interval    = interval,
                                   first_date      = first_date,
                                   last_date       = last_date,
                                   dots            = dots
                                   )
  if (!is.numeric(interval) && grepl("week", interval)) {
    interval <- get_week_duration(interval)
  }

  ## Trim the dates and groups as necessary
  trimmed <- trim_observations(dates, first_date, last_date)
  dates   <- dates[trimmed]
  groups  <- groups[trimmed]

  ## compute counts within bins defined by the breaks
  if (!is.null(groups)) {
    counts <- tapply(dates, groups, count.dates, breaks)
    counts <- matrix(as.integer(unlist(counts, use.names = FALSE)),
                     ncol = length(levels(groups)))
    colnames(counts) <- levels(groups)
  } else {
    counts <- count.dates(dates, breaks)
    counts <- matrix(as.integer(counts), ncol = 1L)
  }

  out <- list(dates      = breaks,      # left side of bins (incl left, excl right)
              counts     = counts,      # computed incidence, 1 col / group
              timespan   = diff(range(breaks, na.rm = TRUE)) + 1,
              interval   = interval,    # fixed bin size
              n          = sum(counts), # total number of cases
              cumulative = FALSE)       # not cumulative at creation
  class(out) <- "incidence"
  out
}

#' translate user input to the start date of the week
#' 
#' @param a weekday specification: ISOweek, MMWRweek, EPIweek, Mon-week, Tue-week, etc. 
#'
#' @return the corresponding weekday
#' @keywords internal
#' @noRd
#' @examples
#' get_week_start("ISOweek")
#' get_week_start("MMWRweek")
#' get_week_start("EPIweek")
#'
#' # weeks that start on saturday
#' 
#' get_week_start("Sat-week")
#' get_week_start("week: Saturday")
#' get_week_start("2 weeks: Saturday")
#' get_week_start("epiweek: Saturday")
get_week_start <- function(weekday) {
  wkdy <- gsub("weeks?", "", tolower(weekday))
  wkdy <- gsub('[[:punct:][:blank:][:digit:]]*', "", wkdy)
  wkdy <- if (wkdy == "") "monday" else wkdy # the input was "weeks"
  res <- switch(wkdy, 
                "mmwr" = "sunday", # MMWR == CDC epiweek
                "epi"  = "sunday", # CDC epiweek
                "iso"  = "monday", # ISOweek == WHO epiweek
                wkdy # all others
                )
  gsub("epi", "", res) # if they specify something like "epiweek:saturday"
}

#' Translate a custom interval to a valid interval
#'
#' @param the_interval an interval like 2 epiweeks or 1 ISOweek
#' @return an interval compatible with `seq.Date()`
#' @keywords internal
#' @noRd
#' @examples
#' get_week_duration("2 weeks (wednesday)") # 2 weeks
#' get_week_duration("2 epiweeks") # 2 weeks
get_week_duration <- function(the_interval) {

  if (the_interval == 7) return(the_interval)
  res <- gsub('^(\\d*) ?.*(weeks?).*$', '\\1 \\2', tolower(the_interval), perl = TRUE)
  trimws(res)

}

get_type_of_week <- function(x) {

  switch(as.character(attr(x$weeks, "week_start")),
         "1" = "ISO",
         "7" = "MMWR",
         sprintf("(%s)", weekdays(x$dates[1]))
         )
}

## These functions are meant for internal use only, and are not
## exported. Functions which check content return it, after potential trivial
## conversions.

#' Count dates within bins
#'
#' @param dates a vector of dates, integers, or numerics
#' @param breaks an ordered vector of dates or integers
#'
#' @author Thibaut Jombart
#' @return an integer vector of the number of incidences per date
#' @noRd
#'
count.dates <- function(dates, breaks){
  counts <- table(cut(as.integer(dates), breaks = c(breaks, Inf), right = FALSE))
  as.integer(counts)
}


## Implement isTRUE and isFALSE to avoid dep on R 3.5.0

isFALSE <- function(x) {
  is.logical(x) && length(x) == 1L && !is.na(x) && !x
}

isTRUE <- function(x) {
  is.logical(x) && length(x) == 1L && !is.na(x) && x
}

## A fix for the nonesensical behaviour of `sample` when first argument is of
## length 1.

sample_ <- function(x, ...) {
  x[sample.int(length(x), ...)]
}

## quantiles for Date objects

quantile_Date <- function(x, ...) {
  if (!inherits(x, "Date")) {
    stop("'x' is not a 'Date' object")
  }

  first_date <- min(x, na.rm = TRUE)
  x_num <- as.numeric(x - min(x))
  out <- stats::quantile(x_num, ...)
  first_date + out
}

#' Check date cromulence
#'
#' This function checks that usable dates are provided, and set non-finite
#' values to NA. It also makes a few trivial conversions on the fly.
#'
#' @param x a vector that represents dates. Can be in almost any format
#' @param error_on_NA a logical specifing whether or not an error should be
#'   thrown if NAs are present in the dates. Defaults to FALSE.
#' @return an object in either integer, Date, or POSIXt
#' @noRd
check_dates <- function(x, error_on_NA = FALSE, ...) {

  if (is.null(x)) {
    stop("dates is NULL", call. = FALSE)
  }

  if (is.character(x)) {
    x <- as.Date(x, ...)
  }

  not_finite <- !is.finite(x)
  if (sum(not_finite) > 0) {
    x[not_finite] <- NA
  }

  if (any(is.na(x)) && error_on_NA) {
    msg <- "NA detected in the dates"
    stop(msg, call. = FALSE)
  }

  if (sum(!is.na(x)) < 1) {
    stop("At least one (non-NA) date must be provided", call. = FALSE)
  }

  if (inherits(x, "Date")) {
    check_timespan(x)
    return(x)
  }

  if (inherits(x, "POSIXt")) {
    check_timespan(x)
    return(x)
  }

  if (is.integer(x)) {
    return(x)
  }

  if (is.numeric(x)) {
    x_ori <- x
    x <- as.integer(floor(x))
    if (!isTRUE(note <- all.equal(x, x_ori))) {
      msg <- paste0(
        "Flooring from non-integer date caused approximations:\n",
        note)
      warning(msg, call. = FALSE)
    }
    return(x)
  }


  formats <- c("Date", "POSIXct", "integer", "numeric", "character")
  msg <- paste0(
    "Input could not be converted to date. Accepted formats are:\n",
    paste(formats, collapse = ", "))
  stop(msg)

}

#' @rdname get_fit
#' @param what the name of the item in the "info" element of the `incidence_fit`
#'   object. 
#' @param groups if `what = "pred"` and `x` is an `incidence_fit_list` object, 
#'   then this indicates what part of the nesting hierarchy becomes the column
#'   named "groups". Defaults to `NULL`, indicating that no groups column will
#'   be added/modified.
#' @param na.rm when `TRUE` (default), missing values will be excluded from the 
#'   results.
#' @param ... currently unused.
#' @export
get_info <- function(x, what = "r", ...) {
  UseMethod("get_info")
}

#' @rdname get_fit
#' @export
get_info.incidence_fit <- function(x, what = "r", ...) {
  x$info[[what]]
}

#' @rdname get_fit
#' @export
get_info.incidence_fit_list <- function(x, what = "r", groups = NULL, na.rm = TRUE, ...) {
  locations <- attr(x, "locations")
  n <- length(locations)
  if (what == "pred") {
    fits <- get_fit(x)
    for (i in names(fits)) {
      fits[[i]] <- fits[[i]]$info$pred
      fits[[i]]$location <- i
      if (!is.null(groups)) {
      	tmp              <- strsplit(i, "_")[[1]][[groups]]
        fits[[i]]$groups <- factor(tmp, tmp)
      }
    }
    res <- do.call("rbind", fits)
    return(res)
  }
  is_matrix <- grepl("conf", what)
  the_names <- vapply(locations, paste, character(1), collapse = "_")
  need_col_names <- TRUE
  if (is_matrix) {
    res <- matrix(0.0, nrow = n, ncol = 2L)
  } else {
    res <- numeric(n)
  }
  for (i in seq_len(n)) {
    tmp <- x[[locations[[i]]]]$info[[what]]
    tmp <- if (is.null(tmp)) NA_real_ else tmp
    if (is_matrix) {
      if (need_col_names && all(!is.na(tmp))) {
        colnames(res) <- colnames(tmp)
        need_col_names <- FALSE
      }
      res[i, ] <- tmp
    } else {
      res[[i]] <- tmp
    }
  }
  if (is_matrix) {
    rownames(res) <- the_names
  } else {
    names(res)    <- the_names
  }
  if (na.rm) {
    nonas <- stats::complete.cases(res)
    res <- if (is_matrix) res[nonas, , drop = FALSE] else res[nonas]
  }
  res
}

#' Conversion of incidence objects
#'
#' These functions convert `incidence` objects into other classes.
#'
#' @rdname conversions
#'
#' @author Thibaut Jombart \email{thibautjombart@@gmail.com}, Rich Fitzjohn
#'
#' @importFrom stats as.ts
#'
#' @export
#'
#' @param x An `incidence` object, or an object to be converted as
#'   `incidence` (see details).
#'
#' @param ... Further arguments passed to other functions (no used).
#'
#' @param long A logical indicating if the output data.frame should be 'long', i.e. where a single
#' column containing 'groups' is added in case of data computed on several groups.
#'
#'
#' @export
#'
#'
#' @seealso the [incidence()] function to generate the 'incidence' objects.
#'
#'
#' @details Conversion to `incidence` objects should only be done when the
#'   original dates are not available. In such case, the argument `x`
#'   should be a matrix corresponding to the `$counts` element of an
#'   `incidence` object, i.e. giving counts with time intervals in rows and
#'   named groups in columns. In the absence of groups, a single unnamed columns
#'   should be given. `data.frame` and vectors will be coerced to a matrix.
#'
#'
#' @examples
#' ## create fake data
#' data <- c(0,1,1,2,1,3,4,5,5,5,5,4,4,26,6,7,9)
#' sex <- sample(c("m","f"), length(data), replace=TRUE)
#'
#' ## get incidence per group (sex)
#' i <- incidence(data, groups = sex)
#' i
#' plot(i)
#'
#' ## convert to data.frame
#' as.data.frame(i)
#'
#' ## same, 'long format'
#' as.data.frame(i, long = TRUE)
#'
#'
#'
#' ## conversion from a matrix of counts to an incidence object
#' i$counts
#' new_i <- as.incidence(i$counts, i$dates)
#' new_i
#' all.equal(i, new_i)
#'

as.data.frame.incidence <- function(x, ..., long = FALSE){
    counts  <- x$counts
    gnames  <- group_names(x)
    unnamed <- is.null(gnames) && ncol(counts) == 1L
    if (unnamed) {
        colnames(counts) <- "counts"
    }

    ws <- attr(x$weeks, "week_start")
    if ("weeks" %in% names(x)) {
      out <- data.frame(dates = x$dates,
                        weeks = as.character(x$weeks),
                        isoweeks = as.character(x$weeks),
                        counts,
                        check.names = FALSE, stringsAsFactors = TRUE)
      out$weeks <- aweek::date2week(out$dates, ws, floor_day = TRUE, factor = TRUE)
    } else {
      out <- data.frame(dates = x$dates, counts, check.names = FALSE, stringsAsFactors = TRUE)
    }

    ## handle the long format here
    if (long && !unnamed) {
        groups <- factor(rep(gnames, each = nrow(out)), levels = gnames)
        counts <- as.vector(x$counts)
        if ("weeks" %in% names(x)) {
          out <- data.frame(dates = out$dates,
                            weeks = as.character(out$weeks),
                            isoweeks = out$isoweeks,
                            counts = counts,
                            groups = groups,
                            check.names = FALSE, stringsAsFactors = TRUE)
          out$weeks <- aweek::date2week(out$dates, ws, floor_day = TRUE, factor = TRUE)
        } else {
          out <- data.frame(dates = out$dates,
                            counts = counts,
                            groups = groups,
                            check.names = FALSE, stringsAsFactors = TRUE)
        }
    }
    if (all(names(x) != "isoweeks")) out$isoweeks <- NULL
    out
}






## Conversion to 'incidence' class can be handy to plot and handle data for
## which incidence has already been computed. To ensure that the ouput is a
## correct object, we use the 'incidence' function on fake data that match the
## counts inputs. This avoids potential issues such as non-regular intervals
## (the first time interval is used for the entire data.

#' @export
#' @rdname conversions

as.incidence <- function(x, ...) {
  UseMethod("as.incidence", x)
}






#' @export
#'
#' @rdname conversions
#'
#' @param dates A vector of dates, each corresponding to the (inclusive) lower
#' limit of the bins.
#'
#' @param interval An integer indicating the time interval used in the
#'   computation of the incidence. If NULL, it will be determined from the first
#'   time interval between provided dates. If only one date is provided, it will
#'   trigger an error.
#'
#' @param standard A logical indicating whether standardised dates should be
#'   used. Defaults to `TRUE`.
#'
#' @param isoweeks Deprecated. Use standard.
#'

as.incidence.matrix <- function(x, dates = NULL, interval = NULL,
                                standard = TRUE, isoweeks = standard, ...) {

  if (is.null(dates)) {
    if (!is.null(interval)) {
      interval <- check_interval(interval)
      dates <- seq(1, length = nrow(x), by = interval)
    } else{
      dates <- seq(1, length = nrow(x), by = 1L)
    }
  }

  dates <- check_dates(dates, error_on_NA = TRUE)
  last_date <- max(dates)

  ## determine interval

  if (is.null(interval)) {
    if (length(dates) < 2L) {
      msg <- "Interval needs to be specified if there is only one date."
      stop(msg)
    } else {
      interval <- as.integer(diff(dates[1:2]))
    }
  } else {
    interval <- check_interval(interval)
  }


  ## generate fake dates

  x_vector <- as.vector(x)
  fake_dates <- rep(rep(dates, ncol(x)), x_vector)


  ## determine groups

  if (ncol(x) > 1L) {
    x_groups <- colnames(x)
    if (is.null(x_groups)) {
      msg <- "Columns should be named to label groups."
      stop(msg)
    }
    group_sizes <- colSums(x)
    fake_groups <- rep(x_groups, group_sizes)
  } else {
    fake_groups <- NULL
  }

  if (inherits(fake_dates, c("Date", "POSIXt"))) {
    incidence(fake_dates,
              interval = interval,
              groups = fake_groups,
              standard = isoweeks,
              last_date = last_date)
  } else {
    incidence(fake_dates,
            interval = interval,
            groups = fake_groups,
            last_date = last_date)
  }
}






#' @export
#'
#' @rdname conversions

as.incidence.data.frame <- function(x, dates = NULL, interval = NULL,
                                    isoweeks = TRUE, ...) {
  as.incidence(as.matrix(x), dates, interval, isoweeks, ...)
}






#' @export
#'
#' @rdname conversions

as.incidence.numeric <- function(x, dates = NULL, interval = NULL,
                                 isoweeks = TRUE, ...) {
  as.incidence(as.matrix(x), dates, interval, isoweeks, ...)
}

#' Estimate the peak date of an incidence curve using bootstrap
#'
#' This function can be used to estimate the peak of an epidemic curve stored as
#' `incidence`, using bootstrap. See [incidence::bootstrap] for more information
#' on the resampling.
#'
#' @author Thibaut Jombart \email{thibautjombart@@gmail.com}, with inputs on
#'   caveats from Michael Höhle.
#'
#' @md
#'
#' @export
#'
#' @details Input dates are resampled with replacement to form bootstrapped
#'   datasets; the peak is reported for each, resulting in a distribution of
#'   peak times. When there are ties for peak incidence, only the first date is
#'   reported.
#'
#' Note that the bootstrapping approach used for estimating the peak time makes
#' the following assumptions:
#'
#' - the total number of event is known (no uncertainty on total incidence)
#' - dates with no events (zero incidence) will never be in bootstrapped datasets
#' - the reporting is assumed to be constant over time, i.e. every case is
#' equally likely to be reported
#'
#' @param x An `incidence` object.
#'
#' @param n The number of bootstrap datasets to be generated; defaults to 100.
#'
#' @param alpha The type 1 error chosen for the confidence interval; defaults to
#'   0.05.
#'
#' @return A list containing the following items:
#'
#' - `observed`: the peak incidence of the original dataset
#' - `estimated`: the mean peak time of the bootstrap datasets
#' - `ci`: the confidence interval based on bootstrap datasets
#' - `peaks`: the peak times of the bootstrap datasets
#'
#' @seealso [incidence::bootstrap] for the bootstrapping underlying this
#'   approach and [incidence::find_peak] to find the peak in a single
#'   `incidence` object.
#'
#' @examples
#'
#' if (require(outbreaks) && require(ggplot2)) { withAutoprint({
#'   i <- incidence(fluH7N9_china_2013$date_of_onset)
#'   i
#'   plot(i)
#'
#'   ## one simple bootstrap
#'   x <- bootstrap(i)
#'   x
#'   plot(x)
#'
#'   ## find 95% CI for peak time using bootstrap
#'   peak_data <- estimate_peak(i)
#'   peak_data
#'   summary(peak_data$peaks)
#'
#'   ## show confidence interval
#'   plot(i) + geom_vline(xintercept = peak_data$ci, col = "red", lty = 2)
#'
#'   ## show the distribution of bootstrapped peaks
#'   df <- data.frame(peak = peak_data$peaks)
#'   plot(i) + geom_density(data = df,
#'                          aes(x = peak, y = 10 * ..scaled..),
#'                          alpha = .2, fill = "red", color = "red")
#'
#' })}
#'

estimate_peak <- function(x, n = 100, alpha = 0.05) {
  if (!inherits(x, "incidence")) {
    stop("x is not an incidence object")
  }

  if (ncol(x$counts) > 1L) {
    msg <- paste("'x' is stratified by groups",
                 "pooling groups before finding peaks",
                 sep = "\n")
    message(msg)
    x <- pool(x)
  }

  out <- list()

  ## use it to find CI for epidemic peak
  out$observed <- find_peak(x)

  ## peaks on 'n' bootstrap samples
  peak_boot <- replicate(n,
                         find_peak(bootstrap(x)),
                         simplify = FALSE)

  ## convert to vector without losing Date class
  peak_boot <- do.call(c, peak_boot)

  ## store relevant stats and sod off
  out$estimated <- mean(peak_boot)
  QUANTILE <- if(inherits(peak_boot, c("Date", "POSIX"))) quantile_Date else stats::quantile
  out$ci <- QUANTILE(peak_boot, c(alpha / 2, 1 - alpha / 2))
  out$peaks <- peak_boot
  out
}

#' Trim observations based on the first and last dates
#'
#' @param observations a vector of dates or integers
#' @param first_date a single date or integer
#' @param last_date a single date or integer
#'
#' @return the trimmed observations as a logical vector
#' @noRd
#' @keywords internal
trim_observations <- function(dates, first_date = NULL, last_date = NULL) {
  # Remove the missing observations -------------------- 
  res <- !is.na(dates)
  if (sum(res) < length(dates)) {
    message(sprintf("%d missing observations were removed.",
                    length(dates) - sum(res)
                    )
    )
  }
  dates <- dates[res]
  # Trim ends ------------------------------------------ 
  res   <- dates >= first_date & dates <= last_date
  if (sum(res) < length(dates)) {
    message(sprintf("%d observations outside of [%s, %s] were removed.",
                    length(dates) - sum(res),
                    format(first_date),
                    format(last_date)
                    )
            )
  }
  res
}

check_timespan <- function(x) {
  max_days <- as.difftime(getOption("incidence.max.days"), units = "days")
  my_range <- range(x, na.rm = TRUE)
  if (diff(my_range) > max_days) {
    msg <- paste("The data has a date range of greater than %d days [%s to %s].",
                 "Please check your data to ensure this is accurate.\n",
                 "To remove this warning, set the `incidence.max.days` option",
                 "to Inf:\n\n\toptions(incidence.max.days = Inf)"
                 )
    msg <- sprintf(msg, 
                   max_days, 
                   as.character(my_range[1]), 
                   as.character(my_range[2]))
    warning(msg, immediate. = TRUE, call. = FALSE)
  }
  invisible()
}

#' Make breaks with dates
#'
#' Because Date objects have a specific `seq` method, it's possible
#' to make breaks with both integers and date objects. This function
#' will check to make sure that the interval is valid.
#'
#' @param date an integer, numeric, or Date vector
#' @param the_interval an integer or character
#' @param last_date an integer, numeric, or Date
#' @param first_date an integer, numeric, or Date
#' @param dots a named list of options
#'
#' @author Zhian Kamvar
#' @return a vector of integers or Dates
#' @noRd
#' @examples
#'
#' set.seed(999)
#' d <- sample(10, replace = TRUE)
#' make_breaks_easier(d, 2L)
make_breaks_easier <- function(dates, the_interval, first_date = NULL,
                               last_date = NULL, dots = 1L) {

  the_interval    <- valid_interval_character(the_interval)
  date_interval   <- is.character(the_interval) && is_date_interval(the_interval)
  is_month        <- grepl("month", the_interval, ignore.case = TRUE)
  is_quarter      <- grepl("quarter", the_interval, ignore.case = TRUE)
  is_year         <- grepl("year", the_interval, ignore.case = TRUE)
  uneven_interval <- date_interval && (is_month || is_quarter || is_year)

  # getting information about the date
  fd        <- as.character(first_date)
  the_day   <- as.integer(substr(fd, 9, 10))
  the_month <- as.integer(substr(fd, 6, 7))

  if ("standard" %in% names(dots)) {
    if (isTRUE(dots$standard)) {
      is_a_week <- !uneven_interval && check_week(the_interval)
      if (is_a_week) {
        week_start    <- get_week_start(the_interval)
        the_interval  <- get_week_duration(the_interval)
        # This returns something like 2018-W29
        first_isoweek <- aweek::date2week(first_date, week_start, floor_day = TRUE)
        # here we convert it back to a date
        first_date    <- aweek::week2date(first_isoweek)
      }
      if (uneven_interval) {
        # Replace the day with the first day of the month
        substr(fd, 9, 10) <- "01"
        if (is_quarter) {
          # Replace the month with the first month of the quarter
          m <- (as.integer(substr(fd, 6, 7)) - 1L) %/% 3L
          substr(fd, 6, 7) <- sprintf("%02d", (m * 3) + 1L)
        }
        if (is_year) {
          # Replace the month with the first month of the year
          substr(fd, 6, 7) <- "01"
        }
        # re-cast the date
        first_date <- as.Date(fd)
      }
    } else {
      if (uneven_interval && !is_year && the_day > 28) {
        # The first date represents a day that doesn't occur in all months
        msg <- paste("The first_date (%s) represents a day that does not",
                     "occur in all months. Because of this, bins may not",
                     "conform to monthly boundaries. To prevent this",
                     "behavior, plese specify a different first_date that",
                     "represents a day within [1, 28]."
                     )
        msg <- paste(strwrap(msg), collapse = "\n")
        warning(sprintf(msg, fd), call. = FALSE)
      }
      if (is_year && the_month == 2 && the_day == 29) {
        # The first date occurs on a leap day.
        msg <- paste("The first_date (%s) represents a day that does not",
                     "occur in all years. Because of this, bins may not",
                     "fall on the same day. To prevent this behavior, please",
                     "specify a first_date that represents a different day."
                     )
        msg <- paste(strwrap(msg), collapse = "\n")
        warning(sprintf(msg, fd), call. = FALSE)
      }
    }
  }
  seq(first_date, last_date, by = the_interval)
}

#' Find the peak date of an incidence curve
#'
#' This function can be used to find the peak of an epidemic curve stored as an
#' `incidence` object.
#'
#' @author Thibaut Jombart \email{thibautjombart@@gmail.com}, Zhian N. Kamvar 
#'   \email{zkamvar@@gmail.com}
#'
#' @md
#'
#' @export
#'
#' @param x An `incidence` object.
#' @param pool If `TRUE` (default), any groups will be pooled before finding
#'   a peak. If `FALSE`, separate peaks will be found for each group.
#'
#' @return The date of the (first) highest incidence in the data.
#'
#' @seealso [estimate_peak()] for bootstrap estimates of the peak time
#'
#' @examples
#'
#' if (require(outbreaks) && require(ggplot2)) { withAutoprint({
#'   i <- incidence(fluH7N9_china_2013$date_of_onset)
#'   i
#'   plot(i)
#'
#'   ## one simple bootstrap
#'   x <- bootstrap(i)
#'   x
#'   plot(x)
#'
#'   ## find 95% CI for peak time using bootstrap
#'   find_peak(i)
#'
#'
#'   ## show confidence interval
#'   plot(i) + geom_vline(xintercept = find_peak(i), col = "red", lty = 2)
#'
#' })}
#'

find_peak <- function(x, pool = TRUE) {
  if (!inherits(x, "incidence")) {
    stop("x is not an incidence object")
  }

  if (ncol(x$counts) > 1L && pool) {
    msg <- paste("'x' is stratified by groups",
                 "pooling groups before finding peaks",
                 sep = "\n")
    message(msg)
    x <- pool(x)
  }
  the_max <- apply(get_counts(x),
		   MARGIN = 2L,
		   FUN    = which.max
		  )

  out <- stats::setNames(x$dates[the_max], colnames(x$counts))
  out
}

#' Access various elements of an incidence object
#'
#' @param x an [incidence] object.
#' @param ... Unused
#'
#' @return 
#'   - `get_interval()` if `integer = TRUE`: an integer vector, otherwise: the 
#'     value stored in `x$interval`
#' @export
#' @keywords accessors
#'
#' @rdname accessors
#' @aliases get_interval
#' @seealso 
#'   - [get_counts()] to access the matrix of counts
#'   - [get_dates()] to access the dates on the right, left, and center of the
#'       interval.
#'   - [group_names()] to access and possibly re-name the groups
#' @examples
#'
#' set.seed(999)
#' dat <- as.Date(Sys.Date()) + sample(-3:50, 100, replace = TRUE)
#' x <- incidence(dat, interval = "month")
#' 
#' # the value stored in the interval element
#' get_interval(x)
#'
#' # the numeric value of the interval in days
#' get_interval(x, integer = FALSE)
#' 
#' # the number of observations in the object
#' get_n(x)
#' 
#' # the length of time represented
#' get_timespan(x)
#'
#' # the number of groups
#' ncol(x)
#' 
#' # the number of bins (intervals)
#' nrow(x)
get_interval <- function(x, ...) {
  UseMethod("get_interval")
}

#' @export
#' @rdname accessors
#' @aliases get_interval.default
get_interval.default <- function(x, ...) {
  stop(sprintf("Not implemented for class %s",
               paste(class(x), collapse = ", ")))
}

#' @param integer When `TRUE` (default), the interval will be converted to an
#'   integer vector if it is stored as a character in the incidence object.
#' @export
#' @rdname accessors
#' @aliases get_interval.incidence
get_interval.incidence <- function(x, integer = TRUE, ...) {
  if (!integer || is.numeric(x$interval)) {
    return(x$interval)
  }
  if (is.character(x$interval)) {
    res <- get_interval_type(x$interval)
    n   <- get_interval_number(x$interval)
    res <- switch(res,
                  day     = 1L * n,
                  week    = 7L * n,
                  month   = get_days_in_month(x$dates, n),
                  quarter = get_days_in_quarter(x$dates, n),
                  year    = get_days_in_year(x$dates, n)
                 )
    return(res)
  } else {
    stop(sprintf("I don't know how to convert a %s to an integer",
                 paste(class(x$interval), collapse = ", ")))
  }
}

get_interval_type <- function(x) {
  res <- NULL
  res <- if (grepl("day", x, ignore.case = TRUE)) "day" else res
  res <- if (grepl("week", x, ignore.case = TRUE)) "week" else res
  res <- if (grepl("month", x, ignore.case = TRUE)) "month" else res
  res <- if (grepl("quarter", x, ignore.case = TRUE)) "quarter" else res
  res <- if (grepl("year", x, ignore.case = TRUE)) "year" else res
  res
}

get_interval_number <- function(x) {

  if (!grepl("^\\d", x)) return(1L)
  as.integer(gsub("^(\\d*).+$", "\\1", x))

}


get_days_in_month <- function(dates, m = 1L) {
  dates <- floor_month(dates)
  res <- vapply(strsplit(format(dates), "-"), 
                add_months, 
                character(1),
                months = m)
  as.integer(as.Date(res) - dates)
}

get_days_in_quarter <- function(dates, m = 1L) {
  dates <- floor_month(dates)
  res <- vapply(strsplit(format(dates), "-"),
                FUN = add_months,
                FUN.VALUE = character(1),
                months = 3L * m)
  as.integer(as.Date(res) - dates)
}

get_days_in_year <- function(dates, m = 1L) {
  dates <- floor_month(dates)
  res <- vapply(strsplit(format(dates), "-"),
                FUN = add_months,
                FUN.VALUE = character(1),
                months = 12L * m)
  as.integer(as.Date(res) - dates)
}

floor_month <- function(x) {
  x - as.integer(format(x, "%d")) + 1L
}

add_months <- function(x, months = 1L) {
  i <- as.integer(x[2]) + months
  if (i > 12L) {
    x[1] <- as.character(as.integer(x[1]) + 1L)
    i    <- i - 12L
  }
  x[2] <- sprintf("%02d", i)
  paste(x, collapse = "-")
}

#' Bootstrap incidence time series
#'
#' This function can be used to bootstrap `incidence` objects. Bootstrapping is
#' done by sampling with replacement the original input dates. See `details` for
#' more information on how this is implemented.
#'
#' @author Thibaut Jombart \email{thibautjombart@@gmail.com}
#'
#' @md
#'
#' @export
#'
#' @details As original data are not stored in `incidence` objects, the
#'   bootstrapping is achieved by multinomial sampling of date bins weighted by
#'   their relative incidence.
#'
#' @param x An `incidence` object.
#'
#' @param randomise_groups A `logical` indicating whether groups should be
#'   randomised as well in the resampling procedure; respective group sizes will
#'   be preserved, but this can be used to remove any group-specific temporal
#'   dynamics. If `FALSE` (default), data are resampled within groups.
#'
#' @return An `incidence` object.
#'
#' @seealso [incidence::find_peak] to use estimate peak date using bootstrap
#'
#' @examples
#'
#' if (require(outbreaks) && require(ggplot2)) { withAutoprint({
#'   i <- incidence(fluH7N9_china_2013$date_of_onset)
#'   i
#'   plot(i)
#'
#'   ## one simple bootstrap
#'   x <- bootstrap(i)
#'   x
#'   plot(x)
#'
#' })}
#'

bootstrap <- function(x, randomise_groups = FALSE) {
  if (!inherits(x, "incidence")) {
    stop("x is not an incidence object")
  }

  ## `counts` is a vector of event counts, meant to be a column of x$counts
  boot_one_group <- function(counts) {
    sample_(x$dates, size = sum(counts), replace = TRUE, prob = counts)
  }

  new_dates <- do.call(c,
                       lapply(seq.int(ncol(x$counts)),
                              function(i) boot_one_group(x$counts[, i])))
  group_sizes <- colSums(x$counts)
  new_groups <- rep(colnames(x$counts), group_sizes)

  if (randomise_groups) {
    new_groups <- sample_(new_groups)
  }

  incidence(new_dates, interval = x$interval, groups = new_groups)

}

#' Fit exponential models to incidence data
#'
#' The function `fit` fits two exponential models to incidence data, of the
#' form: \eqn{log(y) = r * t + b} \cr where 'y' is the incidence, 't' is time
#' (in days), 'r' is the growth rate, and 'b' is the origin. The function `fit`
#' will fit one model by default, but will fit two models on either side of a
#' splitting date (typically the peak of the epidemic) if the argument `split`
#' is provided. When groups are present, these are included in the model as main
#' effects and interactions with dates. The function `fit_optim_split()` can be
#' used to find the optimal 'splitting' date, defined as the one for which the
#' best average R2 of the two models is obtained. Plotting can be done using
#' `plot`, or added to an existing incidence plot by the piping-friendly
#' function `add_incidence_fit()`.
#'
#' @export
#'
#' @rdname fit
#'
#' @return For `fit()`, a list with the class `incidence_fit` (for a
#' single model), or a list containing two `incidence_fit` objects (when
#' fitting two models). `incidence_fit` objects contain:
#'
#' - `$model`: the fitted linear model
#' - `$info`: a list containing various information extracted from the model
#'   (detailed further)
#' - `$origin`: the date corresponding to day '0'
#'
#' The `$info` item is a list containing:
#'
#' - `r`: the growth rate
#' - `r.conf`: the confidence interval of 'r'
#' - `pred`: a `data.frame` containing predictions of the model,
#'   including the true dates (`dates`), their numeric version used in the
#'   model (`dates.x`), the predicted value (`fit`), and the lower
#'   (`lwr`) and upper (`upr`) bounds of the associated confidence
#'   interval.
#'
#' - `doubling`: the predicted doubling time in days; exists only if 'r' is
#'   positive
#' - `doubling.conf`: the confidence interval of the doubling time
#' - `halving`: the predicted halving time in days; exists only if 'r' is
#'   negative
#' - `halving.conf`: the confidence interval of the halving time
#'
#' For `fit_optim_split`, a list containing:
#'
#' - `df`: a `data.frame` of dates that were used in the optimization
#'   procedure, and the corresponding average R2 of the resulting models.
#' - `split`: the optimal splitting date
#' - `fit`: an `incidence_fit_list` object containing the fit for each split.
#'   If the `separate_split = TRUE`, then the `incidence_fit_list` object will
#'   contain these splits nested within each group. All of the `incidence_fit`
#'   objects can be retrieved with [get_fit()].
#' - `plot`: a plot showing the content of `df` (ggplot2 object)
#'
#' @author Thibaut Jombart \email{thibautjombart@@gmail.com}, Zhian N. Kamvar
#'   \email{zkamvar@@gmail.com}.
#'
#' @seealso the [incidence()] function to generate the 'incidence'
#' objects. The [get_fit()] function to flatten `incidence_fit_list` objects to
#' a list of `incidence_fit` objects.
#'
#' @param x An incidence object, generated by the function
#' [incidence()]. For the plotting function, an `incidence_fit`
#' object.
#'
#' @param split An optional time point identifying the separation between the
#' two models. If NULL, a single model is fitted. If provided, two models would
#' be fitted on the time periods on either side of the split.
#'
#' @param level The confidence interval to be used for predictions; defaults to
#' 95%.
#'
#' @param quiet A logical indicating if warnings from `fit` should be
#' hidden; FALSE by default. Warnings typically indicate some zero incidence,
#' which are removed before performing the log-linear regression.
#'
#' @examples
#'
#' if (require(outbreaks)) { withAutoprint({
#'  dat <- ebola_sim$linelist$date_of_onset
#'
#'  ## EXAMPLE WITH A SINGLE MODEL
#'
#'  ## compute weekly incidence
#'  i.7 <- incidence(dat, interval=7)
#'  plot(i.7)
#'  plot(i.7[1:20])
#'
#'  ## fit a model on the first 20 weeks
#'  f <- fit(i.7[1:20])
#'  f
#'  names(f)
#'  head(get_info(f, "pred"))
#'
#'  ## plot model alone (not recommended)
#'  plot(f)
#'
#'  ## plot data and model (recommended)
#'  plot(i.7, fit = f)
#'  plot(i.7[1:25], fit = f)
#'
#' ## piping versions
#' if (require(magrittr)) { withAutoprint({
#'   plot(i.7) %>% add_incidence_fit(f)
#'
#'
#'   ## EXAMPLE WITH 2 PHASES
#'   ## specifying the peak manually
#'   f2 <- fit(i.7, split = as.Date("2014-10-15"))
#'   f2
#'   plot(i.7) %>% add_incidence_fit(f2)
#'
#'   ## finding the best 'peak' date
#'   f3 <- fit_optim_split(i.7)
#'   f3
#'   plot(i.7) %>% add_incidence_fit(f3$fit)
#' })}
#'
#' })}
#'


## The model fitted is a simple linear regression on the log-incidence.

## Non-trivial bits involve:

## 1) Fitting several models
## I.e. in case there is a increasing and a decreasing phase, we fit one
##  model for each phase separately.

## 2) log(0)
## No satisfying solutions so far; for now removing the NAs

## 3) Several groups
## In this case, the number of models does not change, but models automatically
## include groups with interaction, whether or not it is significant.

## 4) Values of dates used as 'x'

## To retain generality, we need to use numbers (not Date or POSIXct) as 'x'
## axis for the model.  Therefore, all dates are expressed as numbers of days
## since the first case (aka 'day 0' or 'origin'), picking the middle of each
## time interval. We also keep track of the origin, so that actual dates can be
## reconstructed during the plotting. Each 'fit' object has its own origin.

fit <- function(x, split = NULL, level = 0.95, quiet = FALSE){
  n.groups <- ncol(x$counts)

  ## remove dates with one incidence of zero
  to.keep <- apply(x$counts, 1, min) > 0
  if (!quiet && !all(to.keep)) {
    warning(sprintf("%d dates with incidence of 0 ignored for fitting",
                    sum(!to.keep)))
  }
  x <- x[to.keep]
  ## If there is only one date with non-zero incidence
  ## then no model cannot be fit. If there are no days with
  ## non-zero incidence, creation of incidence object throws
  ## error anyway.
  if (x$timespan == 1) {
    stop("Only 1 date with non-zero incidence. Cannot fit model to 1 data point.")
  }
  ## Ensure that model coefficients are based on a daily timestep (not seconds)
  if (inherits(x$dates, 'POSIXt')) {
    x$dates <- as.Date(x$dates)
  }

  ## Constructing the model based on number of groups present
  if (n.groups == 1) {
    the_model <- "log(counts) ~ dates.x"
  } else {
    the_model <- "log(counts) ~ dates.x * groups"
  }
  the_model <- stats::formula(the_model)

  ## model without split (1 model)
  if (is.null(split)) {
    df <- as.data.frame(x, long = TRUE)
    ## exact dates
    df$dates.x <- get_dates(x, position = "center", count_days = TRUE)
    lm1 <- stats::lm(the_model, data = df)
    # updating the call for easier inspection by the user
    lm1$call[[2]] <- the_model
    out <- extract_info(lm1, min(get_dates(x)), level)
  } else {
    x1 <- x[x$dates <= split]
    x2 <- x[x$dates >= split]
    df1 <- as.data.frame(x1, long = TRUE)
    df2 <- as.data.frame(x2, long = TRUE)
    ## exact dates
    df1$dates.x <- get_dates(x1, position = "center", count_days = TRUE)
    df2$dates.x <- get_dates(x2, position = "center", count_days = TRUE)
    lm1 <- stats::lm(the_model, data = df1)
    lm2 <- stats::lm(the_model, data = df2)
    # updating the call for easier inspection by the user
    lm1$call[[2]] <- the_model -> lm2$call[[2]]
    before <- extract_info(lm1, min(get_dates(x1)), level)
    after <- extract_info(lm2, min(get_dates(x2)), level)
    out <- list(before = before,
                after = after
                )
    attr(out, "locations") <- list("before", "after")
    class(out) <- "incidence_fit_list"
  }
  out
}

#' Check the interval between bins
#'
#' This enforces that an interval is:
#' - strictly positive
#' - integer (rounded) OR compatibile with date
#' - finite
#' - of length 1
#'
#' @param x an integer or numeric interval
#' @return an integer interval
#' @noRd
check_interval <- function(x, standard = TRUE){
  if (missing(x) || is.null(x)) {
    stop("Interval is missing or NULL")
  }
  if (length(x) != 1L) {
    stop(sprintf(
      "Exactly one value should be provided as interval (%d provided)",
      length(x)))
  }
  if (!is.finite(x)) {
    if (is.character(x)) {
      x <- valid_interval_character(x, standard)
    } else {
      stop("Interval is not finite")
    }
  }
  if (is.numeric(x)) {
    x <- as.integer(round(old <- x))
  }
  if (x < 1L) {
    stop(sprintf(
      "Interval must be at least 1 (input: %.3f; after rounding: %d)",
      old, x))
  }
  x
}

#' Plot function for incidence objects
#'
#' This function is used to visualise the output of the [incidence()]
#' function using the package `ggplot2`. #'
#'
#' @export
#'
#' @importFrom graphics plot
#'
#' @author Thibaut Jombart \email{thibautjombart@@gmail.com}
#'   Zhian N. Kamvar \email{zkamvar@@gmail.com}
#'
#' @seealso The [incidence()] function to generate the 'incidence'
#' objects.
#'
#' @param x An incidence object, generated by the function
#' [incidence()].
#'
#' @param fit An 'incidence_fit' object as returned by [fit()].
#'
#' @param stack A logical indicating if bars of multiple groups should be
#' stacked, or displayed side-by-side.
#'
#' @param color The color to be used for the filling of the bars; NA for
#' invisible bars; defaults to "black".
#'
#' @param border The color to be used for the borders of the bars; NA for
#' invisible borders; defaults to NA.
#'
#' @param col_pal The color palette to be used for the groups; defaults to
#' `incidence_pal1`. See [incidence_pal1()] for other palettes implemented in
#' incidence.
#'
#' @param alpha The alpha level for color transparency, with 1 being fully
#' opaque and 0 fully transparent; defaults to 0.7.
#'
#' @param xlab The label to be used for the x-axis; empty by default.
#'
#' @param ylab The label to be used for the y-axis; by default, a label will be
#' generated automatically according to the time interval used in incidence
#' computation.
#'
#' @param labels_week a logical value indicating whether labels x axis tick
#' marks are in week format YYYY-Www when plotting weekly incidence; defaults to
#' TRUE.
#'
#' @param labels_iso (deprecated) This has been superceded by `labels_iso`.
#' Previously:a logical value indicating whether labels x axis tick marks are
#' in ISO 8601 week format yyyy-Www when plotting ISO week-based weekly
#' incidence; defaults to be TRUE.
#'
#' @param show_cases if `TRUE` (default: `FALSE`), then each observation will be
#' colored by a border. The border defaults to a white border unless specified
#' otherwise. This is normally used outbreaks with a small number of cases.
#' Note: this can only be used if `stack = TRUE`
#'
#' @param n_breaks the ideal number of breaks to be used for the x-axis
#'   labeling
#' 
#' @return
#'  - `plot()` a [ggplot2::ggplot()] object.
#'  - `make_breaks()` a two-element list. The "breaks" element will contain the
#'    evenly-spaced breaks as either dates or numbers and the "labels" element
#'    will contain either a vector of weeks OR a [ggplot2::waiver()] object.
#'  - `scale_x_incidence()` a \pkg{ggplot2} "ScaleContinuous" object. 
#'
#' @details 
#'  - `plot()` will visualise an incidence object using `ggplot2`
#'  - `make_breaks()` calculates breaks from an incidence object that always
#'    align with the bins and start on the first observed incidence. 
#'  - `scale_x_incidence()` produces and appropriate `ggplot2` scale based on
#'    an incidence object. 
#' 
#' @examples
#'
#' if(require(outbreaks) && require(ggplot2)) { withAutoprint({
#'   onset <- outbreaks::ebola_sim$linelist$date_of_onset
#'
#'   ## daily incidence
#'   inc <- incidence(onset)
#'   inc
#'   plot(inc)
#'
#'   ## weekly incidence
#'   inc.week <- incidence(onset, interval = 7)
#'   inc.week
#'   plot(inc.week) # default to label x axis tick marks with isoweeks
#'   plot(inc.week, labels_week = FALSE) # label x axis tick marks with dates
#'   plot(inc.week, border = "white") # with visible border
#'
#'   ## use group information
#'   sex <- outbreaks::ebola_sim$linelist$gender
#'   inc.week.gender <- incidence(onset, interval = "1 epiweek", groups = sex)
#'   plot(inc.week.gender)
#'   plot(inc.week.gender, labels_week = FALSE)
#'
#'   ## show individual cases at the beginning of the epidemic
#'   inc.week.8 <- subset(inc.week.gender, to = "2014-06-01")
#'   p <- plot(inc.week.8, show_cases = TRUE, border = "black")
#'   p
#'
#'   ## update the range of the scale
#'   lim <- c(min(get_dates(inc.week.8)) - 7*5, 
#'            aweek::week2date("2014-W50", "Sunday"))
#'   lim
#'   p + scale_x_incidence(inc.week.gender, limits = lim)
#'
#'   ## customize plot with ggplot2
#'   plot(inc.week.8, show_cases = TRUE, border = "black") +
#'     theme_classic(base_size = 16) +
#'     theme(axis.text.x = element_text(angle = 90, hjust = 1, vjust = 0.5)) 
#'
#'   ## adding fit
#'   fit <- fit_optim_split(inc.week.gender)$fit
#'   plot(inc.week.gender, fit = fit)
#'   plot(inc.week.gender, fit = fit, labels_week = FALSE)
#'
#' })}
#'
plot.incidence <- function(x, ..., fit = NULL, stack = is.null(fit),
                           color = "black", border = NA, col_pal = incidence_pal1,
                           alpha = .7, xlab = "", ylab = NULL,
                           labels_week = !is.null(x$weeks),
                           labels_iso = !is.null(x$isoweeks),
                           show_cases = FALSE,
                           n_breaks = 6) {
  stopifnot(is.logical(labels_iso), is.logical(labels_week))

  the_call <- match.call()
  if (any(names(the_call) == "labels_iso"))  {
    if (any(names(the_call) == "labels_week")) {
      warning("labels_iso is deprecated. The value of `labels_week` will be used.")
    } else {
      warning("labels_iso is deprecated. Use `labels_week` instead.")
      labels_week <- labels_iso
    }
  }
  ## extract data in suitable format for ggplot2
  df <- as.data.frame(x, long = TRUE, stringsAsFactors = TRUE)
  n.groups <- ncol(x$counts)
  gnames   <- group_names(x)

  
  ## Use custom labels for usual time intervals
  if (is.null(ylab)) {
    if (is.numeric(x$interval)) {
      if (x$interval == 1) {
        ylab <- "daily incidence"
      } else if (x$interval == 7) {
        ylab <- "weekly incidence"
      } else if (x$interval == 14) {
        ylab <- "semi-weekly incidence"
      } else {
        ylab <- sprintf("incidence by period of %d days",
                        x$interval)
      }
    } else if (is.character(x$interval)) {
      
      # capturing the number and type
      p     <- "(\\d*)\\s?([a-z]+?)s?$"
      num   <- gsub(p, "\\1", tolower(x$interval))
      itype <- gsub(p, "\\2", tolower(x$interval))
      if (num == "" || num == "1") {
        ylab <- sprintf("%sly incidence", itype)
      } else {
        ylab <- sprintf("incidence by a period of %s %ss", num, itype)
      }
    }
    if (!is.null(x$weeks)) {
      type_of_week <- get_type_of_week(x)
      ylab <- gsub("(weekl?y?)", sprintf("%s \\1", type_of_week), ylab)
    }
    if (isTRUE(x$cumulative)) {
      ylab <- sub("incidence", "cumulative incidence", ylab)
    }
    first_letter          <- substring(ylab, 1, 1)
    substring(ylab, 1, 1) <- toupper(first_letter)
  }

  ## Handle stacking
  stack.txt <- if (stack) "stack" else "dodge"

  ## By default, the annotation of bars in geom_bar puts the label in the
  ## middle of the bar. This is wrong in our case as the annotation of a time
  ## interval is the lower (left) bound, and should therefore be left-aligned
  ## with the bar. Note that we cannot use position_nudge to create the
  ## x-offset as we need the 'position' argument for stacking. This can be
  ## addressed by adding interval/2 to the x-axis, but this only works until we
  ## have an interval such as "month", "quarter", or "year" where the number of
  ## days for each can vary. To alleviate this, we can create a new column that
  ## counts the number of days within each interval.

  ## Adding a variable for width in ggplot
  df$interval_days <- get_interval(x, integer = TRUE)
  ## if the date type is POSIXct, then the interval is actually interval seconds
  ## and needs to be converted to days
  if (inherits(df$dates, "POSIXct")) {
    df$interval_days <- df$interval_days * 86400 # 24h * 60m * 60s
  }
  ## Important note: it seems safest to specify the aes() as part of the geom,
  ## not in ggplot(), as it interacts badly with some other geoms like
  ## geom_ribbon - used e.g. in projections::add_projections().


  ## add mid-interval positions for x-axis

  ## THIS BREAKS THE PLOT WITH ggplot2_3.3.0
  ## See bug reports at:
  ## https://github.com/reconhub/incidence/issues/119
  ## https://github.com/tidyverse/ggplot2/issues/3873
  ## Temporary fix: changing placement to default of ggplot2::scale_x_date
  
  x_axis <- "dates + (interval_days/2)"
  y_axis <- "counts"
  out <- ggplot2::ggplot(df) +
    ggplot2::geom_col(ggplot2::aes_string(
                        x = x_axis,
                        y = y_axis
                        ),
                      width = df$interval_days,
                      position = stack.txt,
                      color = border,
                      alpha = alpha) +
    ggplot2::labs(x = xlab, y = ylab)

  ## Handle show_cases here

  if (show_cases && stack) {
    squaredf <- df[rep(seq.int(nrow(df)), df$counts), ]
    squaredf$counts <- 1
    squares <- ggplot2::geom_col(ggplot2::aes_string(
                                   x = x_axis,
                                   y = y_axis
                                   ),
                                 color = if (is.na(border)) "white" else border,
                                 fill  = NA,
                                 position = "stack",
                                 data = squaredf,
                                 width = squaredf$interval_days
                                 )
    out <- out + squares
  }
  if (show_cases && !stack) {
    message("the argument `show_cases` requires the argument `stack = TRUE`")
  }
  ## Handle fit objects here; 'fit' can be either an 'incidence_fit' object,
  ## or a list of these. In the case of a list, we add geoms one after the
  ## other.

  if (!is.null(fit)) {
    if (inherits(fit, "incidence_fit")) {
      out <- add_incidence_fit(out, fit)
    } else if (is.list(fit)) {
      for (i in seq_along(fit)) {
        fit.i <- fit[[i]]
        if (!inherits(fit.i, c("incidence_fit", "incidence_fit_list"))) {
          stop(sprintf(
            "The %d-th item in 'fit' is not an 'incidence_fit' object, but a %s",
            i, class(fit.i)))
        }
        out <- add_incidence_fit(out, fit.i)
      }
    } else {
      stop("Fit must be a 'incidence_fit' object, or a list of these")
    }
  }


  ## Handle colors

  ## Note 1: because of the way 'fill' works, we need to specify it through
  ## 'aes' if not directly in the geom. This causes the kludge below, where we
  ## make a fake constant group to specify the color and remove the legend.

  ## Note 2: when there are groups, and the 'color' argument does not have one
  ## value per group, we generate colors from a color palette. This means that
  ## by default, the palette is used, but the user can manually specify the
  ## colors.

  if (n.groups < 2 && is.null(gnames)) {
    out <- out + ggplot2::aes(fill = 'a') +
      ggplot2::scale_fill_manual(values = color, guide = FALSE)
  } else {
    if (!is.null(names(color))) {
      tmp     <- color[gnames] 
      matched <- names(color) %in% names(tmp)
      if (!all(matched)) {
        removed <- paste(names(color)[!matched], 
                         color[!matched],
                         sep = '" = "',
                         collapse = '", "')
        message(sprintf("%d colors were not used: \"%s\"", sum(!matched), removed))
      }
      color <- tmp
    }
                                 
    ## find group colors
    if (length(color) != n.groups) {
      msg <- "The number of colors (%d) did not match the number of groups (%d)"
      msg <- paste0(msg, ".\nUsing `col_pal` instead.")
      default_color <- length(color) == 1L && color == "black"
      if (!default_color) {
        message(sprintf(msg, length(color), n.groups))
      }
      group.colors <- col_pal(n.groups)
    } else {
      group.colors <- color
    }

    ## add colors to the plot
    out <- out + ggplot2::aes_string(fill = "groups") +
      ggplot2::scale_fill_manual(values = group.colors)
    if (!is.null(fit)) {
      out <- out + ggplot2::aes_string(color = "groups") +
        ggplot2::scale_color_manual(values = group.colors)
    }
  }

  out <- out + scale_x_incidence(x, n_breaks, labels_week, ...)
  out
}


## This function will take an existing 'incidence' plot object ('p') and add lines from an
## 'incidence_fit' object ('x')

#' @export
#' @rdname plot.incidence
#'
#' @param p An existing incidence plot.
add_incidence_fit <- function(p, x, col_pal = incidence_pal1){
  if (inherits(x, "incidence_fit_list")) {
    x <- get_fit(x)
  }
  ## 'x' could be a list of fit, in which case all fits are added to the plot
  if (is.list(x) && !inherits(x, "incidence_fit")) {
    out <- p
    for (e in x) {
      if (inherits(e, "incidence_fit")) {
        out <- add_incidence_fit(out, e, col_pal)
      }
    }
    return(out)
  }
  df <- get_info(x, "pred")

  # In the case that the incidence object is of the type POSIXt, the data from
  # the fit object must be presented as POSIXt or ggplot2 will throw a fit.

  if (inherits(p$data$dates, "POSIXt")) {
    # I add half a day here because any fractional days (0.5) will be thrown out
    # on conversion and I'm not quite sure why that is
    df$dates <- as.POSIXlt(df$dates) + 43200 # adding half a day
    if (inherits(p$data$dates, "POSIXct")) {
      df$dates <- as.POSIXct(df$dates)
    } 
  }

  out <- suppressMessages(
    p + ggplot2::geom_line(
      data = df,
      ggplot2::aes_string(x = "dates", y = "fit"), linetype = 1) +
      ggplot2::geom_line(
        data = df,
        ggplot2::aes_string(x = "dates", y = "lwr"), linetype = 2) +
      ggplot2::geom_line(
        data = df,
        ggplot2::aes_string(x = "dates", y = "upr"), linetype = 2)
  )


  if ("groups" %in% names(df)) {
    n.groups <- length(levels(df$groups))
    out <- out + ggplot2::aes_string(color = "groups") +
      ggplot2::scale_color_manual(values = col_pal(n.groups))
  }

  out
}





#' @export
#' @rdname plot.incidence

plot.incidence_fit <- function(x, ...){
  base <- ggplot2::ggplot()
  out <- add_incidence_fit(base, x, ...) +
    ggplot2::labs(x = "", y = "Predicted incidence")
  out
}

#' @export
#' @rdname plot.incidence

plot.incidence_fit_list <- function(x, ...){
  base <- ggplot2::ggplot()
  fits <- get_fit(x)
  out  <- add_incidence_fit(base, fits, ...) +
    ggplot2::labs(x = "", y = "Predicted incidence")
  out
}

#' Color palettes used in incidence
#'
#' These functions are color palettes used in incidence.
#'
#' @author Thibaut Jombart \email{thibautjombart@@gmail.com}
#'
#' @param n a number of colors
#'
#' @rdname palettes
#' @aliases palettes incidence_pal1 incidence_pal1_light incidence_pal1_dark
#'
#' @export
#' @importFrom grDevices colorRampPalette
#'
#' @examples
#'
#' plot(1:4, cex=8, pch=20, col = incidence_pal1(4),
#'      main = "palette: incidence_pal1")
#' plot(1:100, cex=8, pch=20, col = incidence_pal1(100),
#'      main ="palette: incidence_pal1")
#' plot(1:100, cex=8, pch=20, col = incidence_pal1_light(100),
#'      main="palette: incidence_pal1_light")
#' plot(1:100, cex=8, pch=20, col = incidence_pal1_dark(100),
#'      main="palette: incidence_pal1_dark")
#'
incidence_pal1 <- function(n){
  if(!is.numeric(n)) stop("n is not a number")
  colors <- c("#aa3939", "#4a6a8a", "#d4aa6a","#499371")
  if (n < 4) return(colors[1:n])
  return(colorRampPalette(colors)(n))
}





#' @export
#' @rdname palettes

incidence_pal1_light <- function(n){
  if(!is.numeric(n)) stop("n is not a number")
  colors <- c("#d46a6a", "#738ca6", "#ffddaa","#76b096")
  if (n < 4) return(colors[1:n])
  return(colorRampPalette(colors)(n))
}






#' @export
#' @rdname palettes

incidence_pal1_dark <- function(n){
  if(!is.numeric(n)) stop("n is not a number")
  colors <- c("#801515", "#2b4c6f", "#aa7d39","#277552")
  if (n < 4) return(colors[1:n])
  return(colorRampPalette(colors)(n))
}

context("Color palettes")

test_that("incidence_pal1", {
  skip_on_cran()

  expect_error(incidence_pal1(NULL), "n is not a number")
  for (n in 1:30) {
    expect_length(incidence_pal1(n), n)
  }
})

test_that("incidence_pal1_light", {
  skip_on_cran()

  expect_error(incidence_pal1_light(NULL), "n is not a number")
  for (n in 1:30) {
    expect_length(incidence_pal1_light(n), n)
  }
})



test_that("incidence_pal1_dark", {
  skip_on_cran()

  expect_error(incidence_pal1_dark(NULL), "n is not a number")
  for (n in 1:30) {
    expect_length(incidence_pal1_dark(n), n)
  }
})

##' Subsetting 'incidence' objects
##'
##' Two functions can be used to subset incidence objects.  The function
##' `subset` permits to retain dates within a specified range and,
##' optionally, specific groups. The operator "[" can be used as for matrices,
##' using the syntax `x[i,j]` where 'i' is a subset of dates, and 'j' is a
##' subset of groups.
##'
##' @author Thibaut Jombart \email{thibautjombart@@gmail.com}
##'
##' @export
##'
##' @rdname subset
##'
##' @aliases "subset.incidence" "[.incidence"
##'
##' @seealso The [incidence()] function to generate the 'incidence'
##' objects.
##'
##' @param x An incidence object, generated by the function
##' [incidence()].
##'
##' @param from The starting date; data strictly before this date are discarded.
##'
##' @param to The ending date; data strictly after this date are discarded.
##'
##' @param groups (optional) The groups to retained, indicated as subsets of the
##' columns of x$counts.
##'
##' @param ... Further arguments passed to other methods (not used).
##'
##' @examples
##' ## example using simulated dataset
##' if(require(outbreaks)) { withAutoprint({
##'   onset <- ebola_sim$linelist$date_of_onset
##'
##'   ## weekly incidence
##'   inc <- incidence(onset, interval = 7)
##'   inc
##'   inc[1:10] # first 10 weeks
##'   plot(inc[1:10])
##'   inc[-c(11:15)] # remove weeks 11-15
##'   plot(inc[-c(11:15)])
##' })}
##'

subset.incidence <- function(x, ..., from = min(x$dates), to = max(x$dates),
                             groups = TRUE){

  ## We need to make sure the comparison with dates is going to work. As for the
  ## [ operator, 'from' and 'to' are assumed to be expressed in the same way as
  ## the x$dates.

  is_date      <- inherits(x$dates, "Date")
  numeric_from <- is.numeric(from)
  numeric_to   <- is.numeric(to)

  if (is_date && (numeric_from || numeric_to)) {
    the_intervals <- get_interval(x, integer = TRUE)
    if (length(the_intervals) == 1L) {
      the_intervals <- rep(the_intervals, length(x$dates))
    }
    the_intervals <- cumsum(c(0, the_intervals))
  }
  if (is_date && numeric_from) {
    if (from <= 0) {
      from <- 0L
    } else if (from >= length(the_intervals) - 1L) {
      from <- the_intervals[length(the_intervals) - 1L]
    } else {
      from <- the_intervals[from]
    }
    from <- min(x$dates) + from
  }

  if (is_date && numeric_to) {
    if (to <= 0) {
      to <- 0L
    } else if (to >= length(the_intervals) - 1L) {
      to <- the_intervals[length(the_intervals) - 1L]
    } else {
      to <- the_intervals[to]
    }
    to <- min(x$dates) + to
  }

  to.keep <- x$dates >= from & x$dates <= to

  if (sum(to.keep) < 1) {
    stop("No data retained.")
  }
  x[to.keep, groups]
}




##' @export
##' @rdname subset
##' @param i a subset of dates to retain
##' @param j a subset of groups to retain

"[.incidence" <- function(x, i, j){
    if (missing(i)) {
        i <- TRUE
    }

    if (missing(j)) {
        j <- TRUE
    }

    out <- x
    if (is.character(j) && !all(j %in% group_names(x))) {
      odd_names <- j[!j %in% group_names(x)]
      groups    <- if (length(odd_names) > 1) "groups do" else "group does"
      odd_names <- paste(j[!j %in% group_names(x)], collapse = "', '")
      msg       <- sprintf("The following %s not exist: '%s'", groups, odd_names)
      stop(msg)
    }
    out$counts <- out$counts[i, j, drop = FALSE]
    out$dates <- out$dates[i]
    if ("weeks" %in% names(x)) {
      out$weeks    <- out$weeks[i] 
      out$isoweeks <- out$isoweeks[i]
    }
    # Need to use 1L here to keep things type-stable:
    # double + integer = double
    # integer + integer = integer
    # Date + integer = Date
    out$timespan <-  diff(range(out$dates, na.rm = TRUE)) + 1L
    out$n <- sum(out$counts)
    out
}

#' @export
#' @rdname fit
#'
#' @param window The size, in days, of the time window either side of the
#' split.
#'
#' @param plot A logical indicating whether a plot should be added to the
#' output (`TRUE`, default), showing the mean R2 for various splits.
#'
#' @param separate_split If groups are present, should separate split dates be
#' determined for each group? Defaults to `TRUE`, in which separate split dates
#' and thus, separate models will be constructed for each group. When `FALSE`,
#' the split date will be determined from the pooled data and modelled with the
#' groups as main effects and interactions with date.
#'
fit_optim_split <- function(x, window = x$timespan/4, plot = TRUE,
                            quiet = TRUE, separate_split = TRUE) {
  if (ncol(x$counts) > 1 && separate_split) {
    # Calculate split for each group separately --------------------------------
    res        <- vector(mode = "list", length = ncol(x$counts))
    names(res) <- colnames(x$counts)
    for (i in names(res)) {
      res[[i]] <- fit_optim_split(x[, i], separate_split = FALSE, plot = FALSE)
    }

    # Rearrange data -----------------------------------------------------------
    # The resulting object will have the follwing structure
    # $df
    # $plot
    #     $group1 <ggplot>
    #     $group2 <ggplot>
    # $split
    # $fit <incidence_fit_list>
    #     $group1 <incidence_fit_list>
    #         $before <incidence_fit>
    #         $after  <incidence_fit>
    #     $group2 <incidence_fit_list>
    #         $before <incidence_fit>
    #         $after  <incidence_fit>
    dates  <- get_dates(x)[[1]]
    dfrows <- vapply(res, function(i) nrow(i$df), integer(1))
    out <- list(
      df = data.frame(dates = seq(dates, by = 1, length.out = sum(dfrows)),
                      mean.R2 = vector(mode = "numeric", length = sum(dfrows)),
                      groups = factor(rep(names(res), dfrows), names(res)),
                      stringsAsFactors = TRUE
                     ),
      plot = ggplot2::ggplot(),
      split = seq(dates, by = 1, length.out = length(res)),
      fit = vector(mode = "list", length = length(res))
    )
    names(out$fit)   <- names(res)
    names(out$plot)  <- names(res)
    names(out$split) <- names(res)
    for (i in names(res)) {
      n <- factor(i, names(res))
      out$fit[[i]]   <- res[[i]]$fit
      out$plot[[i]]  <- res[[i]]$plot
      out$split[[i]] <- res[[i]]$split
      out$fit[[i]]$after$info$pred$groups  <- n
      out$fit[[i]]$before$info$pred$groups <- n
      out$df[out$df$groups == i, ]$dates   <- res[[i]]$df$dates
      out$df[out$df$groups == i, ]$mean.R2 <- res[[i]]$df$mean.R2
    }
    if (plot) {
      out$plot <- ggplot2::ggplot(
          out$df,
          ggplot2::aes_string(x = "dates", y = "mean.R2", color = "groups")
        ) +
        ggplot2::geom_point() +
        ggplot2::geom_line() +
        ggplot2::geom_text(ggplot2::aes_string(label="dates"),
                           hjust = -0.1, angle = 35
        ) +
        ggplot2::ylim(min = min(out$df$mean.R2) - 0.1, max = 1)
    } else {
      out$plot <- NULL
    }

    # Adding attributes for incidence_fit_list ---------------------------------
    attr(out$fit, "locations") <- c(
      lapply(names(res), c, "before"),
      lapply(names(res), c, "after")
    )
    class(out$fit) <- "incidence_fit_list"
    return(out)
  }
  date.peak <- x$dates[which.max(pool(x)$counts)]
  try.since <- date.peak - window / 2
  try.until <- date.peak + window / 2
  to.keep <- x$dates >= try.since & x$dates <= try.until
  if (sum(to.keep) < 1) {
    stop("No date left to try after defining splits to try.")
  }

  splits.to.try <- x$dates[to.keep]
  need.to.try   <- length(splits.to.try) > 1

  f <- function(split) {
    fits <- fit(x, split = split, quiet = quiet)
    mean(vapply(fits, function(e) summary(e$model)$`adj.r.squared`, double(1)), na.rm = TRUE)
  }

  results <- vapply(splits.to.try, f, double(1))

  ## shape output
  df <- data.frame(dates = splits.to.try, mean.R2 = results, stringsAsFactors = TRUE)
  split <- if (need.to.try) splits.to.try[which.max(results)] else splits.to.try
  fit <- suppressWarnings(fit(x, split = split))
  out <- list(df = df,
              split = split,
              fit = fit)

  if (plot) {
    out$plot <- ggplot2::ggplot(
      df, ggplot2::aes_string(x = "dates", y = "mean.R2")) +
      ggplot2::geom_point() + ggplot2::geom_line() +
      ggplot2::geom_text(ggplot2::aes_string(label="dates"),
                         hjust=-.1, angle=35) +
      ggplot2::ylim(min=min(results)-.1, max=1)
  }

  out
}

---
title: "Details of the incidence class"
author: "Thibaut Jombart, Zhian N. Kamvar"
date: "`r Sys.Date()`"
output:
   rmarkdown::html_vignette:
     toc: true
     toc_depth: 3
vignette: >
  %\VignetteIndexEntry{Incidence class}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---




```{r, options, echo = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>", 
  fig.width=7, 
  fig.height=5 
)

```

This vignette details the structure of *incidence* objects, as produced by the
`incidence` function.

<br>

# Structure of an *incidence* object.

We generate a toy dataset of dates to examine the content of *incidence* objects.

```{r, data}
library(incidence)
set.seed(1)
dat <- sample(1:50, 200, replace = TRUE, prob = 1 + exp(1:50 * 0.1))
sex <- sample(c("female", "male"), 200, replace = TRUE)
```

The incidence by 48h period is computed as:

```{r, i}
i <- incidence(dat, interval = 2)
i
plot(i)
```

We also compute incidence by gender:

```{r, sex}
i.sex <- incidence(dat, interval = 2, group = sex)
i.sex
plot(i.sex)
```


The object `i` is a `list` with the class *incidence*:

```{r, names}
class(i)
is.list(i)
names(i)
```

Items in `i` can be accessed using the same indexing as any lists, but it's
safer to use the accessors for each item:

```{r, access}
## use name
head(i$dates)

head(get_dates(i))
```

In the following sections, we examine each of the components of the object.


## `$dates`

The `$dates` component contains a vector for all the dates for which incidence
have been computed, in the format of the input dataset (e.g. `Date`, `numeric`,
`integer`).

```{r, dates1}
date_bins <- get_dates(i)
class(date_bins)
class(dat)

date_bins
```

The dates correspond to the lower bounds of the time intervals used as bins for
the incidence. Bins always include the lower bound and exclude the upper bound.
In the example provided above, this means that the first bin counts events that
happened at day 5-6, the second bin counts events from 7-8, etc.

Note that if we had actual `Date`-class dates, they would be returned as dates

```{r date-dates1}
dat_Date <- as.Date("2018-10-31") + dat
head(dat_Date)
i.date <- incidence(dat_Date, interval = 2, group = sex)
i.date
get_dates(i.date)
class(get_dates(i.date))
```

These can be converted to integers, counting the number of days from the first
date.

```{r get-dates-integer}
get_dates(i.date, count_days = TRUE)
get_dates(i, count_days = TRUE)
```

To facilitate modelling, it's also possible to get the center of the interval by
using the `position = "center"` argument:

```{r get-dates-center}
get_dates(i.date, position = "center")
get_dates(i.date, position = "center", count_days = TRUE)
```


## `$counts`

The `$counts` component contains the actual incidence, i.e. counts of events
for the defined bins. It is a `matrix` of `integers` where rows correspond to
time intervals, with one column for each group for which incidence is computed
(a single, unnamed column if no groups were provided). If groups were provided,
columns are named after the groups. We illustrate the difference comparing the
two objects `i` and `i.sex`:

```{r, counts1}
counts <- get_counts(i)
class(counts)
storage.mode(counts)

counts
get_counts(i.sex)
```

You can see the dimensions of the incidence object by using `dim()`, `ncol()`,
and `nrow()`, which returns the dimensions of the counts matrix:

```{r counts1.1}
dim(get_counts(i.sex))
dim(i.sex)
nrow(i.sex) # number of date bins
ncol(i.sex) # number of groups
```


There are also accessors for handling groups:

```{r groups}
# Number of groups
ncol(i.sex)
ncol(i)

# Names of groups
group_names(i.sex)
group_names(i)

# You can also rename the groups
group_names(i.sex) <- c("F", "M")
group_names(i.sex)
```


Note that a `data.frame` containing *dates* and *counts* can be obtained using
`as.data.frame`:

```{r, as.data.frame}
## basic conversion
as.data.frame(i)
as.data.frame(i.sex)

## long format for ggplot2
as.data.frame(i.sex, long = TRUE)
```

Note that `incidence` has an argument called `na_as_group` which is `TRUE` by
default, which will pool all missing groups into a separate group, in which
case it will be a separate column in `$counts`.

## `$timespan`

The `$timespan` component stores the length of the time period covered by the
object:

```{r, timespan}
get_timespan(i)
print(date_range <- range(get_dates(i)))
diff(date_range) + 1
```

## `$interval`

The `$interval` component contains the length of the time interval for the bins:

```{r, interval}
get_interval(i)
diff(get_dates(i))
```

## `$n`

The `$n` component stores the total number of events in the data:

```{r, n}
get_n(i)
```

Note that to obtain the number of cases by groups, one can use:

```{r, n2}
colSums(get_counts(i.sex))
```

## `$weeks`

The `$weeks` component is optional, and used to store
[aweek](https://www.repidemicsconsortium.org/aweek/) objects whenever they have
been used. Weeks are used by default when weekly incidence is computed from
dates (see argument `standard` in `?incidence`).

```{r, isoweek}
library(outbreaks)
dat <- ebola_sim$linelist$date_of_onset
i.7 <- incidence(dat, "1 epiweek", standard = TRUE)
i.7
i.7$weeks
```

Because `$weeks` is an optional element, it does not have a dedicated
accessor. If the element is not present, attempting to access it will result in
a `NULL`:

```{r isoweek-null}
i$weeks
```

Both dates and weeks are returned when converting an `incidence` object to
`data.frame`:

```{r, isoweek3}
head(as.data.frame(i.7))
```

---
title: "Conversions to and from the incidence class"
author: "Thibaut Jombart, Zhian N. Kamvar"
date: "`r Sys.Date()`"
output:
   rmarkdown::html_vignette:
     toc: true
     toc_depth: 2
vignette: >
  %\VignetteIndexEntry{Conversions}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---



```{r, echo = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>", 
  fig.width=7, 
  fig.height=5 
)
```


This vignette documents to types of conversion which can be made using the *incidence* class:

- *'exports'*: conversion from an *incidence* object to another type of object;
   this can be useful for processing incidence data in another software, or for
   reporting results.

- *'imports'*conversion from already computed incidence into an *incidence*
   object; this can be useful for using features of the *incidence* package for
   data handling and plotting with incidence data computed elsewhere.



<br>

# Exporting results

To export results, we first compute semi-weekly incidence (with weeks starting
on Sunday, the beginning of the CDC epiweek) by gender from the simulated Ebola
data used in the [overview
vignette](https://www.repidemicsconsortium.org/incidence/articles/overview.html):

```{r example}
library(outbreaks)
library(incidence)
dat <- ebola_sim$linelist$date_of_onset
i_14 <- incidence(dat, interval = "2 epiweeks", groups = ebola_sim$linelist$gender)
i_14
plot(i_14, border = "white")
```

To export the data to a `data.frame`, one simply needs:

```{r}
as.data.frame(i_14)
```

The first column contains the dates marking the (inclusive) left side of the
time intervals used for computing incidence, and the other columns give counts
for the different groups. This function also has an option for exporting data as
a 'long' format, i.e. with a column for 'groups' and a column for counts. This
format can be useful especially when working with *ggplot2*, which expect data
in this shape:

```{r, long}
df <- as.data.frame(i_14, long = TRUE)
head(df)
tail(df)

## example of custom plot using steps:
library(ggplot2)
ggplot(df, aes(x = dates, y = counts)) + geom_step(aes(color = groups))
```


Finally, note that when ISO weeks are used, these are also reported in the output:

```{r, iso}
i_7 <- incidence(dat, interval = "week")
i_7
plot(i_7, border = "white")
head(as.data.frame(i_7))
tail(as.data.frame(i_7))

```




<br>

# Importing pre-computed incidence

The function `as.incidence` facilitates the conversion of pre-computed
incidences to an *incidence* object. Typically, the input will be imported into
R from a *.csv* file or other spreadsheet formats.


`as.incidence` is a generic with methods for several types of objects (see
`?as.incidence`). The main method is `matrix`, as other types are coerced to
`matrix` first and then passed to `as.incidence.matrix`:

```{r, conversions}
args(incidence:::as.incidence.matrix)
```

The only mandatory argument `x` is a table of counts, with time intervals in
rows and groups in columns; if there are no groups, then the column doesn't need
a name; but if there are several groups, then columns should be named to
indicate group labels. Optionally, `dates` can be provided to indicate the
(inclusive) lower bounds of the time intervals, corresponding to the rows of
`x`; most sensible date formats will do; if indicated as a character string,
make sure the format is `YYYY-mm-dd`, e.g. `2017-04-01` for the 1st April 2017.


Let us illustrate the conversion using a simple vector of incidence:
```{r}
vec <- c(1,2,3,0,3,2,4,1,2,1)

i <- as.incidence(vec)
i

plot(vec, type = "s")
plot(i, border = "white")

```

Assuming the above incidences are computed weekly, we would then use:
```{r}
i <- as.incidence(vec, interval = 7)
i
plot(i, border = "white")

```

Note that in this case, incidences have been treated as per week, and
corresponding dates in days have been computed during the conversion (the first
day is always '1'), so that the first days of weeks 1, 2, 3... are:
```{r}
i$dates
```

In practice, it is best to provide the actual dates marking the lower bounds of
the time intervals. We can illustrate this by a round trip using the example of
the previous section:

```{r, round_trip}
## convertion: incidence --> data.frame:
i_14
df <- as.data.frame(i_14)
head(df)
tail(df)

## conversion: data.frame --> incidence
new_i <- as.incidence(df[group_names(i_14)], df$dates, interval = "2 epiweeks")
new_i

## check round trip
identical(new_i, i_14)

```

---
title: "Customize plots of incidence"
author: "Thibaut Jombart, Zhian N. Kamvar"
date: "`r Sys.Date()`"
output:
   rmarkdown::html_vignette:
     toc: true
     toc_depth: 4
vignette: >
  %\VignetteIndexEntry{Customise graphics}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---



```{r, echo = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width=7,
  fig.height=5
)
```


This vignette provides some tips for the most common customisations of graphics produced by
`plot.incidence`. Our graphics use *ggplot2*, which is a distinct graphical system from base
graphics. If you want advanced customisation of your incidence plots, we recommend following an
introduction to *ggplot2*.


<br>

# Example data: simulated Ebola outbreak

This example uses the simulated Ebola Virus Disease (EVD) outbreak from the package
[*outbreaks*](https://cran.r-project.org/package=outbreaks): `ebola_sim_clean`.

First, we load the data:
```{r, data}
library(outbreaks)
library(ggplot2)
library(incidence)

onset <- ebola_sim_clean$linelist$date_of_onset
class(onset)
head(onset)
```

We compute the weekly incidence:
```{r, incid1}
i <- incidence(onset, interval = 7)
i

i.sex <- incidence(onset, interval = 7, group =  ebola_sim_clean$linelist$gender)
i.sex

i.hosp <- incidence(onset, interval = 7, group =  ebola_sim_clean$linelist$hospital)
i.hosp

```




<br>

# The `plot.incidence` function

When calling `plot` on an *incidence* object, the function `plot.incidence` is implicitly used. To access its documentation, use `?plot.incidence`. In this section, we illustrate existing customisations.


## Default behaviour

By default, the function uses grey for single time series, and colors from the color palette `incidence_pal1` when incidence is computed by groups:
```{r, default}
plot(i)
plot(i.sex)
plot(i.hosp)
```

However, some of these defaults can be altered through the various arguments of the function:
```{r, args}
args(incidence:::plot.incidence)
```




## Changing colors

### The default palette

A color palette is a function which outputs a specified number of colors. By
default, the color used in *incidence* is called `incidence_pal1`. Its
behaviour is different from usual palettes, in the sense that the first 4
colours are not interpolated:

```{r, incidence_pal1,   fig.height = 8}
par(mfrow = c(3, 1), mar = c(4,2,1,1))
barplot(1:2, col = incidence_pal1(2))
barplot(1:4, col = incidence_pal1(4))
barplot(1:20, col = incidence_pal1(20))
```

This palette also has a light and a dark version:

```{r, pal2, fig.height = 8}
par(mfrow = c(3,1))
barplot(1:20, col = incidence_pal1_dark(20), main = "palette:  incidence_pal1_dark")
barplot(1:20, col = incidence_pal1(20), main = "palette:  incidence_pal1")
barplot(1:20, col = incidence_pal1_light(20), main = "palette:  incidence_pal1_light")
```



### Using different palettes

Other color palettes can be provided via `col_pal`. Various palettes are part of the base R distribution, and many more are provided in additional packages. We provide a couple of examples:

```{r, palettes}
plot(i.hosp, col_pal = rainbow)
plot(i.sex, col_pal = cm.colors)
```



### Specifying colors manually

Colors can be  specified manually using the argument `color`; note that whenever incidence is computed by groups, the number of colors must match the number of groups, otherwise `color` is ignored.

#### Example 1: changing a single color
```{r, colors1}
plot(i, color = "darkred")
```

#### Example 2: changing several colors (note that naming colors is optional)
```{r, colors2}
plot(i.sex, color = c(m = "orange2", f = "purple3"))
```

#### Example 3: using color to highlight specific groups
```{r, colors3}
plot(i.hosp, 
     color = c("#ac3973", "#6666ff", "white", "white", "white", "white"))
```





<br>

# Useful *ggplot2* tweaks

Numerous tweaks for *ggplot2* are documented online. In the following, we merely provide a few useful tips in the context of *incidence*.



##  Changing dates on the *x*-axis

### Changing date format

By default, the dates indicated on the *x*-axis of an incidence plot may not
have the suitable format.
The package *scales* can be used to change the way dates are labeled (see
`?strptime` for possible formats):

```{r, scales1}
library(scales)
plot(i, labels_week = FALSE) +
   scale_x_date(labels = date_format("%d %b %Y"))
```

Notice how the labels are all situated at the first of the month? If you want to
make sure the labels are situated in a different orientation, you can use the
`make_breaks()` function to calculate breaks for the plot:

```{r scales_breaks}
b <- make_breaks(i, labels_week = FALSE)
b
plot(i) +
  scale_x_date(breaks = b$breaks, 
               labels = date_format("%d %b %Y"))
```



And for another example, with a subset of the data (first 50 weeks), using more
detailed dates and rotating the annotations:

```{r, scales2}
plot(i[1:50]) +  
  scale_x_date(breaks = b$breaks, labels = date_format("%a %d %B %Y")) +
  theme(axis.text.x = element_text(angle = 45, hjust = 1, size = 12))
```

Note that you can save customisations for later use:

```{r, scales3}
rotate.big <- theme(axis.text.x = element_text(angle = 45, hjust = 1, size = 12))
```


### Changing the grid

The last example above illustrates that it can be useful to have denser
annotations of the *x*-axis, especially over short time periods. Here, we
provide an example where we try to zoom on the peak of the epidemic, using the
data by hospital:

```{r, grid1}
plot(i.hosp)
```

Let us look at the data 40 days before and after the 1st of October:

```{r, grid2}
period <- as.Date("2014-10-01") + c(-40, 40)
i.zoom <- subset(i.hosp, from = period[1], to = period[2])
detailed.x <- scale_x_date(labels = date_format("%a %d %B %Y"), 
                           date_breaks = "2 weeks", 
                           date_minor_breaks = "week")

plot(i.zoom, border = "black") + detailed.x + rotate.big
```

### Handling non-ISO weeks

If you have weekly incidence that starts on a day other than monday, then the
above solution may produce breaks that fall inside of the bins:

```{r, saturday-epiweek}
i.sat <- incidence(onset, interval = "1 week: saturday", groups = ebola_sim_clean$linelist$hospital)
i.szoom <- subset(i.sat, from = period[1], to = period[2])

plot(i.szoom, border = "black") + detailed.x + rotate.big
```

In this case, you may want to either calculate breaks using `make_breaks()` or
use the `scale_x_incidence()` function to automatically calculate these for you:

```{r, saturday-epiweek2}
plot(i.szoom, border = "black") + 
  scale_x_incidence(i.szoom, n_breaks = nrow(i.szoom)/2, labels_week = FALSE) +
  rotate.big
```

```{r, saturday-epiweek3}
sat_breaks <- make_breaks(i.szoom, n_breaks = nrow(i.szoom)/2)
plot(i.szoom, border = "black") + 
  scale_x_date(breaks = sat_breaks$breaks, labels = date_format("%a %d %B %Y")) +
  rotate.big
```

### Labelling every bin


Sometimes you may want to label every bin of the incidence object. To do this,
you can simply set `n_breaks` to the number of rows in your incidence object:

```{r label-bins}
plot(i.szoom, n_breaks = nrow(i.szoom), border = "black") + rotate.big
```


## Changing the legend

The previous plot has a fairly large legend which we may want to move around.
Let us save the plot as a new object `p` and customize the legend:

```{r, legend1}
p <- plot(i.zoom, border = "black") + detailed.x + rotate.big
p + theme(axis.text.x = element_text(angle = 45, hjust = 1, size = 12), 
          legend.position = "top", legend.direction = "horizontal", 
          legend.title = element_blank())
```


## Applying the style of European Programme for Intervention Epidemiology Training (EPIET)

### Display individual cases

For small datasets it is convention of EPIET to display individual cases as
rectangles. It can be done by doing two things: first, adding using the option
`show_cases = TRUE` with a white border and second, setting the background to
white. We also add `coord_equal()` which forces each case to be a square.

```{r, EPIET1}
i.small <- incidence(onset[160:180])

plot(i.small, border = "white", show_cases = TRUE) +
  theme(panel.background = element_rect(fill = "white")) + 
  rotate.big +
  coord_equal() 
```

---
title: "Overview of the incidence package"
author: "Thibaut Jombart, Zhian N. Kamvar"
date: "`r Sys.Date()`"
output:
   rmarkdown::html_vignette:
     toc: true
     toc_depth: 2
vignette: >
  %\VignetteIndexEntry{Overview}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---



```{r, echo = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>", 
  fig.width=7, 
  fig.height=5
)
```


*incidence* implements functions and classes to compute, handle, visualise and model incidences from
dates data. This vignette provides an overview of current features. It largely reproduces the
content of `REAME.md`.



<br>

# Installing the package

To install the current stable, CRAN version of the package, type:

```{r install, eval=FALSE}
install.packages("incidence")
```

To benefit from the latest features and bug fixes, install the development, *github* version of the package using:

```{r install2, eval=FALSE}
devtools::install_github("reconhub/incidence")
```

Note that this requires the package *devtools* installed.



<br>

# What does it do?

The main functions of the package include:

- **`incidence`**: compute incidence from dates in various formats; any fixed
  time interval can be used; the returned object is an instance of the (S3)
  class *incidence*.

- **`plot`**: this method (see `?plot.incidence` for details) plots *incidence*
  objects, and can also add predictions of the model(s) contained in an
  *incidence_fit* object (or a list of such objects).

- **`fit`**: fit one or two exponential models (i.e. linear regression on
  log-incidence) to an *incidence* object; two models are calibrated only if a
  date is provided to split the time series in two (argument `split`); this is
  typically useful to model the two phases of exponential growth, and decrease
  of an outbreak; each model returned is an instance of the (S3) class
  *incidence_fit*, each of which contains various useful information
  (e.g. growth rate *r*, doubling/halving time, predictions and confidence
  intervals); results can be plotted using `plot`, or added to an existing
  `uncudence` plot using the piping-friendly function `add_incidence_fit`.

- **`fit_optim_split`**: finds the optimal date to split the time series in two,
  typically around the peak of the epidemic.

- **`[`**: lower-level subsetting of *incidence* objects, permitting to specify
  which dates and groups to retain; uses a syntax similar to matrices,
  i.e. `x[i, j]`, where `x` is the *incidence* object, `i` a subset of dates,
  and `j` a subset of groups.

- **`subset`**: subset an *incidence* object by specifying a time window.

- **`pool`**: pool incidence from different groups into one global incidence
  time series.

- **`cumulate`**: computes cumulative incidence over time from and `incidence`
  object.

- **`as.data.frame`**: converts an *incidence* object into a `data.frame`
  containing dates and incidence values.

- **`bootstrap`**: generates a bootstrapped *incidence* object by re-sampling,
  with replacement, the original dates of events.

- **`find_peak`**: locates the peak time of the epicurve.

- **`estimate_peak`**: uses bootstrap to estimate the peak time (and related
  confidence interval) of a partially observed outbreak.






# Worked example: simulated Ebola outbreak

## Loading the data

This example uses the simulated Ebola Virus Disease (EVD) outbreak from the package
[*outbreaks*](https://cran.r-project.org/package=outbreaks). We will compute incidence for various time
steps, calibrate two exponential models around the peak of the epidemic, and analyse the results.

First, we load the data:
```{r, data}
library(outbreaks)
library(ggplot2)
library(incidence)

dat <- ebola_sim$linelist$date_of_onset
class(dat)
head(dat)
```


## Computing and plotting incidence
We compute the daily incidence:
```{r, incid1}
i <- incidence(dat)
i
plot(i)
```

The daily incidence is quite noisy, but we can easily compute other incidence
using larger time intervals:

```{r, interv}
# weekly, starting on Monday (ISO week, default)
i.7 <- incidence(dat, interval = "1 week")
plot(i.7)

# semi-weekly, starting on Saturday
i.14 <- incidence(dat, interval = "2 saturday weeks")
plot(i.14, border = "white")

## monthly
i.month <- incidence(dat, interval = "1 month")
plot(i.month, border = "white")

```

`incidence` can also compute incidence by specified groups using the `groups`
argument. For instance, we can compute incidence by gender:

```{r, gender}
i.7.sex <- incidence(dat, interval = "1 week", groups = ebola_sim$linelist$gender)
i.7.sex
plot(i.7.sex, stack = TRUE, border = "grey")
```

We can do the same for hospitals, using the 'clean' version of the dataset, with
some customization of the legend:

```{r, hosp}
i.7.hosp <- with(ebola_sim_clean$linelist, 
	 incidence(date_of_onset, interval = "week", groups = hospital))
i.7.hosp
head(get_counts(i.7.hosp))
plot(i.7.hosp, stack=TRUE) + 
    theme(legend.position= "top") + 
    labs(fill="")
```



## Handling `incidence` objects

`incidence` objects can be manipulated easily. The `[` operator implements
subsetting of dates (first argument) and groups (second argument). For
instance, to keep only the peak of the distribution:

```{r, middle}
i[100:250]
plot(i[100:250])
```

Or to keep every other week:

```{r, stripes}
i.7[c(TRUE,FALSE)]
plot(i.7[c(TRUE,FALSE)])
```

Some temporal subsetting can be even simpler using `subset`, which permits to
retain data within a specified time window:

```{r, tail}
i.tail <- subset(i, from=as.Date("2015-01-01"))
i.tail
plot(i.tail, border="white")
```

Subsetting groups can also matter. For instance, let's try and visualise the
incidence based on onset of symptoms by outcome:

```{r, i7outcome}
i.7.outcome <- incidence(dat, 7, groups=ebola_sim$linelist$outcome)
i.7.outcome
plot(i.7.outcome, stack = TRUE, border = "grey")
```

By default, `incidence` treats missing data (NA) as a separate group (see
argument `na_as_group`). We could disable this to retain only known outcomes,
but alternatively we can simply subset the object to exclude the last (3rd)
group:

```{r, groupsub}
i.7.outcome[,1:2]
plot(i.7.outcome[,1:2], stack = TRUE, border = "grey")
```

Groups can also be collapsed into a single time series using `pool`:

```{r, pool}
i.pooled <- pool(i.7.outcome)
i.pooled
identical(i.7$counts, i.pooled$counts)
```



## Modelling incidence

Incidence data, excluding zeros, can be modelled using log-linear regression of
the form: log(*y*) = *r* x *t* + *b*

where *y* is the incidence, *r* is the growth rate, *t* is the number of days
since a specific point in time (typically the start of the outbreak), and *b*
is the intercept.

Such model can be fitted to any incidence object using `fit`.  Of course, a
single log-linear model is not sufficient for modelling our epidemic curve, as
there is clearly an growing and a decreasing phase.  As a start, we can
calibrate a model on the first 20 weeks of the epidemic:


```{r, fit1}
plot(i.7[1:20])
early.fit <- fit(i.7[1:20])
early.fit
```

The resulting objects (known as `incidence_fit` objects) can be plotted, in
which case the prediction and its confidence interval is displayed:

```{r}
plot(early.fit)
```

However, a better way to display these predictions is adding them to the
incidence plot using the argument `fit`:

```{r}
plot(i.7[1:20], fit = early.fit)
```


In this case, we would ideally like to fit two models, before and after the
peak of the epidemic. This is possible using the following approach, if you
know what date to use to split the data in two phases:

```{r, fit.both}
fit.both <- fit(i.7, split=as.Date("2014-10-15"))
fit.both
plot(i.7, fit=fit.both)
```

This is much better, but the splitting date is not completely optimal. To look
for the best possible splitting date (i.e. the one maximizing the average fit
of both models), we use:

```{r, optim}
best.fit <- fit_optim_split(i.7)
best.fit
plot(i.7, fit=best.fit$fit)
```

These models are very good approximation of these data, showing a doubling time
of `r round(get_info(best.fit$fit, "doubling"), 1)` days during the first
phase, and a halving time of `r round(get_info(best.fit$fit, "halving"), 1)`
days during the second. 
To access these parameters, you can use the `get_info()` function.

The possible parameters are:

 - "r", the daily growth rate
 - "doubling" the rate of doubling in days (if "r" is positive)
 - "halving" the rate of halving in days (if "r" is negative)
 - "pred" a data frame of incidence predictions
 
For "r", "doubling", and "halving", you can also add ".conf" to get the
confidence intervals. Here's how you can get the doubling and halving times of
the above epi curve:

```{r, get_info}
get_info(best.fit$fit, "doubling")      # doubling time
get_info(best.fit$fit, "doubling.conf") # confidence interval
get_info(best.fit$fit, "halving")       
get_info(best.fit$fit, "halving.conf")       
```

Note that `fit` will also take groups into account if incidence has been
computed for several groups:

```{r, optim2}
best.fit2 <- fit_optim_split(i.7.sex)
best.fit2
plot(i.7.sex, fit=best.fit2$fit)
```

Using `get_info()` on this fit object will return all groups together:

```{r, get_info_groups}
get_info(best.fit2$fit, "doubling")      # doubling time
get_info(best.fit2$fit, "doubling.conf") # confidence interval
get_info(best.fit2$fit, "halving")       
get_info(best.fit2$fit, "halving.conf")       
```

---
title: "Details of the incidence_fit class"
author: "Zhian N. Kamvar"
date: "`r Sys.Date()`"
output:
   rmarkdown::html_vignette:
     toc: true
     toc_depth: 3
vignette: >
  %\VignetteIndexEntry{Incidence fit class}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>", 
  fig.width=7, 
  fig.height=5
)
```

This vignette details the structure and construction of the `incidence_fit` and 
`incidence_fit_list` classes, which are produced by the `fit()` and 
`fit_optim_split()` functions, respectively. By the end of this tutorial, you
should be able to construct `incidence_fit` and `incidence_fit_list` objects for
use with your own models.

# Structure of an `incidence_fit` object

An `incidence_fit` object contains three elements:

 - `$model`: The model fit to an `incidence` object. Currently, this represents a 
    log-linear model, but it can be any model. 
 - `$info`: Information derived from the model
   - `r` The growth rate
   - `r.conf` the confidence interval of `r`
   - `pred` a data frame containing the predictions of the model using the true
      dates (`dates`), their numeric version used in the model (`dates.x`), the
      predicted value (`fit`), and the lower (`lwr`) and upper (`upr`) bounds of
      the associated confidence interval.
   - `doubling` the predicted doubling time in days (only if `r` is positive)
   - `doubling.conf` the confidence interval of the doubling time
   - `halving` the predicted halving time in days (only if `r` is negative)
   - `halving.conf` the confidence interval of the halving time
 - `$origin`: the date corresponding to day '0'

Internally, when `fit()` is run, these elements are constructed by
function `incidence:::extract_info()`. First we need to setup data. We will use 
simulated Ebola outbreak data from the *outbreaks* package over weekly intervals
and calculate the fit for the first 20 weeks:

```{r fit_dates}
library(outbreaks)
library(incidence)
dat <- ebola_sim$linelist$date_of_onset
i <- incidence(dat, interval = "week")
i
f <- fit(i[1:20])
f
plot(i, fit = f)
```

As you can see, the `incidence_fit` object has a print method and a plot method.
If you want to access individual elements in the `$info` element, you can use
the `get_info()` function:

```{r get_info}
get_info(f, "r")
get_info(f, "r.conf")
get_info(f, "doubling.conf")
```

This will be important later when we combine several `incidence_fit` objects into
a single `incidence_fit_list`.

# Building an `incidence_fit` object from scratch

The `incidence_fit` object can be constructed from any model from which you can
derive the daily growth rate, doubling/halving times, predictions, and confidence
intervals. The following three steps show roughly how it is done from model 
fitting to construction.

### Step 1: create the model

The default model for `fit()` is a log-linear model on the intervals between 
dates. To fit this model, we will need to create a data frame with the counts
and the midpoints of the intervals:

```{r create_model}
# ensure all dates have at least one incidence
i2 <- i[1:20]
i2 <- i2[apply(get_counts(i2), 1, min) > 0]
df <- as.data.frame(i2, long = TRUE)
df$dates.x <- get_dates(i2, position = "center", count_days = TRUE)
head(df)
lm1 <- stats::lm(log(counts) ~ dates.x, data = df)
lm1
```

If we compare that to the `$model` element produced from `fit()`, we can see that
it is identical:

```{r fit_model}
all.equal(f$model, lm1)
```

### Step 2: creation of the `$info` list:

The `$info` list is created directly from the model itself:

```{r make_info}
r <- stats::coef(lm1)["dates.x"]
r.conf <- stats::confint(lm1, "dates.x", 0.95)
new.data <- data.frame(dates.x = sort(unique(lm1$model$dates.x)))
pred     <- exp(stats::predict(lm1, newdata = new.data, interval = "confidence",
                               level = 0.95))
pred <- cbind.data.frame(new.data, pred)
info_list <- list(
  r = r,
  r.conf = r.conf,
  doubling = log(2) / r,
  doubling.conf = log(2) / r.conf,
  pred = pred
)
info_list
```

### Step 3: combine lists and create object

the last step is to combine everything into a list and create the object.

```{r combine}
origin <- min(get_dates(i2))
info_list$pred$dates <- origin + info_list$pred$dates.x
the_fit <- list(
  lm = lm1,
  info = info_list,
  origin = min(get_dates(i2))
)
class(the_fit) <- "incidence_fit"
the_fit
plot(i, fit = the_fit)
```


# Structure of an `incidence_fit_list` object

There are several reasons for having multiple fits to a single `incidence`
object. One may want to have a separate fit for different groups represented in
the object, or one may want to split the fits at the peak of the epidemic. To
aid in plotting and summarizing the different fits, we've created the 
`incidence_fit_list` class. This class has two defining features:

 - It consists of a named list containing one or more `incidence_fit` objects or
   lists containing `incidence_fit` objects.
 - An attribute called "locations" contains a list whose length is equal to the
   number of `incidence_fit` objects in the object. Each list element contains
   a vector that defines where an `incidence_fit` object is within the 
   `incidence_fit_list`. 

The reason for this structure is because it is sometimes necessary to nest
lists of `incidence_fit` objects within lists. When this happens, accessing
individual elements of the objects cumbersome. To alleviate this, each object
has a distinct path within the named list in the "locations" attribute that 
allows one to access the object directly since R allows you to traverse the
elements of a nested list by subsetting with a vector:

```{r nest}
l <- list(a = list(b = 1, c = 2),d = list(e = list(f = 3, g = 4), h = 5))
str(l)
l[[c("a", "b")]]
l[[c("d", "e", "f")]]
```

## Example: A tale of two fits

The function `fit_optim_split()` attempts to find the optimal split point in an
epicurve, producing an `incidence_fit_list` object in the `$fit` element of the 
returned list:

```{r incidence_fit_list}
fl <- fit_optim_split(i)
fl$fit
plot(i, fit = fl$fit)
```

Here you can see that the object looks very similar to the `incidence_fit` object,
but it has extra information. The first thing you may notice is the fact that
both "doubling" and "halving" are shown. This is because the two fits have 
different signs for the daily growth rate. The second thing you may notice is
the fact that there is something called `attr(x, 'locations')`. This attribute
gives the location of the `incidence_fit` objects within the list. We can
illustrate how this works if we look at the structure of the object:

```{r incidence_fit_list_str}
str(fl$fit, max.level = 2)
```

Internally, all of the methods for `incidence_fit_list` use the 'locations'
attribute to navigate:

```{r incidence_fit_methods}
methods(class = "incidence_fit_list")
```

For example, it's often useful to extract the growth rate for all models at
once. The `get_info()` method allows us to do this easily:

```{r get_info_incidence_fit_list}
get_info(fl$fit, "r")
get_info(fl$fit, "r.conf")
```

Because doubling or halving is determined by whether or not `r` is negative, we
automatically filter out the results that don't make sense, but you can include
them with `na.rm = FALSE`:

```{r get_doubling}
get_info(fl$fit, "doubling.conf")
get_info(fl$fit, "doubling.conf", na.rm = FALSE)
```

## Example: Nested incidence_fit

Above, we showed the example of a basic `incidence_fit_list` class with two
objects representing the fits before and after the peak of an epicurve. However,
it is often useful evaluate fits for different groups separately. Here, we will
construct an incidence object, but define groups by gender:

```{r incidence_by_gender}
gen <- ebola_sim$linelist$gender
ig <- incidence(dat, interval = "week", group = gen)
plot(ig, border = "grey98")
```

Now if we calculate an optimal fit split, we will end up with four different
fits: two for each defined gender.

```{r fit_gender}
fg <- fit_optim_split(ig)
plot(ig, fit = fg$fit, border = "grey98", stack = FALSE)
```

If we look at the fit object, we can see again that it is an `incidence_fit_list`
but this time with four fits defined. 

```{r fit_gender_print}
fg$fit
str(fg$fit, max.level = 3)
```

> Notice that the nested lists themselves are of class `incidence_fit_list`.

Now, even though the fits within nested lists, the 'locations' attributes still
defines where they are within the object so that the `get_info()` function still
operates normally:

```{r get_info_gender}
get_info(fg$fit, "r.conf")
```

If you need to access all the fits easily, a convenience function to flatten the
list is available in `get_fit()`:

```{r get_fit}
str(get_fit(fg$fit), max.level = 2)
```

Because all that defines an `incidence_fit_list` is the class definition and
the 'locations' attribute that defines the positions of the `incidence_fit`
objects within the nesting, then it's also possible to define the output of
`fit_optim_split()` as an `incidence_fit_list` class:

```{r incidence_fit_listify}
print(locs <- attributes(fg$fit)$locations)

for (i in seq_along(locs)) {
	locs[[i]] <- c("fit", locs[[i]])
}
print(locs)
fg.ifl <- fg
attributes(fg.ifl)$locations<- locs
class(fg.ifl) <- "incidence_fit_list"
```

Now when we print the object, we can see that it prints only the information
related to the `incidence_fit_list`:

```{r new_fit_list_print}
fg.ifl
```

But, we still retain all of the extra information in the list:

```{r list_stuff}
str(fg.ifl, max.level = 1)
fg.ifl$split
fg.ifl$df
fg.ifl$plot
```

---
title: "Customize plots of incidence"
author: "Thibaut Jombart, Zhian N. Kamvar"
date: "`r Sys.Date()`"
output:
   rmarkdown::html_vignette:
     toc: true
     toc_depth: 4
vignette: >
  %\VignetteIndexEntry{Customise graphics}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---



```{r, echo = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width=7,
  fig.height=5
)
```


This vignette provides some tips for the most common customisations of graphics produced by
`plot.incidence`. Our graphics use *ggplot2*, which is a distinct graphical system from base
graphics. If you want advanced customisation of your incidence plots, we recommend following an
introduction to *ggplot2*.


<br>

# Example data: simulated Ebola outbreak

This example uses the simulated Ebola Virus Disease (EVD) outbreak from the package
[*outbreaks*](https://github.com/reconhub/outbreaks/): `ebola_sim_clean`.

First, we load the data:
```{r, data}
library(outbreaks)
library(ggplot2)
library(incidence)

onset <- ebola_sim_clean$linelist$date_of_onset
class(onset)
head(onset)
```

We compute the weekly incidence:
```{r, incid1}
i <- incidence(onset, interval = 7)
i

i.sex <- incidence(onset, interval = 7, group =  ebola_sim_clean$linelist$gender)
i.sex

i.hosp <- incidence(onset, interval = 7, group =  ebola_sim_clean$linelist$hospital)
i.hosp

```




<br>

# The `plot.incidence` function

When calling `plot` on an *incidence* object, the function `plot.incidence` is implicitly used. To access its documentation, use `?plot.incidence`. In this section, we illustrate existing customisations.


## Default behaviour

By default, the function uses grey for single time series, and colors from the color palette `incidence_pal1` when incidence is computed by groups:
```{r, default}
plot(i)
plot(i.sex)
plot(i.hosp)
```

However, some of these defaults can be altered through the various arguments of the function:
```{r, args}
args(incidence:::plot.incidence)
```




## Changing colors

### The default palette

A color palette is a function which outputs a specified number of colors. By
default, the color used in *incidence* is called `incidence_pal1`. Its
behaviour is different from usual palettes, in the sense that the first 4
colours are not interpolated:

```{r, incidence_pal1,   fig.height = 8}
par(mfrow = c(3, 1), mar = c(4,2,1,1))
barplot(1:2, col = incidence_pal1(2))
barplot(1:4, col = incidence_pal1(4))
barplot(1:20, col = incidence_pal1(20))
```

This palette also has a light and a dark version:

```{r, pal2, fig.height = 8}
par(mfrow = c(3,1))
barplot(1:20, col = incidence_pal1_dark(20), main = "palette:  incidence_pal1_dark")
barplot(1:20, col = incidence_pal1(20), main = "palette:  incidence_pal1")
barplot(1:20, col = incidence_pal1_light(20), main = "palette:  incidence_pal1_light")
```



### Using different palettes

Other color palettes can be provided via `col_pal`. Various palettes are part of the base R distribution, and many more are provided in additional packages. We provide a couple of examples:

```{r, palettes}
plot(i.hosp, col_pal = rainbow)
plot(i.sex, col_pal = cm.colors)
```



### Specifying colors manually

Colors can be  specified manually using the argument `color`; note that whenever incidence is computed by groups, the number of colors must match the number of groups, otherwise `color` is ignored.

#### Example 1: changing a single color
```{r, colors1}
plot(i, color = "darkred")
```

#### Example 2: changing several colors (note that naming colors is optional)
```{r, colors2}
plot(i.sex, color = c(m = "orange2", f = "purple3"))
```

#### Example 3: using color to highlight specific groups
```{r, colors3}
plot(i.hosp, 
     color = c("#ac3973", "#6666ff", "white", "white", "white", "white"))
```





<br>

# Useful *ggplot2* tweaks

Numerous tweaks for *ggplot2* are documented online. In the following, we merely provide a few useful tips in the context of *incidence*.



##  Changing dates on the *x*-axis

### Changing date format

By default, the dates indicated on the *x*-axis of an incidence plot may not
have the suitable format.
The package *scales* can be used to change the way dates are labeled (see
`?strptime` for possible formats):

```{r, scales1}
library(scales)
plot(i, labels_week = FALSE) +
   scale_x_date(labels = date_format("%d %b %Y"))
```

Notice how the labels are all situated at the first of the month? If you want to
make sure the labels are situated in a different orientation, you can use the
`make_breaks()` function to calculate breaks for the plot:

```{r scales_breaks}
b <- make_breaks(i, labels_week = FALSE)
b
plot(i) +
  scale_x_date(breaks = b$breaks, 
               labels = date_format("%d %b %Y"))
```



And for another example, with a subset of the data (first 50 weeks), using more
detailed dates and rotating the annotations:

```{r, scales2}
plot(i[1:50]) +  
  scale_x_date(breaks = b$breaks, labels = date_format("%a %d %B %Y")) +
  theme(axis.text.x = element_text(angle = 45, hjust = 1, size = 12))
```

Note that you can save customisations for later use:

```{r, scales3}
rotate.big <- theme(axis.text.x = element_text(angle = 45, hjust = 1, size = 12))
```


### Changing the grid

The last example above illustrates that it can be useful to have denser
annotations of the *x*-axis, especially over short time periods. Here, we
provide an example where we try to zoom on the peak of the epidemic, using the
data by hospital:

```{r, grid1}
plot(i.hosp)
```

Let us look at the data 40 days before and after the 1st of October:

```{r, grid2}
period <- as.Date("2014-10-01") + c(-40, 40)
i.zoom <- subset(i.hosp, from = period[1], to = period[2])
detailed.x <- scale_x_date(labels = date_format("%a %d %B %Y"), 
                           date_breaks = "2 weeks", 
                           date_minor_breaks = "week")

plot(i.zoom, border = "black") + detailed.x + rotate.big
```

### Handling non-ISO weeks

If you have weekly incidence that starts on a day other than monday, then the
above solution may produce breaks that fall inside of the bins:

```{r, saturday-epiweek}
i.sat <- incidence(onset, interval = "1 week: saturday", groups = ebola_sim_clean$linelist$hospital)
i.szoom <- subset(i.sat, from = period[1], to = period[2])

plot(i.szoom, border = "black") + detailed.x + rotate.big
```

In this case, you may want to either calculate breaks using `make_breaks()` or
use the `scale_x_incidence()` function to automatically calculate these for you:

```{r, saturday-epiweek2}
plot(i.szoom, border = "black") + 
  scale_x_incidence(i.szoom, n_breaks = nrow(i.szoom)/2, labels_week = FALSE) +
  rotate.big
```

```{r, saturday-epiweek3}
sat_breaks <- make_breaks(i.szoom, n_breaks = nrow(i.szoom)/2)
plot(i.szoom, border = "black") + 
  scale_x_date(breaks = sat_breaks$breaks, labels = date_format("%a %d %B %Y")) +
  rotate.big
```

### Labelling every bin


Sometimes you may want to label every bin of the incidence object. To do this,
you can simply set `n_breaks` to the number of rows in your incidence object:

```{r label-bins}
plot(i.szoom, n_breaks = nrow(i.szoom), border = "black") + rotate.big
```


## Changing the legend

The previous plot has a fairly large legend which we may want to move around.
Let us save the plot as a new object `p` and customize the legend:

```{r, legend1}
p <- plot(i.zoom, border = "black") + detailed.x + rotate.big
p + theme(axis.text.x = element_text(angle = 45, hjust = 1, size = 12), 
          legend.position = "top", legend.direction = "horizontal", 
          legend.title = element_blank())
```


## Applying the style of European Programme for Intervention Epidemiology Training (EPIET)

### Display individual cases

For small datasets it is convention of EPIET to display individual cases as
rectangles. It can be done by doing two things: first, adding using the option
`show_cases = TRUE` with a white border and second, setting the background to
white. We also add `coord_equal()` which forces each case to be a square.

```{r, EPIET1}
i.small <- incidence(onset[160:180])

plot(i.small, border = "white", show_cases = TRUE) +
  theme(panel.background = element_rect(fill = "white")) + 
  rotate.big +
  coord_equal() 
```

---
title: "Overview of the incidence package"
author: "Thibaut Jombart, Zhian N. Kamvar"
date: "`r Sys.Date()`"
output:
   rmarkdown::html_vignette:
     toc: true
     toc_depth: 2
vignette: >
  %\VignetteIndexEntry{Overview}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---



```{r, echo = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>", 
  fig.width=7, 
  fig.height=5
)
```


*incidence* implements functions and classes to compute, handle, visualise and model incidences from
dates data. This vignette provides an overview of current features. It largely reproduces the
content of `REAME.md`.



<br>

# Installing the package

To install the current stable, CRAN version of the package, type:

```{r install, eval=FALSE}
install.packages("incidence")
```

To benefit from the latest features and bug fixes, install the development, *github* version of the package using:

```{r install2, eval=FALSE}
devtools::install_github("reconhub/incidence")
```

Note that this requires the package *devtools* installed.



<br>

# What does it do?

The main functions of the package include:

- **`incidence`**: compute incidence from dates in various formats; any fixed
  time interval can be used; the returned object is an instance of the (S3)
  class *incidence*.

- **`plot`**: this method (see `?plot.incidence` for details) plots *incidence*
  objects, and can also add predictions of the model(s) contained in an
  *incidence_fit* object (or a list of such objects).

- **`fit`**: fit one or two exponential models (i.e. linear regression on
  log-incidence) to an *incidence* object; two models are calibrated only if a
  date is provided to split the time series in two (argument `split`); this is
  typically useful to model the two phases of exponential growth, and decrease
  of an outbreak; each model returned is an instance of the (S3) class
  *incidence_fit*, each of which contains various useful information
  (e.g. growth rate *r*, doubling/halving time, predictions and confidence
  intervals); results can be plotted using `plot`, or added to an existing
  `uncudence` plot using the piping-friendly function `add_incidence_fit`.

- **`fit_optim_split`**: finds the optimal date to split the time series in two,
  typically around the peak of the epidemic.

- **`[`**: lower-level subsetting of *incidence* objects, permitting to specify
  which dates and groups to retain; uses a syntax similar to matrices,
  i.e. `x[i, j]`, where `x` is the *incidence* object, `i` a subset of dates,
  and `j` a subset of groups.

- **`subset`**: subset an *incidence* object by specifying a time window.

- **`pool`**: pool incidence from different groups into one global incidence
  time series.

- **`cumulate`**: computes cumulative incidence over time from and `incidence`
  object.

- **`as.data.frame`**: converts an *incidence* object into a `data.frame`
  containing dates and incidence values.

- **`bootstrap`**: generates a bootstrapped *incidence* object by re-sampling,
  with replacement, the original dates of events.

- **`find_peak`**: locates the peak time of the epicurve.

- **`estimate_peak`**: uses bootstrap to estimate the peak time (and related
  confidence interval) of a partially observed outbreak.






# Worked example: simulated Ebola outbreak

## Loading the data

This example uses the simulated Ebola Virus Disease (EVD) outbreak from the package
[*outbreaks*](https://github.com/reconhub/outbreaks/). We will compute incidence for various time
steps, calibrate two exponential models around the peak of the epidemic, and analyse the results.

First, we load the data:
```{r, data}
library(outbreaks)
library(ggplot2)
library(incidence)

dat <- ebola_sim$linelist$date_of_onset
class(dat)
head(dat)
```


## Computing and plotting incidence
We compute the daily incidence:
```{r, incid1}
i <- incidence(dat)
i
plot(i)
```

The daily incidence is quite noisy, but we can easily compute other incidence
using larger time intervals:

```{r, interv}
# weekly, starting on Monday (ISO week, default)
i.7 <- incidence(dat, interval = "1 week")
plot(i.7)

# semi-weekly, starting on Saturday
i.14 <- incidence(dat, interval = "2 saturday weeks")
plot(i.14, border = "white")

## monthly
i.month <- incidence(dat, interval = "1 month")
plot(i.month, border = "white")

```

`incidence` can also compute incidence by specified groups using the `groups`
argument. For instance, we can compute incidence by gender:

```{r, gender}
i.7.sex <- incidence(dat, interval = "1 week", groups = ebola_sim$linelist$gender)
i.7.sex
plot(i.7.sex, stack = TRUE, border = "grey")
```

We can do the same for hospitals, using the 'clean' version of the dataset, with
some customization of the legend:

```{r, hosp}
i.7.hosp <- with(ebola_sim_clean$linelist, 
	 incidence(date_of_onset, interval = "week", groups = hospital))
i.7.hosp
head(get_counts(i.7.hosp))
plot(i.7.hosp, stack=TRUE) + 
    theme(legend.position= "top") + 
    labs(fill="")
```



## Handling `incidence` objects

`incidence` objects can be manipulated easily. The `[` operator implements
subsetting of dates (first argument) and groups (second argument). For
instance, to keep only the peak of the distribution:

```{r, middle}
i[100:250]
plot(i[100:250])
```

Or to keep every other week:

```{r, stripes}
i.7[c(TRUE,FALSE)]
plot(i.7[c(TRUE,FALSE)])
```

Some temporal subsetting can be even simpler using `subset`, which permits to
retain data within a specified time window:

```{r, tail}
i.tail <- subset(i, from=as.Date("2015-01-01"))
i.tail
plot(i.tail, border="white")
```

Subsetting groups can also matter. For instance, let's try and visualise the
incidence based on onset of symptoms by outcome:

```{r, i7outcome}
i.7.outcome <- incidence(dat, 7, groups=ebola_sim$linelist$outcome)
i.7.outcome
plot(i.7.outcome, stack = TRUE, border = "grey")
```

By default, `incidence` treats missing data (NA) as a separate group (see
argument `na_as_group`). We could disable this to retain only known outcomes,
but alternatively we can simply subset the object to exclude the last (3rd)
group:

```{r, groupsub}
i.7.outcome[,1:2]
plot(i.7.outcome[,1:2], stack = TRUE, border = "grey")
```

Groups can also be collapsed into a single time series using `pool`:

```{r, pool}
i.pooled <- pool(i.7.outcome)
i.pooled
identical(i.7$counts, i.pooled$counts)
```



## Modelling incidence

Incidence data, excluding zeros, can be modelled using log-linear regression of
the form: log(*y*) = *r* x *t* + *b*

where *y* is the incidence, *r* is the growth rate, *t* is the number of days
since a specific point in time (typically the start of the outbreak), and *b*
is the intercept.

Such model can be fitted to any incidence object using `fit`.  Of course, a
single log-linear model is not sufficient for modelling our epidemic curve, as
there is clearly an growing and a decreasing phase.  As a start, we can
calibrate a model on the first 20 weeks of the epidemic:


```{r, fit1}
plot(i.7[1:20])
early.fit <- fit(i.7[1:20])
early.fit
```

The resulting objects (known as `incidence_fit` objects) can be plotted, in
which case the prediction and its confidence interval is displayed:

```{r}
plot(early.fit)
```

However, a better way to display these predictions is adding them to the
incidence plot using the argument `fit`:

```{r}
plot(i.7[1:20], fit = early.fit)
```


In this case, we would ideally like to fit two models, before and after the
peak of the epidemic. This is possible using the following approach, if you
know what date to use to split the data in two phases:

```{r, fit.both}
fit.both <- fit(i.7, split=as.Date("2014-10-15"))
fit.both
plot(i.7, fit=fit.both)
```

This is much better, but the splitting date is not completely optimal. To look
for the best possible splitting date (i.e. the one maximizing the average fit
of both models), we use:

```{r, optim}
best.fit <- fit_optim_split(i.7)
best.fit
plot(i.7, fit=best.fit$fit)
```

These models are very good approximation of these data, showing a doubling time
of `r round(get_info(best.fit$fit, "doubling"), 1)` days during the first
phase, and a halving time of `r round(get_info(best.fit$fit, "halving"), 1)`
days during the second. 
To access these parameters, you can use the `get_info()` function.

The possible parameters are:

 - "r", the daily growth rate
 - "doubling" the rate of doubling in days (if "r" is positive)
 - "halving" the rate of halving in days (if "r" is negative)
 - "pred" a data frame of incidence predictions
 
For "r", "doubling", and "halving", you can also add ".conf" to get the
confidence intervals. Here's how you can get the doubling and halving times of
the above epi curve:

```{r, get_info}
get_info(best.fit$fit, "doubling")      # doubling time
get_info(best.fit$fit, "doubling.conf") # confidence interval
get_info(best.fit$fit, "halving")       
get_info(best.fit$fit, "halving.conf")       
```

Note that `fit` will also take groups into account if incidence has been
computed for several groups:

```{r, optim2}
best.fit2 <- fit_optim_split(i.7.sex)
best.fit2
plot(i.7.sex, fit=best.fit2$fit)
```

Using `get_info()` on this fit object will return all groups together:

```{r, get_info_groups}
get_info(best.fit2$fit, "doubling")      # doubling time
get_info(best.fit2$fit, "doubling.conf") # confidence interval
get_info(best.fit2$fit, "halving")       
get_info(best.fit2$fit, "halving.conf")       
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit.R, R/fit_optim_split.R, R/print.R
\name{fit}
\alias{fit}
\alias{fit_optim_split}
\alias{print.incidence_fit}
\alias{print.incidence_fit_list}
\title{Fit exponential models to incidence data}
\usage{
fit(x, split = NULL, level = 0.95, quiet = FALSE)

fit_optim_split(
  x,
  window = x$timespan/4,
  plot = TRUE,
  quiet = TRUE,
  separate_split = TRUE
)

\method{print}{incidence_fit}(x, ...)

\method{print}{incidence_fit_list}(x, ...)
}
\arguments{
\item{x}{An incidence object, generated by the function
\code{\link[=incidence]{incidence()}}. For the plotting function, an \code{incidence_fit}
object.}

\item{split}{An optional time point identifying the separation between the
two models. If NULL, a single model is fitted. If provided, two models would
be fitted on the time periods on either side of the split.}

\item{level}{The confidence interval to be used for predictions; defaults to
95\%.}

\item{quiet}{A logical indicating if warnings from \code{fit} should be
hidden; FALSE by default. Warnings typically indicate some zero incidence,
which are removed before performing the log-linear regression.}

\item{window}{The size, in days, of the time window either side of the
split.}

\item{plot}{A logical indicating whether a plot should be added to the
output (\code{TRUE}, default), showing the mean R2 for various splits.}

\item{separate_split}{If groups are present, should separate split dates be
determined for each group? Defaults to \code{TRUE}, in which separate split dates
and thus, separate models will be constructed for each group. When \code{FALSE},
the split date will be determined from the pooled data and modelled with the
groups as main effects and interactions with date.}

\item{...}{currently unused.}
}
\value{
For \code{fit()}, a list with the class \code{incidence_fit} (for a
single model), or a list containing two \code{incidence_fit} objects (when
fitting two models). \code{incidence_fit} objects contain:
\itemize{
\item \verb{$model}: the fitted linear model
\item \verb{$info}: a list containing various information extracted from the model
(detailed further)
\item \verb{$origin}: the date corresponding to day '0'
}

The \verb{$info} item is a list containing:
\itemize{
\item \code{r}: the growth rate
\item \code{r.conf}: the confidence interval of 'r'
\item \code{pred}: a \code{data.frame} containing predictions of the model,
including the true dates (\code{dates}), their numeric version used in the
model (\code{dates.x}), the predicted value (\code{fit}), and the lower
(\code{lwr}) and upper (\code{upr}) bounds of the associated confidence
interval.
\item \code{doubling}: the predicted doubling time in days; exists only if 'r' is
positive
\item \code{doubling.conf}: the confidence interval of the doubling time
\item \code{halving}: the predicted halving time in days; exists only if 'r' is
negative
\item \code{halving.conf}: the confidence interval of the halving time
}

For \code{fit_optim_split}, a list containing:
\itemize{
\item \code{df}: a \code{data.frame} of dates that were used in the optimization
procedure, and the corresponding average R2 of the resulting models.
\item \code{split}: the optimal splitting date
\item \code{fit}: an \code{incidence_fit_list} object containing the fit for each split.
If the \code{separate_split = TRUE}, then the \code{incidence_fit_list} object will
contain these splits nested within each group. All of the \code{incidence_fit}
objects can be retrieved with \code{\link[=get_fit]{get_fit()}}.
\item \code{plot}: a plot showing the content of \code{df} (ggplot2 object)
}
}
\description{
The function \code{fit} fits two exponential models to incidence data, of the
form: \eqn{log(y) = r * t + b} \cr where 'y' is the incidence, 't' is time
(in days), 'r' is the growth rate, and 'b' is the origin. The function \code{fit}
will fit one model by default, but will fit two models on either side of a
splitting date (typically the peak of the epidemic) if the argument \code{split}
is provided. When groups are present, these are included in the model as main
effects and interactions with dates. The function \code{fit_optim_split()} can be
used to find the optimal 'splitting' date, defined as the one for which the
best average R2 of the two models is obtained. Plotting can be done using
\code{plot}, or added to an existing incidence plot by the piping-friendly
function \code{add_incidence_fit()}.
}
\examples{

if (require(outbreaks)) { withAutoprint({
 dat <- ebola_sim$linelist$date_of_onset

 ## EXAMPLE WITH A SINGLE MODEL

 ## compute weekly incidence
 i.7 <- incidence(dat, interval=7)
 plot(i.7)
 plot(i.7[1:20])

 ## fit a model on the first 20 weeks
 f <- fit(i.7[1:20])
 f
 names(f)
 head(get_info(f, "pred"))

 ## plot model alone (not recommended)
 plot(f)

 ## plot data and model (recommended)
 plot(i.7, fit = f)
 plot(i.7[1:25], fit = f)

## piping versions
if (require(magrittr)) { withAutoprint({
  plot(i.7) \%>\% add_incidence_fit(f)


  ## EXAMPLE WITH 2 PHASES
  ## specifying the peak manually
  f2 <- fit(i.7, split = as.Date("2014-10-15"))
  f2
  plot(i.7) \%>\% add_incidence_fit(f2)

  ## finding the best 'peak' date
  f3 <- fit_optim_split(i.7)
  f3
  plot(i.7) \%>\% add_incidence_fit(f3$fit)
})}

})}

}
\seealso{
the \code{\link[=incidence]{incidence()}} function to generate the 'incidence'
objects. The \code{\link[=get_fit]{get_fit()}} function to flatten \code{incidence_fit_list} objects to
a list of \code{incidence_fit} objects.
}
\author{
Thibaut Jombart \email{thibautjombart@gmail.com}, Zhian N. Kamvar
\email{zkamvar@gmail.com}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/find_peak.R
\name{find_peak}
\alias{find_peak}
\title{Find the peak date of an incidence curve}
\usage{
find_peak(x, pool = TRUE)
}
\arguments{
\item{x}{An \code{incidence} object.}

\item{pool}{If \code{TRUE} (default), any groups will be pooled before finding
a peak. If \code{FALSE}, separate peaks will be found for each group.}
}
\value{
The date of the (first) highest incidence in the data.
}
\description{
This function can be used to find the peak of an epidemic curve stored as an
\code{incidence} object.
}
\examples{

if (require(outbreaks) && require(ggplot2)) { withAutoprint({
  i <- incidence(fluH7N9_china_2013$date_of_onset)
  i
  plot(i)

  ## one simple bootstrap
  x <- bootstrap(i)
  x
  plot(x)

  ## find 95\% CI for peak time using bootstrap
  find_peak(i)


  ## show confidence interval
  plot(i) + geom_vline(xintercept = find_peak(i), col = "red", lty = 2)

})}

}
\seealso{
\code{\link[=estimate_peak]{estimate_peak()}} for bootstrap estimates of the peak time
}
\author{
Thibaut Jombart \email{thibautjombart@gmail.com}, Zhian N. Kamvar
\email{zkamvar@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conversion.R
\name{as.data.frame.incidence}
\alias{as.data.frame.incidence}
\alias{as.incidence}
\alias{as.incidence.matrix}
\alias{as.incidence.data.frame}
\alias{as.incidence.numeric}
\title{Conversion of incidence objects}
\usage{
\method{as.data.frame}{incidence}(x, ..., long = FALSE)

as.incidence(x, ...)

\method{as.incidence}{matrix}(
  x,
  dates = NULL,
  interval = NULL,
  standard = TRUE,
  isoweeks = standard,
  ...
)

\method{as.incidence}{data.frame}(x, dates = NULL, interval = NULL, isoweeks = TRUE, ...)

\method{as.incidence}{numeric}(x, dates = NULL, interval = NULL, isoweeks = TRUE, ...)
}
\arguments{
\item{x}{An \code{incidence} object, or an object to be converted as
\code{incidence} (see details).}

\item{...}{Further arguments passed to other functions (no used).}

\item{long}{A logical indicating if the output data.frame should be 'long', i.e. where a single
column containing 'groups' is added in case of data computed on several groups.}

\item{dates}{A vector of dates, each corresponding to the (inclusive) lower
limit of the bins.}

\item{interval}{An integer indicating the time interval used in the
computation of the incidence. If NULL, it will be determined from the first
time interval between provided dates. If only one date is provided, it will
trigger an error.}

\item{standard}{A logical indicating whether standardised dates should be
used. Defaults to \code{TRUE}.}

\item{isoweeks}{Deprecated. Use standard.}
}
\description{
These functions convert \code{incidence} objects into other classes.
}
\details{
Conversion to \code{incidence} objects should only be done when the
original dates are not available. In such case, the argument \code{x}
should be a matrix corresponding to the \verb{$counts} element of an
\code{incidence} object, i.e. giving counts with time intervals in rows and
named groups in columns. In the absence of groups, a single unnamed columns
should be given. \code{data.frame} and vectors will be coerced to a matrix.
}
\examples{
## create fake data
data <- c(0,1,1,2,1,3,4,5,5,5,5,4,4,26,6,7,9)
sex <- sample(c("m","f"), length(data), replace=TRUE)

## get incidence per group (sex)
i <- incidence(data, groups = sex)
i
plot(i)

## convert to data.frame
as.data.frame(i)

## same, 'long format'
as.data.frame(i, long = TRUE)



## conversion from a matrix of counts to an incidence object
i$counts
new_i <- as.incidence(i$counts, i$dates)
new_i
all.equal(i, new_i)

}
\seealso{
the \code{\link[=incidence]{incidence()}} function to generate the 'incidence' objects.
}
\author{
Thibaut Jombart \email{thibautjombart@gmail.com}, Rich Fitzjohn
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/incidence.R, R/print.R
\name{incidence}
\alias{incidence}
\alias{incidence.default}
\alias{incidence.Date}
\alias{incidence.character}
\alias{incidence.integer}
\alias{incidence.numeric}
\alias{incidence.POSIXt}
\alias{print.incidence}
\title{Compute incidence of events from a vector of dates.}
\usage{
incidence(dates, interval = 1L, ...)

\method{incidence}{default}(dates, interval = 1L, ...)

\method{incidence}{Date}(
  dates,
  interval = 1L,
  standard = TRUE,
  groups = NULL,
  na_as_group = TRUE,
  first_date = NULL,
  last_date = NULL,
  ...
)

\method{incidence}{character}(
  dates,
  interval = 1L,
  standard = TRUE,
  groups = NULL,
  na_as_group = TRUE,
  first_date = NULL,
  last_date = NULL,
  ...
)

\method{incidence}{integer}(
  dates,
  interval = 1L,
  groups = NULL,
  na_as_group = TRUE,
  first_date = NULL,
  last_date = NULL,
  ...
)

\method{incidence}{numeric}(
  dates,
  interval = 1L,
  groups = NULL,
  na_as_group = TRUE,
  first_date = NULL,
  last_date = NULL,
  ...
)

\method{incidence}{POSIXt}(
  dates,
  interval = 1L,
  standard = TRUE,
  groups = NULL,
  na_as_group = TRUE,
  first_date = NULL,
  last_date = NULL,
  ...
)

\method{print}{incidence}(x, ...)
}
\arguments{
\item{dates}{A vector of dates, which can be provided as objects of the
class: integer, numeric, Date, POSIXct, POSIXlt, and character. (See Note
about \code{numeric} and \code{character} formats)}

\item{interval}{An integer or character indicating the (fixed) size of the
time interval used for computing the incidence; defaults to 1 day. This can
also be a text string that corresponds to a valid date interval: day, week,
month, quarter, or year. (See Note).}

\item{...}{Additional arguments passed to other methods (none are used).}

\item{standard}{(Only applicable to Date objects) When \code{TRUE} (default) and the
\code{interval} one of "week", "month", "quarter", or "year", then this will
cause the bins for the counts to start at the beginning of the interval
(See Note).}

\item{groups}{An optional factor defining groups of observations for which
incidence should be computed separately.}

\item{na_as_group}{A logical value indicating if missing group (NA) should be
treated as a separate group.}

\item{first_date, last_date}{optional first/last dates to be used in the
epicurve. When these are \code{NULL} (default), the dates from the first/last
dates are taken from the observations. If these dates are provided, the
observations will be trimmed to the range of [first_date, last_date].}

\item{x}{An 'incidence' object.}
}
\value{
An list with the class \code{incidence}, which contains the
following items:
\itemize{
\item \strong{dates}: The dates marking the left side of the bins used for counting
events. When \code{standard = TRUE} and the interval represents weeks, months,
quarters, or years, the first date will represent the first standard date
(See Interval specification, below).
\item \strong{counts}: A matrix of incidence counts, which one column per group (and
a single column if no groups were used).
\item \strong{timespan}: The length of the period for which incidence is computed, in
days.
\item \strong{interval}: The bin size. If it's an integer, it represents the number
of days between each bin. It can also be a character, e.g. "2 weeks" or
"6 months".
\item \strong{n}: The total number of cases.
\item \strong{weeks}: Dates in week format (YYYY-Www), where YYYY corresponds to the
year of the given week and ww represents the numeric week of the year.
This will be a produced from the function \code{\link[aweek:date2week]{aweek::date2week()}}. Note that
these will have a special \code{"week_start"} attribute indicating which day of
the ISO week the week starts on (see Weeks, below).
\item \strong{isoweeks}: ISO 8601 week format YYYY-Www, which is returned only when
ISO week-based weekly incidence is computed.
}
}
\description{
This function computes incidence based on dates of events provided in
various formats. A fixed interval, provided as numbers of days, is used to
define time intervals. Counts within an interval always include the first
date, after which they are labeled, and exclude the second. For instance,
intervals labeled as 0, 3, 6, ... mean that the first bin includes days 0, 1
and 2, the second interval includes 3, 4 and 5 etc.
}
\details{
For details about the \verb{incidence class}, see the dedicated
vignette:\cr \code{vignette("incidence_class", package = "incidence")}
}
\note{
\subsection{Input data (\code{dates})}{
\itemize{
\item \strong{Decimal (numeric) dates}: will be truncated with a warning
\item \strong{Character dates} should be in the unambiguous \code{yyyy-mm-dd} (ISO 8601)
format. Any other format will trigger an error.
}
}

\subsection{Interval specification (\code{interval})}{
If \code{interval} is a valid character (e.g. "week" or "1 month"), then
the bin will start at the beginning of the interval just before the first
observation by default. For example, if the first case was recorded on
Wednesday, 2018-05-09:
\itemize{
\item "week"    : first day of the week (i.e. Monday, 2018-05-07) (defaults to ISO weeks, see "Week intervals", below)
\item "month"   : first day of the month (i.e. 2018-05-01)
\item "quarter" : first day of the quarter (i.e. 2018-04-01)
\item "year"    : first day of the calendar year (i.e. 2018-01-01)
}

These default intervals can be overridden with \code{standard = FALSE}, which
sets the interval to begin at the first observed case.
}

\subsection{Week intervals}{

As of \emph{incidence} version 1.7.0, it is possible to construct standardized
incidence objects standardized to any day of the week thanks to the
\code{\link[aweek:date2week]{aweek::date2week()}} function from the \pkg{aweek} package. The default
state is to use ISO 8601 definition of weeks, which start on Monday. You can
specify the day of the week an incidence object should be standardised to by
using the pattern "{n} {W} weeks" where "{W}" represents the weekday in an
English or current locale and "{n}" represents the duration, but this can be
ommitted.  Below are examples of specifying weeks starting on different days
assuming we had data that started on 2016-09-05, which is ISO week 36 of
2016:
\itemize{
\item interval = "2 monday weeks" (Monday 2016-09-05)
\item interval = "1 tue week" (Tuesday 2016-08-30)
\item interval = "1 Wed week" (Wednesday 2016-08-31)
\item interval = "1 Thursday week" (Thursday 2016-09-01)
\item interval = "1 F week" (Friday 2016-09-02)
\item interval = "1 Saturday week" (Saturday 2016-09-03)
\item interval = "Sunday week" (Sunday 2016-09-04)
}

It's also possible to use something like "3 weeks: Saturday"; In addition,
there are keywords reserved for specific days of the week:
\itemize{
\item interval = "week", standard = TRUE (Default, Monday)
\item interval = "ISOweek"  (Monday)
\item interval = "EPIweek"  (Sunday)
\item interval = "MMWRweek" (Sunday)
}

The "EPIweek" specification is not strictly reserved for CDC epiweeks, but
can be prefixed (or posfixed) by a day of the week: "1 epiweek: Saturday".

}

\subsection{The \code{first_date} argument}{
Previous versions of \emph{incidence} had the \code{first_date} argument override
\code{standard = TRUE}. It has been changed as of \emph{incidence} version 1.6.0 to
be more consistent with the behavior when \code{first_date = NULL}. This, however
may be a change in behaviour, so a warning is now issued once and only once
if \code{first_date} is specified, but \code{standard} is not. To never see this
warning, use \code{options(incidence.warn.first_date = FALSE)}.
}

The intervals for "month", "quarter", and "year" will necessarily vary in the
number of days they encompass and warnings will be generated when the first
date falls outside of a calendar date that is easily represented across the
interval.
}
\examples{
## toy example
incidence(c(1, 5, 8, 3, 7, 2, 4, 6, 9, 2))
incidence(c(1, 5, 8, 3, 7, 2, 4, 6, 9, 2), 2)

## example using simulated dataset
if(require(outbreaks)) { withAutoprint({
  onset <- outbreaks::ebola_sim$linelist$date_of_onset

  ## daily incidence
  inc <- incidence(onset)
  inc
  plot(inc)

  ## weekly incidence
  inc.week <- incidence(onset, interval = 7, standard = FALSE)
  inc.week
  plot(inc.week)
  plot(inc.week, border = "white") # with visible border
  
  # Starting on Monday
  inc.isoweek <- incidence(onset, interval = "isoweek")
  inc.isoweek
  
  # Starting on Sunday
  inc.epiweek <- incidence(onset, interval = "epiweek")
  inc.epiweek

  # Starting on Saturday
  inc.epiweek <- incidence(onset, interval = "saturday epiweek")
  inc.epiweek

  ## use group information
  sex <- outbreaks::ebola_sim$linelist$gender
  inc.week.gender <- incidence(onset, interval = 7,
                               groups = sex, standard = FALSE)
  inc.week.gender
  head(inc.week.gender$counts)
  plot(inc.week.gender, border = "grey90")
  inc.satweek.gender <- incidence(onset, interval = "2 epiweeks: saturday",
                                  groups = sex)
  inc.satweek.gender
  plot(inc.satweek.gender, border = "grey90")

})}

# Use of first_date
d <- Sys.Date() + sample(-3:10, 10, replace = TRUE)

# `standard` specified, no warning
di <- incidence(d, interval = "week", first_date = Sys.Date() - 10, standard = TRUE)

# warning issued if `standard` not specified
di <- incidence(d, interval = "week", first_date = Sys.Date() - 10)

# second instance: no warning issued
di <- incidence(d, interval = "week", first_date = Sys.Date() - 10)


}
\seealso{
The main other functions of the package include:
\itemize{
\item \code{\link[=plot.incidence]{plot.incidence()}}: Plot epicurves from an incidence object.
\item \code{\link[=fit]{fit()}}: Fit log-linear model to computed incidence.
\item \code{\link[=fit_optim_split]{fit_optim_split()}}: Find the optimal peak of the epidemic
and fits log-linear models on either side of the peak.
\item \code{\link[=subset]{subset()}}: Handling of \code{incidence}
objects.
\item \code{\link[=pool]{pool()}}: Sum incidence over groups.
\item \code{\link[=as.data.frame.incidence]{as.data.frame.incidence()}}: Convert an \code{incidence} object to a
\code{data.frame}.
}

The following vignettes are also available:
\itemize{
\item \code{overview}: Provides an overview of the package's features.
\item \code{customize_plot}: Provides some tips on finer plot customization.
\item \code{incidence_class}: Details the content of the \code{incidence}
class.
}
}
\author{
Thibaut Jombart, Rich Fitzjohn, Zhian Kamvar
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R, R/scale_x_incidence.R
\name{plot.incidence}
\alias{plot.incidence}
\alias{add_incidence_fit}
\alias{plot.incidence_fit}
\alias{plot.incidence_fit_list}
\alias{scale_x_incidence}
\alias{make_breaks}
\title{Plot function for incidence objects}
\usage{
\method{plot}{incidence}(
  x,
  ...,
  fit = NULL,
  stack = is.null(fit),
  color = "black",
  border = NA,
  col_pal = incidence_pal1,
  alpha = 0.7,
  xlab = "",
  ylab = NULL,
  labels_week = !is.null(x$weeks),
  labels_iso = !is.null(x$isoweeks),
  show_cases = FALSE,
  n_breaks = 6
)

add_incidence_fit(p, x, col_pal = incidence_pal1)

\method{plot}{incidence_fit}(x, ...)

\method{plot}{incidence_fit_list}(x, ...)

scale_x_incidence(x, n_breaks = 6, labels_week = TRUE, ...)

make_breaks(x, n_breaks = 6L, labels_week = TRUE)
}
\arguments{
\item{x}{An incidence object, generated by the function
\code{\link[=incidence]{incidence()}}.}

\item{...}{arguments passed to \code{\link[ggplot2:scale_date]{ggplot2::scale_x_date()}},
\code{\link[ggplot2:scale_date]{ggplot2::scale_x_datetime()}}, or \code{\link[ggplot2:scale_continuous]{ggplot2::scale_x_continuous()}}, depending
on how the \verb{$date} element is stored in the incidence object.}

\item{fit}{An 'incidence_fit' object as returned by \code{\link[=fit]{fit()}}.}

\item{stack}{A logical indicating if bars of multiple groups should be
stacked, or displayed side-by-side.}

\item{color}{The color to be used for the filling of the bars; NA for
invisible bars; defaults to "black".}

\item{border}{The color to be used for the borders of the bars; NA for
invisible borders; defaults to NA.}

\item{col_pal}{The color palette to be used for the groups; defaults to
\code{incidence_pal1}. See \code{\link[=incidence_pal1]{incidence_pal1()}} for other palettes implemented in
incidence.}

\item{alpha}{The alpha level for color transparency, with 1 being fully
opaque and 0 fully transparent; defaults to 0.7.}

\item{xlab}{The label to be used for the x-axis; empty by default.}

\item{ylab}{The label to be used for the y-axis; by default, a label will be
generated automatically according to the time interval used in incidence
computation.}

\item{labels_week}{a logical value indicating whether labels x axis tick
marks are in week format YYYY-Www when plotting weekly incidence; defaults to
TRUE.}

\item{labels_iso}{(deprecated) This has been superceded by \code{labels_iso}.
Previously:a logical value indicating whether labels x axis tick marks are
in ISO 8601 week format yyyy-Www when plotting ISO week-based weekly
incidence; defaults to be TRUE.}

\item{show_cases}{if \code{TRUE} (default: \code{FALSE}), then each observation will be
colored by a border. The border defaults to a white border unless specified
otherwise. This is normally used outbreaks with a small number of cases.
Note: this can only be used if \code{stack = TRUE}}

\item{n_breaks}{the ideal number of breaks to be used for the x-axis
labeling}

\item{p}{An existing incidence plot.}
}
\value{
\itemize{
\item \code{plot()} a \code{\link[ggplot2:ggplot]{ggplot2::ggplot()}} object.
\item \code{make_breaks()} a two-element list. The "breaks" element will contain the
evenly-spaced breaks as either dates or numbers and the "labels" element
will contain either a vector of weeks OR a \code{\link[ggplot2:waiver]{ggplot2::waiver()}} object.
\item \code{scale_x_incidence()} a \pkg{ggplot2} "ScaleContinuous" object.
}
}
\description{
This function is used to visualise the output of the \code{\link[=incidence]{incidence()}}
function using the package \code{ggplot2}. #'
}
\details{
\itemize{
\item \code{plot()} will visualise an incidence object using \code{ggplot2}
\item \code{make_breaks()} calculates breaks from an incidence object that always
align with the bins and start on the first observed incidence.
\item \code{scale_x_incidence()} produces and appropriate \code{ggplot2} scale based on
an incidence object.
}
}
\examples{

if(require(outbreaks) && require(ggplot2)) { withAutoprint({
  onset <- outbreaks::ebola_sim$linelist$date_of_onset

  ## daily incidence
  inc <- incidence(onset)
  inc
  plot(inc)

  ## weekly incidence
  inc.week <- incidence(onset, interval = 7)
  inc.week
  plot(inc.week) # default to label x axis tick marks with isoweeks
  plot(inc.week, labels_week = FALSE) # label x axis tick marks with dates
  plot(inc.week, border = "white") # with visible border

  ## use group information
  sex <- outbreaks::ebola_sim$linelist$gender
  inc.week.gender <- incidence(onset, interval = "1 epiweek", groups = sex)
  plot(inc.week.gender)
  plot(inc.week.gender, labels_week = FALSE)

  ## show individual cases at the beginning of the epidemic
  inc.week.8 <- subset(inc.week.gender, to = "2014-06-01")
  p <- plot(inc.week.8, show_cases = TRUE, border = "black")
  p

  ## update the range of the scale
  lim <- c(min(get_dates(inc.week.8)) - 7*5, 
           aweek::week2date("2014-W50", "Sunday"))
  lim
  p + scale_x_incidence(inc.week.gender, limits = lim)

  ## customize plot with ggplot2
  plot(inc.week.8, show_cases = TRUE, border = "black") +
    theme_classic(base_size = 16) +
    theme(axis.text.x = element_text(angle = 90, hjust = 1, vjust = 0.5)) 

  ## adding fit
  fit <- fit_optim_split(inc.week.gender)$fit
  plot(inc.week.gender, fit = fit)
  plot(inc.week.gender, fit = fit, labels_week = FALSE)

})}

}
\seealso{
The \code{\link[=incidence]{incidence()}} function to generate the 'incidence'
objects.
}
\author{
Thibaut Jombart \email{thibautjombart@gmail.com}
Zhian N. Kamvar \email{zkamvar@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bootstrap.R
\name{bootstrap}
\alias{bootstrap}
\title{Bootstrap incidence time series}
\usage{
bootstrap(x, randomise_groups = FALSE)
}
\arguments{
\item{x}{An \code{incidence} object.}

\item{randomise_groups}{A \code{logical} indicating whether groups should be
randomised as well in the resampling procedure; respective group sizes will
be preserved, but this can be used to remove any group-specific temporal
dynamics. If \code{FALSE} (default), data are resampled within groups.}
}
\value{
An \code{incidence} object.
}
\description{
This function can be used to bootstrap \code{incidence} objects. Bootstrapping is
done by sampling with replacement the original input dates. See \code{details} for
more information on how this is implemented.
}
\details{
As original data are not stored in \code{incidence} objects, the
bootstrapping is achieved by multinomial sampling of date bins weighted by
their relative incidence.
}
\examples{

if (require(outbreaks) && require(ggplot2)) { withAutoprint({
  i <- incidence(fluH7N9_china_2013$date_of_onset)
  i
  plot(i)

  ## one simple bootstrap
  x <- bootstrap(i)
  x
  plot(x)

})}

}
\seealso{
\link{find_peak} to use estimate peak date using bootstrap
}
\author{
Thibaut Jombart \email{thibautjombart@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/subset.R
\name{subset.incidence}
\alias{subset.incidence}
\alias{"subset.incidence"}
\alias{"[.incidence"}
\alias{[.incidence}
\title{Subsetting 'incidence' objects}
\usage{
\method{subset}{incidence}(x, ..., from = min(x$dates), to = max(x$dates), groups = TRUE)

\method{[}{incidence}(x, i, j)
}
\arguments{
\item{x}{An incidence object, generated by the function
\code{\link[=incidence]{incidence()}}.}

\item{...}{Further arguments passed to other methods (not used).}

\item{from}{The starting date; data strictly before this date are discarded.}

\item{to}{The ending date; data strictly after this date are discarded.}

\item{groups}{(optional) The groups to retained, indicated as subsets of the
columns of x$counts.}

\item{i}{a subset of dates to retain}

\item{j}{a subset of groups to retain}
}
\description{
Two functions can be used to subset incidence objects.  The function
\code{subset} permits to retain dates within a specified range and,
optionally, specific groups. The operator "[" can be used as for matrices,
using the syntax \code{x[i,j]} where 'i' is a subset of dates, and 'j' is a
subset of groups.
}
\examples{
## example using simulated dataset
if(require(outbreaks)) { withAutoprint({
  onset <- ebola_sim$linelist$date_of_onset

  ## weekly incidence
  inc <- incidence(onset, interval = 7)
  inc
  inc[1:10] # first 10 weeks
  plot(inc[1:10])
  inc[-c(11:15)] # remove weeks 11-15
  plot(inc[-c(11:15)])
})}

}
\seealso{
The \code{\link[=incidence]{incidence()}} function to generate the 'incidence'
objects.
}
\author{
Thibaut Jombart \email{thibautjombart@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cumulate.R
\name{cumulate}
\alias{cumulate}
\alias{cumulate.default}
\alias{cumulate.incidence}
\title{Compute cumulative 'incidence'}
\usage{
cumulate(x)

\method{cumulate}{default}(x)

\method{cumulate}{incidence}(x)
}
\arguments{
\item{x}{An incidence object.}
}
\description{
\code{cumulate} is an S3 generic to compute cumulative numbers, with methods
for different types of objects:
}
\details{
\itemize{

\item default method is a wrapper for \code{cumsum}

\item \code{incidence} objects: computes cumulative incidence over time

\item \code{projections} objects: same, for \code{projections} objects,
implemented in the similarly named package; see \code{?cumulate.projections}
for more information, after loading the package

}
}
\examples{
dat <- as.integer(c(0,1,2,2,3,5,7))
group <- factor(c(1, 2, 3, 3, 3, 3, 1))
i <- incidence(dat, groups = group)
i
plot(i)

i_cum <- cumulate(i)
i_cum
plot(i_cum)

}
\seealso{
The \code{\link[=incidence]{incidence()}} function to generate the 'incidence'
objects.
}
\author{
Thibaut Jombart \email{thibautjombart@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/estimate_peak.R
\name{estimate_peak}
\alias{estimate_peak}
\title{Estimate the peak date of an incidence curve using bootstrap}
\usage{
estimate_peak(x, n = 100, alpha = 0.05)
}
\arguments{
\item{x}{An \code{incidence} object.}

\item{n}{The number of bootstrap datasets to be generated; defaults to 100.}

\item{alpha}{The type 1 error chosen for the confidence interval; defaults to
0.05.}
}
\value{
A list containing the following items:
\itemize{
\item \code{observed}: the peak incidence of the original dataset
\item \code{estimated}: the mean peak time of the bootstrap datasets
\item \code{ci}: the confidence interval based on bootstrap datasets
\item \code{peaks}: the peak times of the bootstrap datasets
}
}
\description{
This function can be used to estimate the peak of an epidemic curve stored as
\code{incidence}, using bootstrap. See \link{bootstrap} for more information
on the resampling.
}
\details{
Input dates are resampled with replacement to form bootstrapped
datasets; the peak is reported for each, resulting in a distribution of
peak times. When there are ties for peak incidence, only the first date is
reported.

Note that the bootstrapping approach used for estimating the peak time makes
the following assumptions:
\itemize{
\item the total number of event is known (no uncertainty on total incidence)
\item dates with no events (zero incidence) will never be in bootstrapped datasets
\item the reporting is assumed to be constant over time, i.e. every case is
equally likely to be reported
}
}
\examples{

if (require(outbreaks) && require(ggplot2)) { withAutoprint({
  i <- incidence(fluH7N9_china_2013$date_of_onset)
  i
  plot(i)

  ## one simple bootstrap
  x <- bootstrap(i)
  x
  plot(x)

  ## find 95\% CI for peak time using bootstrap
  peak_data <- estimate_peak(i)
  peak_data
  summary(peak_data$peaks)

  ## show confidence interval
  plot(i) + geom_vline(xintercept = peak_data$ci, col = "red", lty = 2)

  ## show the distribution of bootstrapped peaks
  df <- data.frame(peak = peak_data$peaks)
  plot(i) + geom_density(data = df,
                         aes(x = peak, y = 10 * ..scaled..),
                         alpha = .2, fill = "red", color = "red")

})}

}
\seealso{
\link{bootstrap} for the bootstrapping underlying this
approach and \link{find_peak} to find the peak in a single
\code{incidence} object.
}
\author{
Thibaut Jombart \email{thibautjombart@gmail.com}, with inputs on
caveats from Michael Höhle.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_dates.R
\name{get_dates}
\alias{get_dates}
\alias{get_dates.default}
\alias{get_dates.incidence}
\title{Retrieve dates from an incidence object}
\usage{
get_dates(x, ...)

\method{get_dates}{default}(x, ...)

\method{get_dates}{incidence}(x, position = "left", count_days = FALSE, ...)
}
\arguments{
\item{x}{an \link{incidence} object}

\item{...}{Unused}

\item{position}{One of "left", "center", "middle", or "right" specifying what
side of the bin the date should be drawn from.}

\item{count_days}{If \code{TRUE}, the result will be represented as the number of
days from the first date.}
}
\value{
a vector of dates or numerics
}
\description{
Retrieve dates from an incidence object
}
\examples{

set.seed(999)
dat <- as.Date(Sys.Date()) + sample(-3:50, 100, replace = TRUE)
x <- incidence(dat, interval = "month")
get_dates(x)
get_dates(x, position = "middle")
set.seed(999)
dat <- as.Date(Sys.Date()) + sample(-3:50, 100, replace = TRUE)
x <- incidence(dat, interval = "month")
get_dates(x)
get_dates(x, "center")
get_dates(x, "right")

# Return dates by number of days from the first date
get_dates(x, count_days = TRUE)
get_dates(incidence(-5:5), count_days = TRUE)
}
\keyword{accessors}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dim.R, R/get_interval.R, R/get_n.R,
%   R/get_timespan.R
\name{dim.incidence}
\alias{dim.incidence}
\alias{get_interval}
\alias{get_interval.default}
\alias{get_interval.incidence}
\alias{get_n}
\alias{get_n.default}
\alias{get_n.incidence}
\alias{get_timespan}
\alias{get_timespan.default}
\alias{get_timespan.incidence}
\title{Access various elements of an incidence object}
\usage{
\method{dim}{incidence}(x)

get_interval(x, ...)

\method{get_interval}{default}(x, ...)

\method{get_interval}{incidence}(x, integer = TRUE, ...)

get_n(x)

\method{get_n}{default}(x)

\method{get_n}{incidence}(x)

get_timespan(x)

\method{get_timespan}{default}(x)

\method{get_timespan}{incidence}(x)
}
\arguments{
\item{x}{an \link{incidence} object.}

\item{...}{Unused}

\item{integer}{When \code{TRUE} (default), the interval will be converted to an
integer vector if it is stored as a character in the incidence object.}
}
\value{
\itemize{
\item \code{dim()} the dimensions in the number of bins and number of groups
}

\itemize{
\item \code{get_interval()} if \code{integer = TRUE}: an integer vector, otherwise: the
value stored in \code{x$interval}
}

\itemize{
\item \code{get_n()} The total number of cases stored in the object
}

\itemize{
\item \code{get_timespan()}: an \code{integer} denoting the timespan represented by the
incidence object.
}
}
\description{
Access various elements of an incidence object
}
\examples{

set.seed(999)
dat <- as.Date(Sys.Date()) + sample(-3:50, 100, replace = TRUE)
x <- incidence(dat, interval = "month")

# the value stored in the interval element
get_interval(x)

# the numeric value of the interval in days
get_interval(x, integer = FALSE)

# the number of observations in the object
get_n(x)

# the length of time represented
get_timespan(x)

# the number of groups
ncol(x)

# the number of bins (intervals)
nrow(x)
}
\seealso{
\itemize{
\item \code{\link[=get_counts]{get_counts()}} to access the matrix of counts
\item \code{\link[=get_dates]{get_dates()}} to access the dates on the right, left, and center of the
interval.
\item \code{\link[=group_names]{group_names()}} to access and possibly re-name the groups
}
}
\keyword{accessors}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_counts.R
\name{get_counts}
\alias{get_counts}
\alias{get_counts.incidence}
\title{Get counts from an incidence object}
\usage{
get_counts(x, groups = NULL)

\method{get_counts}{incidence}(x, groups = NULL)
}
\arguments{
\item{x}{an \code{incidence} object.}

\item{groups}{if there are groups, use this to specify a group or groups to
subset. Defaults to \code{NULL} indicating that all groups are returned.}
}
\value{
a matrix of counts where each row represents a date bin
}
\description{
Get counts from an incidence object
}
\examples{
if (require(outbreaks)) { withAutoprint({
  dat  <- ebola_sim$linelist$date_of_onset
  gend <- ebola_sim$linelist$gender
  i    <- incidence(dat, interval = "week", groups = gend)
  
  ## Use with an object and no arguments gives the counts matrix
  head(get_counts(i))

  ## Specifying a position or group name will return a matrix subset to that
  ## group
  head(get_counts(i, 1L))
  head(get_counts(i, "f"))

  ## Specifying multiple groups allows you to rearrange columns
  head(get_counts(i, c("m", "f")))

  ## If you want a vector, you can use drop
  drop(get_counts(i, "f"))
})}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_fit.R, R/get_info.R
\name{get_fit}
\alias{get_fit}
\alias{get_fit.incidence_fit}
\alias{get_fit.incidence_fit_list}
\alias{get_info}
\alias{get_info.incidence_fit}
\alias{get_info.incidence_fit_list}
\title{Accessors for \code{incidence_fit} objects}
\usage{
get_fit(x)

\method{get_fit}{incidence_fit}(x)

\method{get_fit}{incidence_fit_list}(x)

get_info(x, what = "r", ...)

\method{get_info}{incidence_fit}(x, what = "r", ...)

\method{get_info}{incidence_fit_list}(x, what = "r", groups = NULL, na.rm = TRUE, ...)
}
\arguments{
\item{x}{an \code{incidence_fit} or \code{incidence_fit_list}
object.}

\item{what}{the name of the item in the "info" element of the \code{incidence_fit}
object.}

\item{...}{currently unused.}

\item{groups}{if \code{what = "pred"} and \code{x} is an \code{incidence_fit_list} object,
then this indicates what part of the nesting hierarchy becomes the column
named "groups". Defaults to \code{NULL}, indicating that no groups column will
be added/modified.}

\item{na.rm}{when \code{TRUE} (default), missing values will be excluded from the
results.}
}
\value{
a list of \code{incidence_fit} objects.
}
\description{
Accessors for \code{incidence_fit} objects
}
\examples{

if (require(outbreaks)) { withAutoprint({

 dat <- ebola_sim$linelist$date_of_onset

 ## EXAMPLE WITH A SINGLE MODEL

 ## compute weekly incidence
 sex <- ebola_sim$linelist$gender
 i.sex <- incidence(dat, interval = 7, group = sex)
 
 ## Compute the optimal split for each group separately
 fits  <- fit_optim_split(i.sex, separate_split = TRUE)

 ## `fits` contains an `incidence_fit_list` object
 fits$fit
 
 ## Grab the list of `incidence_fit` objects
 get_fit(fits$fit)
 
 ## Get the predictions for all groups
 get_info(fits$fit, "pred", groups = 1)
 
 ## Get the predictions, but set `groups` to "before" and "after"
 get_info(fits$fit, "pred", groups = 2)
 
 ## Get the reproduction number
 get_info(fits$fit, "r")

 ## Get the doubling confidence interval
 get_info(fits$fit, "doubling.conf")

 ## Get the halving confidence interval
 get_info(fits$fit, "halving.conf")
})}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/group_names.R
\name{group_names}
\alias{group_names}
\alias{group_names<-}
\alias{group_names.default}
\alias{group_names<-.default}
\alias{`group_names<-`.default}
\alias{group_names.incidence}
\alias{group_names<-.incidence}
\title{extract and set group names}
\usage{
group_names(x, value)

group_names(x) <- value

\method{group_names}{default}(x, value)

\method{group_names}{default}(x) <- value

\method{group_names}{incidence}(x, value = NULL)

\method{group_names}{incidence}(x) <- value
}
\arguments{
\item{x}{an \code{\link[=incidence]{incidence()}} object.}

\item{value}{character vector used to rename groups}
}
\value{
an integer indicating the number of groups present in the incidence
object.
}
\description{
extract and set group names
}
\details{
This accessor will return a
}
\examples{
i <- incidence(dates = sample(10, 100, replace = TRUE), 
               interval = 1L,
               groups = sample(letters[1:3], 100, replace = TRUE))
i
group_names(i)

# change the names of the groups
group_names(i) <- c("Group 1", "Group 2", "Group 3")
i

# example if there are mistakes in the original data, e.g. 
# something is misspelled
set.seed(50)
grps <- sample(c("child", "adult", "adlut"), 100, replace = TRUE, prob = c(0.45, 0.45, 0.05))
i <- incidence(dates = sample(10, 100, replace = TRUE), 
               interval = 1L,
               groups = grps)
colSums(get_counts(i))

# If you change the name of the mis-spelled group, it will be merged with the
# correctly-spelled group
gname <- group_names(i)
gname[gname == "adlut"] <- "adult"
# without side-effects
print(ii <- group_names(i, gname))
colSums(get_counts(i))  # original still has three groups
colSums(get_counts(ii))
# with side-effects
group_names(i) <- gname
colSums(get_counts(i))
}
\keyword{accessors}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pool.R
\name{pool}
\alias{pool}
\title{Pool 'incidence' across groups}
\usage{
pool(x)
}
\arguments{
\item{x}{An 'incidence' object.}
}
\description{
This function pools incidence across all groups of an \code{incidence}
object. The resulting \code{\link[=incidence]{incidence()}} object will contains counts
summed over all groups present in the input.
}
\examples{
dat <- as.integer(c(0,1,2,2,3,5,7))
group <- factor(c(1, 2, 3, 3, 3, 3, 1))
i <- incidence(dat, groups = group)
i
i$counts

## pool all groups
pool(i)
pool(i)$counts

## pool only groups 1 and 3
pool(i[,c(1,3)])
pool(i[,c(1,3)])$counts

}
\seealso{
The \code{\link[=incidence]{incidence()}} function to generate the 'incidence'
objects.
}
\author{
Thibaut Jombart \email{thibautjombart@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/palettes.R
\name{incidence_pal1}
\alias{incidence_pal1}
\alias{palettes}
\alias{incidence_pal1_light}
\alias{incidence_pal1_dark}
\title{Color palettes used in incidence}
\usage{
incidence_pal1(n)

incidence_pal1_light(n)

incidence_pal1_dark(n)
}
\arguments{
\item{n}{a number of colors}
}
\description{
These functions are color palettes used in incidence.
}
\examples{

plot(1:4, cex=8, pch=20, col = incidence_pal1(4),
     main = "palette: incidence_pal1")
plot(1:100, cex=8, pch=20, col = incidence_pal1(100),
     main ="palette: incidence_pal1")
plot(1:100, cex=8, pch=20, col = incidence_pal1_light(100),
     main="palette: incidence_pal1_light")
plot(1:100, cex=8, pch=20, col = incidence_pal1_dark(100),
     main="palette: incidence_pal1_dark")

}
\author{
Thibaut Jombart \email{thibautjombart@gmail.com}
}
